# FLASH makefile definitions for ix86-64 Linux (Absoft compiler)

#----------------------------------------------------------------------------
# Set the HDF/HDF5 library paths -- these need to be updated for your system
#----------------------------------------------------------------------------
MPI_PATH   =  /opt/mpich2/current
HDF4_PATH =
HDF5_PATH = /opt/hdf5/current

HYPRE_PATH = /opt/hypre/2.8.0b
ifeq ("$(USEOPENMP)", "1")
HYPRE_PATH=/opt/hypre/2.8.0b_omp
endif

ZLIB_PATH  =

PAPI_PATH  =
PAPI_FLAGS =

NCMPI_PATH = /opt/pnetcdf/current
MPE_PATH   =

#----------------------------------------------------------------------------
# Compiler and linker commands
#
#   Use the MPICH wrappers around the compilers -- these will automatically
#   load the proper libraries and include files.  Version of MPICH prior
#   to 1.2.2 (?) do not recognize .F90 as a valid Fortran file extension.
#   You need to edit mpif90 and add .F90 to the test of filename extensions,
#   or upgrade your MPICH.
#----------------------------------------------------------------------------

FCOMP   = ${MPI_PATH}/bin/mpif90
CCOMP   = ${MPI_PATH}/bin/mpicc
CPPCOMP = ${MPI_PATH}/bin/mpiCC
LINK    = ${MPI_PATH}/bin/mpif90

# pre-processor flag
PP      = -D

#----------------------------------------------------------------------------
# Compilation flags
#
#  Three sets of compilation/linking flags are defined: one for optimized
#  code, one for testing, and one for debugging.  The default is to use the
#  _OPT version.  Specifying -debug to setup will pick the _DEBUG version,
#  these should enable bounds checking.  Specifying _TEST is used for
#  flash_test, and is set for quick code generation, and (sometimes)
#  profiling.  The Makefile generated by setup will assign the generic token
#  (ex. FFLAGS) to the proper set of flags (ex. FFLAGS_OPT).
#----------------------------------------------------------------------------

#Absoft 64-bit Pro Fortran 11.1.3 for Linux x86_64 is not OpenMP standard
#conforming because it does not provide the macro _OPENMP.   I manually
#define it because I have some code that depends on _OPENMP.  The release
#notes say the compiler provides OpenMP 3.0 Support which was released on 
#May 2008 and so I make the yyyymm date string 200805.
OPENMP_FORTRAN = -openmp -D_OPENMP=200805
OPENMP_C = -fopenmp
OPENMP_LINK = -openmp

FFLAGS_OPT = -c -N113 -m64 -safefp
FFLAGS_DEBUG = -g -c -N113 -m64 -TENV:simd_imask=off,simd_zmask=off,simd_omask=off -Rc -Rb -Rs -safefp
#FFLAGS_DEBUG = -g -c -N113 -m64 -Rb -Rc -Rs -safefp
FFLAGS_TEST = -g -c -N113 -safefp -m64
FFLAGS_HYPRE = -I${HYPRE_PATH}/include
CFLAGS_HYPRE = -I${HYPRE_PATH}/include


F90FLAGS =



CFLAGS_OPT = -ggdb -c -O2 -Wuninitialized -D_FORTIFY_SOURCE=2

CFLAGS_DEBUG = -ggdb -c -Wno-div-by-zero -Wundef \
-Wconversion -Wstrict-prototypes -Wunreachable-code \
-pedantic -Wall -Wextra -Winit-self -ftree-vrp -Wfloat-equal \
-Wunsafe-loop-optimizations -Wpadded -fstack-protector-all

CFLAGS_TEST = -ggdb -c


# if we are using HDF5, we need to specify the path to the include files
CFLAGS_HDF5 = -I ${HDF5_PATH}/include -DH5_USE_16_API
CFLAGS_NCMPI = -I ${NCMPI_PATH}/include


#----------------------------------------------------------------------------
# Linker flags
#
#  There is a seperate version of the linker flags for each of the _OPT,
#  _DEBUG, and _TEST cases.
#----------------------------------------------------------------------------

LFLAGS_OPT   = -ggdb -lU77 -lV77 -o
LFLAGS_DEBUG = -ggdb -lU77 -lV77 -o
LFLAGS_TEST  = -ggdb -lU77 -lV77 -o


#----------------------------------------------------------------------------
# Library specific linking
#
#  If a FLASH module has a 'LIBRARY xxx' line in its Config file, we need to
#  create a macro in this Makefile.h for LIB_xxx, which will be added to the
#  link line when FLASH is built.  This allows us to switch between different
#  (incompatible) libraries.  We also create a _OPT, _DEBUG, and _TEST
#  library macro to add any performance-minded libraries (like fast math),
#  depending on how FLASH was setup.
#----------------------------------------------------------------------------

LIB_OPT   =
LIB_DEBUG =
LIB_TEST  =

LIB_HDF4  = 
LIB_HDF5  = -L ${HDF5_PATH}/lib -lhdf5 -lz

LIB_PAPI  =
LIB_MATH  = 

LIB_MPI   = 
LIB_NCMPI = -L $(NCMPI_PATH)/lib -lpnetcdf
LIB_MPE   =
LIB_HYPRE = -L${HYPRE_PATH}/lib -lHYPRE

LIB_LIBNBC = -L/opt/libNBC/1.1.1/lib -lnbc



# mod files generated by this compiler uses uppercase BASENAMES
MODUPPERCASE = 1




#----------------------------------------------------------------------------
# Additional machine-dependent object files
#
#  Add any machine specific files here -- they will be compiled and linked
#  when FLASH is built.
#----------------------------------------------------------------------------

MACHOBJ =


#----------------------------------------------------------------------------
# Additional commands
#----------------------------------------------------------------------------

MV = mv -f
AR = ar -r
RM = rm -f
CD = cd
RL = ranlib
ECHO = echo


# these particular files break the absoft optimizer... maybe they can be
# tweaked to not break it

ifeq ($(FLASHBINARY),true)
gr_markRefineDerefine.o : %.o : %.F90 PARAMESH_INTERFACES.mod TREE.mod PHYSICALDATA.mod PARAMESH_DIMENSIONS.mod GRID_DATA.mod constants.h Flash.h Flash_mpi.h
	$(FCOMP) $(FFLAGS_TEST) -v -m0 -g -O0 -Rc -Rs -Rp -YNDFP=1 -en -et -OPT:roundoff=0 $(F90FLAGS) $(FDEFINES) $<
endif

