# FLASH makefile definitions for ASCI Blue Pacific machine (IBM)

#----------------------------------------------------------------------------
# Set the HDF/HDF5 and PAPI library paths 
# -- these need to be updated for your system
# If PAPI doesn't exist on your system, comment them out
#----------------------------------------------------------------------------

HDF5_PATH  = 
HDF4_PATH  =
ZLIB_PATH  =

NCMPI_PATH = /bgl/BlueLight/ppcfloor/bglsys
MPE_PATH   =

#----------------------------------------------------------------------------
# Compiler and linker commands
#
#   Use the MPICH wrappers around the compilers -- these will automatically
#   load the proper libraries and include files.  Version of MPICH prior
#   to 1.2.2 (?) do not recognize .F90 as a valid Fortran file extension.
#   You need to edit mpif90 and add .F90 to the test of filename extensions,
#   or upgrade your MPICH.
#----------------------------------------------------------------------------
FCOMP   = /opt/ibmcmp/xlf/9.1/bin/blrts_xlf
CCOMP   = /opt/ibmcmp/vac/7.0/bin/blrts_xlc
CPPCOMP = cpp 	
LINK    = /opt/ibmcmp/xlf/9.1/bin/blrts_xlf
FFIXED  = -qfixed

# pre-processor flag
PP      = -D -traditional
MPI_INCLUDES = -I$(NCMPI_PATH)/include     #----YKT added

ifdef PDTDIR
OPT=-optPDBFile=merged.pdb -optTauSelectFile="select.tau" -optReset=""
LIB_NCMPI = -L/home/riley/lib -L$(NCMPI_PATH)/lib -lmass -lmassv
else
#CONFIG_LIB = -L/bgl/BlueLight/ppcfloor/bglsys/lib -lmpich.rts -lmsglayer.rts -ldevices.rts -lrts.rts -ldevices.rts
LIB_NCMPI = -L/home/riley/lib -L$(NCMPI_PATH)/lib -lmpich.rts -lmsglayer.rts -\lrts.rts -ldevices.rts -lmass -lmassv
endif

FCOMP   = $(TAU_COMPILER) $(OPT) blrts_xlf90 
CCOMP   = $(TAU_COMPILER) blrts_xlc 
CPPCOMP = $(TAU_COMPILER) blrts_xlC 
LINK    = $(TAU_COMPILER) blrts_xlf90  


#----------------------------------------------------------------------------
# Compilation flags
#
#  Three sets of compilation/linking flags are defined: one for optimized
#  code, one for testing, and one for debugging.  The default is to use the 
#  _OPT version.  Specifying -debug to setup will pick the _DEBUG version,
#  these should enable bounds checking.  Specifying _TEST is used for 
#  flash_test, and is set for quick code generation, and (sometimes) 
#  profiling.  The Makefile generated by setup will assign the generic token 
#  (ex. FFLAGS) to the proper set of flags (ex. FFLAGS_OPT).
#----------------------------------------------------------------------------

FFLAGS_OPT   = -g -O3 -qarch=440 -qtune=440 \
               -qmaxmem=64000 -qnosave \
               -qrealsize=8 -qintsize=4 \
	       -qalias=noaryovrlp -c

FFLAGS_DEBUG = -g -qarch=440 -qrealsize=8 -cpp -c 

FFLAGS_TEST  = -g -qarch=440 -qmaxmem=64000 -qnosave -qrealsize=8 -c \
                $(MPI_INCLUDES)

F90FLAGS     = -qsuffix=f=F90:cpp=F90 -qfree=f90
f90FLAGS     = -qsuffix=f=f90:cpp=F90 -qfree=f90


# if we are using HDF5, we need to specify the path to the include files
CFLAGS_HDF5  = 

CFLAGS_OPT   = -g -c -O3 $(MPI_INCLUDES) -DIBM -D_FILE_OFFSET_BITS=64
CFLAGS_DEBUG = -g -c $(MPI_INCLUDES) -DIBM 
CFLAGS_TEST  = -c $(MPI_INCLUDES) -DIBM 

#CFLAGS_HDF5  = 
#CFLAGS_NCMPI =

MDEFS = -WF,

#----------------------------------------------------------------------------
# Linker flags
#
#  There is a seperate version of the linker flags for each of the _OPT, 
#  _DEBUG, and _TEST cases.
#----------------------------------------------------------------------------

#  Linker flags
LFLAGS_OPT   = -o
LFLAGS_DEBUG = -o
LFLAGS_TEST  = -o

#----------------------------------------------------------------------------
# Library specific linking
#
#  If a FLASH module has a 'LIBRARY xxx' line in its Config file, we need to
#  create a macro in this Makefile.h for LIB_xxx, which will be added to the
#  link line when FLASH is built.  This allows us to switch between different
#  (incompatible) libraries.  We also create a _OPT, _DEBUG, and _TEST
#  library macro to add any performance-minded libraries (like fast math),
#  depending on how FLASH was setup.
#----------------------------------------------------------------------------

LIB_HDF5  = 
LIB_MATH  = 
LIB_DEBUG =

#LIB_NCMPI = -L/home/riley/lib -L$(NCMPI_PATH)/lib -lmpich.rts -lmsglayer.rts -lrts.rts -ldevices.rts -lmass -lmassv

LIB_MPE   =
LIB_OPT   = $(LIB_NCMPI)
LIB_TEST  = $(LIB_NCMPI)

#----------------------------------------------------------------------------
# Additional machine-dependent object files
#
#  Add any machine specific files here -- they will be compiled and linked
#  when FLASH is built.
#----------------------------------------------------------------------------

MACHOBJ = 

#----------------------------------------------------------------------------
# Additional commands
#---------------------------------------------------------------------------- 

MV   = mv -f
AR   = ar -r
RM   = rm -f
CD   = cd
RL   = ranlib
ECHO = echo
