#-------------------------------------------------------------------
# FLASH makefile definitions for the NERSC IBM SP2 (seabor) : 64-bit
# 
#
# NOTE: seaborg makes a lot of use of Modules, so it is 
# important to make sure that the following modules are
# loaded in order to compile FLASH:
#
#  python
#  hdf5_par_64/1.4.5-post2
#
# Optional modules:
#
#  GNU
#  MASS
#
# These modules set environment variables pointing to the
# proper library locations. Only python is essential.
#-------------------------------------------------------------------

#----------------------------------------------------------------------------
# Set the HDF/HDF5 library paths -- these need to be updated for your system
#----------------------------------------------------------------------------

HDF5_PATH = /opt/pkgs/hdf5/hdf5-1.6.5-xlf
PNG_PATH = /sw
#ZLIB_PATH = /usr/common/usg/zlib_64/1.2.1/lib


#----------------------------------------------------------------------------
# Compiler and linker commands
#
#  We use the f90 compiler as the linker, so some C libraries may explicitly
#  need to be added into the link line.
#----------------------------------------------------------------------------

FCOMP   = /opt/pkgs/mpich-1.2.7p1/xlf/bin/mpif90
CCOMP   = /opt/pkgs/mpich-1.2.7p1/xlf/bin/mpicc
CPPCOMP = /opt/pkgs/mpich-1.2.7p1/xlf/bin/mpicc
LINK    = /opt/pkgs/mpich-1.2.7p1/xlf/bin/mpif90

#-----------------------------------------------------------------------------
# Compilation flags
#
#  Three sets of compilation/linking flags are defined: one for optimized code
#  code ("-opt"), one for debugging ("-debug"), and one for testing ("-test").
#  Passing these flags to the setup script will cause the value associated with
#  the corresponding keys (i.e. those ending in "_OPT", "_DEBUG", or "_TEST") to
#  be incorporated into the final Makefile. For example, passing "-opt" to the
#  setup script will cause the flags following "FFLAGS_OPT" to be assigned to
#  "FFLAGS" in the final Makefile. If none of these flags are passed, the default
#  behavior will match that of the "-opt" flag.
#  In general, "-opt" is meant to optimize compilation and linking. "-debug"
#  should enable runtime bounds checking, debugger symbols, and other compiler-
#  specific debugging options. "-test" is useful for testing different
#  combinations of compiler flags particular to your individual system.
#----------------------------------------------------------------------------
MDEFS = -WF,


FFLAGS_OPT   = -O3 -qmaxmem=64000 -qintsize=4 -qautodbl=dbl4 -qfixed -qnosave -c -qsuffix=cpp=F 
FFLAGS_TEST  = -O2 -qintsize=4 -qautodbl=dbl4 -qfixed -qnosave -c -qsuffix=cpp=F 
FFLAGS_DEBUG = -g -qintsize=4 -qautodbl=dbl4 -qfixed -qnosave  -c 

F90FLAGS     = -qsuffix=f=F90:cpp=F90 -qfree=f90 $(MDEFS)-DIBM
f90FLAGS     = -qsuffix=f=f90:cpp=F90 -qfree=f90

# if we are using HDF5, we need to specify the path to the include files
CFLAGS_HDF5  = -I${HDF5_PATH}/include -I${PNG_PATH}/include

CFLAGS_OPT   = -O2 -DIBM   -c $(CFLAGS_HDF5)

CFLAGS_TEST  = -O2 -DIBM   -c $(CFLAGS_HDF5)

CFLAGS_DEBUG = -g  -DIBM   -c $(CFLAGS_HDF5)




.SUFFIXES: .o .c .f .F .h .fh .F90 .f90

#----------------------------------------------------------------------------
# Linker flags
#
#  There is a seperate version of the linker flags for each of the _OPT, 
#  _DEBUG, and _TEST cases.
#----------------------------------------------------------------------------

LFLAGS_OPT   =  -o 
LFLAGS_TEST  =  -o 
LFLAGS_DEBUG =  -g -o 

#----------------------------------------------------------------------------
# Library specific linking
#
#  If a FLASH module has a 'LIBRARY xxx' line in its Config file, we need to
#  create a macro in this Makefile.h for LIB_xxx, which will be added to the
#  link line when FLASH is built.  This allows us to switch between different
#  (incompatible) libraries.  We also create a _OPT, _DEBUG, and _TEST
#  library macro to add any performance-minded libraries (like fast math),
#  depending on how FLASH was setup.
#----------------------------------------------------------------------------

LIB_HDF5  = -L${HDF5_PATH}/lib -lhdf5 -lz 
#            -L${PNG_PATH}/lib -lpng
LIB_MPI = -lmpichf90 -lpmpich -lmpich -lpmpich -lmpich 
#LIB_MATH  = -lessl

LIB_OPT   = -lSystemStubs
LIB_DEBUG = -lSystemStubs
LIB_TEST  = -lSystemStubs

#----------------------------------------------------------------------------
# Additional machine-dependent object files
#
#  Add any machine specific files here -- they will be compiled and linked
#  when FLASH is built.
#----------------------------------------------------------------------------

MACHOBJ = 

#----------------------------------------------------------------------------
# Additional commands
#---------------------------------------------------------------------------- 

MV = mv -f
AR = ar -r
RM = rm -f
CD = cd
RL = ranlib
ECHO = echo

# full optimization does not work on runtime_parameters.F90
#runtime_parameters.o : runtime_parameters.F90
#	${FCOMP} ${FFLAGS_TEST}  $(F90FLAGS) $<

#amr_%.o : amr_%.F90
#	${FCOMP} ${FFLAGS_OPT_NOIPA} ${F90FLAGS} ${FDEFINES} $<
