!----------------------------------------------------------------------
! PARAMESH - an adaptive mesh library.
! Copyright (C) 2003
!
! Use of the PARAMESH software is governed by the terms of the
! usage agreement which can be found in the file
! 'PARAMESH_USERS_AGREEMENT' in the main paramesh directory.
!----------------------------------------------------------------------

!
! These are wrapper routines for some mpi calls. We need these to avoid an error
! message generated by the NAG F95 compiler, when the mpi routines are called for
! different datatypes within the same calling routine.
!


! Packing and Unpacking routines

       subroutine mpi_int_pack(inbuf,incount,datatype,outbuf,outsize, & 
     &                         position,comm,ierror)
       integer :: inbuf(*),outbuf(*)
       integer :: incount,datatype,outsize,position,comm,ierror

       call mpi_pack(inbuf,incount,datatype,outbuf,outsize,position, & 
     &               comm,ierror)

       return
       end subroutine mpi_int_pack



       subroutine mpi_int_unpack(inbuf,insize,position,outbuf,outcount, & 
     &                           datatype,comm,ierror)
       integer :: inbuf(*),outbuf(*)
       integer :: insize,position,outcount,datatype,comm,ierror

       call mpi_unpack(inbuf,insize,position,outbuf,outcount, & 
     &                 datatype,comm,ierror)

       return
       end subroutine mpi_int_unpack




! Sending and Receiving routines

       subroutine mpi_int_irecv(buf,count,datatype,source,tag, & 
     &                          comm,request,ierror)
       integer :: buf(*)
       integer :: count,datatype,source,tag,comm,request,ierror

       call mpi_irecv(buf,count,datatype,source,tag,comm,request,ierror)

       return
       end subroutine mpi_int_irecv


       subroutine mpi_int_recv(buf,count,datatype,source,tag, & 
     &                          comm,status,ierror)
       integer :: buf(*),status(*)
       integer :: count,datatype,source,tag,comm,ierror

       call mpi_recv(buf,count,datatype,source,tag,comm,status,ierror)

       return
       end subroutine mpi_int_recv


       subroutine mpi_int_ssend(buf,count,datatype,dest,tag,comm,ierror)
       integer :: buf(*)
       integer :: count,datatype,dest,tag,comm,ierror

       call mpi_ssend(buf,count,datatype,dest,tag,comm,ierror)

       return
       end subroutine mpi_int_ssend


       subroutine mpi_int_send(buf,count,datatype,dest,tag,comm,ierror)
       integer :: buf(*)
       integer :: count,datatype,dest,tag,comm,ierror

       call mpi_send(buf,count,datatype,dest,tag,comm,ierror)

       return
       end subroutine mpi_int_send


       subroutine mpi_int_sendrecv(sendbuf,sendcount,sendtype,dest, & 
     &                             sendtag, & 
     &                             recvbuf,recvcount,recvtype,source, & 
     &                             recvtag, & 
     &                             comm,istatus,ierror)
       include 'mpif.h'
       integer :: sendbuf(*), recvbuf(*)
       integer :: sendcount,sendtype,dest,sendtag
       integer :: recvcount,recvtype,source,recvtag
       integer :: comm,ierror
       integer :: istatus(MPI_STATUS_SIZE)

       call mpi_sendrecv(sendbuf,sendcount,sendtype,dest,sendtag, & 
     &                   recvbuf,recvcount,recvtype,source,recvtag, & 
     &                   comm,istatus,ierror)

       return
       end subroutine mpi_int_sendrecv


      subroutine send_int(iv,n,idest,itag,icom,ierr)

!------------------------------------------------------------------------
!
! This routine encapsulats the MPI_Send for integer arrays.
!
!
! Written :     Maharaj Bhat          February 2000
!------------------------------------------------------------------------
!
! Arguments:
!      iv             integer valued array
!      n              dimension of iv
!      idest          MPI destination of msg
!      itag           MPI tag of msg
!      icom           MPI communicator
!      ierr           MPI error flag
!
!------------------------------------------------------------------------

      implicit none
      include 'mpif.h'
     
      integer iv(*)
      integer n,idest,itag,icom,ierr

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      call MPI_SEND(iv,n,MPI_INTEGER, & 
     &                   idest,itag,icom,ierr)

      return
      end subroutine send_int


      subroutine recv_int(iv,n,isource,itag,icom,istatus,ierr)

!------------------------------------------------------------------------
!
! This routine encapsulats the MPI_Send for integer arrays.
!
!
! Written :     Maharaj Bhat          February 2000
!------------------------------------------------------------------------
!
! Arguments:
!      iv             integer valued array
!      n              dimension of iv
!      isource        MPI source of msg
!      itag           MPI tag of msg
!      icom           MPI communicator
!      istatus        MPI status flag
!      ierr           MPI error flag
!
!------------------------------------------------------------------------

      implicit none
      include 'mpif.h'
     
      integer iv(*)
      integer n,isource,itag,icom,ierr
      integer istatus(MPI_STATUS_SIZE)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      call MPI_RECV(iv,n,MPI_INTEGER, & 
     &                   isource,itag,icom,istatus,ierr)

      return
      end subroutine recv_int



! Reduction operations


       subroutine mpi_int_allreduce(sendbuf,recvbuf,count,datatype, & 
     &                              op,comm,ierror)
       integer :: sendbuf(*),recvbuf(*)
       integer :: count,datatype,op,comm,ierror

       call mpi_allreduce(sendbuf,recvbuf,count,datatype,op,comm,ierror)

       return
       end subroutine mpi_int_allreduce



       subroutine mpi_int_scan(sendbuf,recvbuf,count,datatype, & 
     &                         op,comm,ierror)
       integer :: sendbuf(*),recvbuf(*)
       integer :: count,datatype,op,comm,ierror

       call mpi_scan(sendbuf,recvbuf,count,datatype,op,comm,ierror)

       return
       end subroutine mpi_int_scan
