!----------------------------------------------------------------------
! PARAMESH - an adaptive mesh library.
! Copyright (C) 2003
!
! Use of the PARAMESH software is governed by the terms of the
! usage agreement which can be found in the file
! 'PARAMESH_USERS_AGREEMENT' in the main paramesh directory.
!----------------------------------------------------------------------

!
! These are wrapper routines for some mpi calls. We need these to avoid an error
! message generated by the NAG F95 compiler, when the mpi routines are called for
! different datatypes within the same calling routine.
!



! Packing and Unpacking routines

       subroutine mpi_logical_pack(inbuf,incount,datatype,outbuf, & 
     &                             outsize,position,comm,ierror)
       logical :: inbuf(*),outbuf(*)
       integer :: incount,datatype,outsize,position,comm,ierror

       call mpi_pack(inbuf,incount,datatype,outbuf,outsize, & 
     &               position,comm,ierror)

       return
       end subroutine mpi_logical_pack



       subroutine mpi_logical_unpack(inbuf,insize,position,outbuf, & 
     &                               outcount,datatype,comm,ierror)
       logical    :: inbuf(*),outbuf(*)
       integer :: insize,position,outcount,datatype,comm,ierror

       call mpi_unpack(inbuf,insize,position,outbuf,outcount, & 
     &                 datatype,comm,ierror)

       return
       end subroutine mpi_logical_unpack


! Sending and Receiving routines


       subroutine mpi_logical_irecv(buf,count,datatype,source,tag,comm, & 
     &                              request,ierror)
       logical :: buf(*)
       integer :: count,datatype,source,tag,comm,request,ierror

       call mpi_irecv(buf,count,datatype,source,tag,comm,request,ierror)

       return
       end subroutine mpi_logical_irecv


       subroutine mpi_logical_recv(buf,count,datatype,source,tag, & 
     &                          comm,status,ierror)
       logical :: buf(*)
       integer :: status(*)
       integer :: count,datatype,source,tag,comm,ierror

       call mpi_recv(buf,count,datatype,source,tag,comm,status,ierror)

       return
       end subroutine mpi_logical_recv


       subroutine mpi_logical_ssend(buf,count,datatype,dest, & 
     &                              tag,comm,ierror)
       logical :: buf(*)
       integer :: count,datatype,dest,tag,comm,ierror

       call mpi_ssend(buf,count,datatype,dest,tag,comm,ierror)

       return
       end subroutine mpi_logical_ssend


       subroutine mpi_logical_send(buf,count,datatype,dest, & 
     &                              tag,comm,ierror)
       logical :: buf(*)
       integer :: count,datatype,dest,tag,comm,ierror

       call mpi_send(buf,count,datatype,dest,tag,comm,ierror)

       return
       end subroutine mpi_logical_send


! Reduction operations


       subroutine mpi_logical_allreduce(sendbuf,recvbuf,count,datatype, & 
     &                                  op,comm,ierror)
       logical :: sendbuf(*),recvbuf(*)
       integer :: count,datatype,op,comm,ierror

       call mpi_allreduce(sendbuf,recvbuf,count,datatype,op,comm,ierror)

       return
       end subroutine mpi_logical_allreduce


       subroutine mpi_logical_scan(sendbuf,recvbuf,count,datatype, & 
     &                             op,comm,ierror)
       logical :: sendbuf(*),recvbuf(*)
       integer :: count,datatype,op,comm,ierror

       call mpi_scan(sendbuf,recvbuf,count,datatype,op,comm,ierror)

       return
       end subroutine mpi_logical_scan
