!----------------------------------------------------------------------
! PARAMESH - an adaptive mesh library.
! Copyright (C) 2003
!
! Use of the PARAMESH software is governed by the terms of the
! usage agreement which can be found in the file
! 'PARAMESH_USERS_AGREEMENT' in the main paramesh directory.
!----------------------------------------------------------------------

!
! These are wrapper routines for some mpi calls. We need these to avoid an error
! message generated by the NAG F95 compiler, when the mpi routines are called for
! different datatypes within the same calling routine.
!


! Packing and Unpacking routines

       subroutine mpi_real_pack(inbuf,incount,datatype,outbuf,outsize, & 
     &                          position,comm,ierror)
       real    :: inbuf(*),outbuf(*)
       integer :: incount,datatype,outsize,position,comm,ierror

       call mpi_pack(inbuf,incount,datatype,outbuf,outsize,position, & 
     &               comm,ierror)

       return
       end subroutine mpi_real_pack



       subroutine mpi_real_unpack(inbuf,insize,position,outbuf,outcount, & 
     &                            datatype,comm,ierror)
       real    :: inbuf(*),outbuf(*)
       integer :: insize,position,outcount,datatype,comm,ierror

       call mpi_unpack(inbuf,insize,position,outbuf,outcount,datatype, & 
     &                 comm,ierror)

       return
       end subroutine mpi_real_unpack



! Sending and Receiving routines

       subroutine mpi_real_irecv(buf,count,datatype,source,tag,comm, & 
     &                          request,ierror)
       real    :: buf(*)
       integer :: count,datatype,source,tag,comm,request,ierror

       call mpi_irecv(buf,count,datatype,source,tag,comm,request,ierror)

       return
       end subroutine mpi_real_irecv


       subroutine mpi_real_recv(buf,count,datatype,source,tag, & 
     &                          comm,status,ierror)
       real    :: buf(*)
       integer :: status(*)
       integer :: count,datatype,source,tag,comm,ierror

       call mpi_recv(buf,count,datatype,source,tag,comm,status,ierror)

       return
       end subroutine mpi_real_recv


       subroutine mpi_real_ssend(buf,count,datatype,dest,tag,comm, & 
     &                           ierror)
       real    :: buf(*)
       integer :: count,datatype,dest,tag,comm,ierror

       call mpi_ssend(buf,count,datatype,dest,tag,comm,ierror)

       return
       end subroutine mpi_real_ssend


       subroutine mpi_real_send(buf,count,datatype,dest,tag,comm, & 
     &                           ierror)
       real    :: buf(*)
       integer :: count,datatype,dest,tag,comm,ierror

       call mpi_send(buf,count,datatype,dest,tag,comm,ierror)

       return
       end subroutine mpi_real_send

       subroutine mpi_real_sendrecv(sendbuf,sendcount,sendtype,dest, & 
     &                             sendtag, & 
     &                             recvbuf,recvcount,recvtype,source, & 
     &                             recvtag, & 
     &                             comm,istatus,ierror)
       include 'mpif.h'
       real    :: sendbuf(*), recvbuf(*)
       integer :: sendcount,sendtype,dest,sendtag
       integer :: recvcount,recvtype,source,recvtag
       integer :: comm,ierror
       integer :: istatus(MPI_STATUS_SIZE)

       call mpi_sendrecv(sendbuf,sendcount,sendtype,dest,sendtag, & 
     &                   recvbuf,recvcount,recvtype,source,recvtag, & 
     &                   comm,istatus,ierror)

       return
       end subroutine mpi_real_sendrecv



      subroutine send_real(rv,n,idest,itag,icom,ierr)

!------------------------------------------------------------------------
!
! This routine encapsulats the MPI_Send for real arrays.
!
!
! Written :     Maharaj Bhat          February 2000
!------------------------------------------------------------------------
!
! Arguments:
!      rv             real valued array
!      n              dimension of rv
!      idest          MPI destination of msg
!      itag           MPI tag of msg
!      icom           MPI communicator
!      ierr           MPI error flag
!
!------------------------------------------------------------------------

      use paramesh_comm_data
      implicit none
      include 'mpif.h'
    
      real rv(*)
      integer n,idest,itag,icom,ierr

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      call MPI_SEND(rv,n,amr_mpi_real, & 
     &                   idest,itag,icom,ierr)

      return
      end subroutine send_real


      subroutine recv_real(rv,n,isource,itag,icom,istatus,ierr)

!------------------------------------------------------------------------
!
! This routine encapsulats the MPI_Send for integer arrays.
!
!
! Written :     Maharaj Bhat          February 2000
!------------------------------------------------------------------------
!
! Arguments:
!      rv             real valued array
!      n              dimension of rv
!      isource        MPI source of msg
!      itag           MPI tag of msg
!      icom           MPI communicator
!      istatus        MPI status flag
!      ierr           MPI error flag
!
!------------------------------------------------------------------------

      use paramesh_comm_data
      implicit none
      include 'mpif.h'
     
      real rv(*)
      integer n,isource,itag,icom,ierr
      integer istatus(MPI_STATUS_SIZE)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      call MPI_RECV(rv,n,amr_mpi_real, &
     &                   isource,itag,icom,istatus,ierr)

      return
      end subroutine recv_real



! Reduction operations


       subroutine mpi_real_allreduce(sendbuf,recvbuf,count,datatype, & 
     &                               op,comm,ierror)
       real    :: sendbuf(*),recvbuf(*)
       integer :: count,datatype,op,comm,ierror

       call mpi_allreduce(sendbuf,recvbuf,count,datatype, & 
     &                    op,comm,ierror)

       return
       end subroutine mpi_real_allreduce


       subroutine mpi_real_scan(sendbuf,recvbuf,count,datatype, & 
     &                          op,comm,ierror)
       real    :: sendbuf(*),recvbuf(*)
       integer :: count,datatype,op,comm,ierror

       call mpi_scan(sendbuf,recvbuf,count,datatype,op,comm,ierror)

       return
       end subroutine mpi_real_scan
