!!****if* source/diagnostics/ProtonImaging/ProtonImagingMain/ProtonDetection/pi_write2DetectorFile
!!
!! NAME
!!
!!  pi_write2DetectorFile
!!
!! SYNOPSIS
!!
!!  call pi_write2DetectorFile (integer, intent (in) :: detector,
!!                              integer, intent (in) :: bucketCount,
!!                              real,    intent (in) :: bucket (:,:))
!!
!! DESCRIPTION
!!
!!  Writes a bucket of screen protons to the appropriate detector file on disk. The file is
!!  already open and the screen protons data is appended. Only the master processor writes
!!  out the bucket.
!!
!! ARGUMENTS
!!
!!  detector      : The detector number
!!  bucketCount   : Number of elements in bucket (currently number of screen [x,y] pairs)
!!  bucket        : The bucket containing all the elements.
!!
!!***

subroutine pi_write2DetectorFile (detector, bucketCount, bucket)

  use Driver_interface,    ONLY : Driver_abortFlash

  use ProtonImaging_data,  ONLY : pi_detectorFilesID,       &
                                  pi_detectorXYwriteFormat, &
                                  pi_globalMe

  implicit none

#include "constants.h"
#include "ProtonImaging.h"
#include "Flash.h"

  integer, intent (in) :: detector
  integer, intent (in) :: bucketCount
  real,    intent (in) :: bucket (1:2,1:bucketCount)

  integer :: fileUnit
  integer :: n

  real    :: x,y
!
!
!   ...Write out the buckets only on the master processor.
!
!
  if (pi_globalMe /= MASTER_PE) then
      return
  end if
!
!
!   ...Do the writing..
!
!
  fileUnit = pi_detectorFilesID (detector)

  do n = 1,bucketCount

     x = bucket (1,n)
     y = bucket (2,n)

     write (fileUnit, pi_detectorXYwriteFormat) x,y

  end do
!
!
!    ...Ready!
!
!
  return
end subroutine pi_write2DetectorFile
