!!****if* source/diagnostics/ProtonImaging/localAPI/pi_beam3DGridSetupStatistical
!!
!! NAME
!!
!!  pi_beam3DGridSetupStatistical
!!
!! SYNOPSIS
!!
!!  call pi_beam3DGridSetupStatistical (logical, intent (in)    :: seedInitialize,
!!                                      logical, intent (in)    :: seedIncrement,
!!                                      integer, intent (inout) :: seedMaximum,
!!                                      integer, intent (inout) :: seedStepping,
!!                                      integer, intent (out)   :: seed)
!!
!! DESCRIPTION
!!
!!  Sets up information needed for establishing a statistical cross sectional grid for a
!!  particular proton beam. Currently this consists only in setting the a seed value for
!!  a random uniform number generator supplied by Fortran 90 and the values for the maximum
!!  seed and the seed stepping number.
!!
!! ARGUMENTS
!!
!!  seedInitialize : indicates, whether the initial seed values need to be set
!!  seedIncrement  : indicates, whether the seed needs to be incremented
!!  seedMaximum    : the maximum seed value allowed
!!  seedStepping   : the seed stepping value
!!  seed           : the seed value
!!
!! NOTES
!!
!!  Only one seed value is needed to generate a definite (reproducible) sequence of random
!!  numbers using the canned f90 random number generator.
!!
!!***

subroutine pi_beam3DGridSetupStatistical (seedInitialize,               &
                                          seedIncrement,                &
                                                          seedMaximum,  &
                                                          seedStepping, &
                                                          seed          )

  implicit none

  logical, intent (in)    :: seedInitialize
  logical, intent (in)    :: seedIncrement
  integer, intent (inout) :: seedMaximum
  integer, intent (inout) :: seedStepping
  integer, intent (out)   :: seed

  seedMaximum  = 0
  seedStepping = 0
  seed         = 0

  return
end subroutine pi_beam3DGridSetupStatistical
