!!****ih* source/diagnostics/ProtonImaging/localAPI/pi_interface
!!
!! NAME
!!
!!  pi_interface
!!
!! SYNOPSIS
!!
!!   use pi_interface
!!
!! DESCRIPTION
!!
!!  This is the header file for the Proton Imaging unit that defines its
!!  private interfaces.
!!
!!***

Module pi_interface

  interface
     subroutine pi_beam3DGridPointsStatistical (radius,                             &
                                                seed,                               &
                                                targetTotalGridPoints,              &
                                                startGrid,                          &
                                                maxGridPoints,                      &
                                                                    moreGridPoints, &
                                                                    nGridPoints,    &
                                                                    xGrid,          &
                                                                    yGrid           )

       real,    intent (in)    :: radius
       integer, intent (in)    :: seed
       integer, intent (in)    :: targetTotalGridPoints
       logical, intent (inout) :: startGrid
       integer, intent (in)    :: maxGridPoints
       logical, intent (out)   :: moreGridPoints
       integer, intent (out)   :: nGridPoints
       real,    intent (out)   :: xGrid (1:maxGridPoints)
       real,    intent (out)   :: yGrid (1:maxGridPoints)
     end subroutine pi_beam3DGridPointsStatistical
  end interface

  interface
     subroutine pi_beam3DGridSetupStatistical (seedInitialize,               &
                                               seedIncrement,                &
                                                               seedMaximum,  &
                                                               seedStepping, &
                                                               seed          )

       logical, intent (in)    :: seedInitialize
       logical, intent (in)    :: seedIncrement
       integer, intent (inout) :: seedMaximum
       integer, intent (inout) :: seedStepping
       integer, intent (out)   :: seed
     end subroutine pi_beam3DGridSetupStatistical
  end interface

  interface
     subroutine pi_beamsCheck2DCyl3D () 
     end subroutine pi_beamsCheck2DCyl3D
  end interface

  interface
     subroutine pi_beamsCheck3DRec () 
     end subroutine pi_beamsCheck3DRec
  end interface

  interface
     subroutine pi_beamsCheck () 
     end subroutine pi_beamsCheck
  end interface

  interface
     subroutine pi_beamsInfo3DRec () 
     end subroutine pi_beamsInfo3DRec
  end interface

  interface
     subroutine pi_beamsInfo () 
     end subroutine pi_beamsInfo
  end interface

  interface
     subroutine pi_blockData3DRec (iminBlock, imaxBlock, &
                                   jminBlock, jmaxBlock, &
                                   kminBlock, kmaxBlock, &
                                   blockData             )

       integer, intent (in) :: iminBlock, imaxBlock
       integer, intent (in) :: jminBlock, jmaxBlock
       integer, intent (in) :: kminBlock, kmaxBlock
       real,    intent (in) :: blockData (:,:,:,:)
     end subroutine pi_blockData3DRec
  end interface

  interface
     subroutine pi_calculateCellData (coordMin, coordMax, indexMin, indexMax, delta, coords)
       real,    intent (in)  :: coordMin, coordMax
       integer, intent (in)  :: indexMin, indexMax
       real,    intent (out) :: delta
       real,    intent (out) :: coords (indexMin:indexMax+1)
     end subroutine pi_calculateCellData
  end interface

  interface
     subroutine pi_closeDetectorFiles () 
     end subroutine pi_closeDetectorFiles
  end interface

  interface
     subroutine pi_createProtons3DRec (blockCount, blockList, timeSimulation)
       integer, intent (in) :: blockCount
       integer, intent (in) :: blockList (1:blockCount)
       real,    intent (in) :: timeSimulation
     end subroutine pi_createProtons3DRec
  end interface

  interface
     subroutine pi_createProtons (blockCount, blockList, timeSimulation)
       integer, intent (in) :: blockCount
       integer, intent (in) :: blockList (1:blockCount)
       real,    intent (in) :: timeSimulation
     end subroutine pi_createProtons
  end interface

  interface
     subroutine pi_createProtonTags () 
     end subroutine pi_createProtonTags
  end interface

  interface
     subroutine pi_detectorsCheck3DRec () 
     end subroutine pi_detectorsCheck3DRec
  end interface

  interface
     subroutine pi_detectorsCheck () 
     end subroutine pi_detectorsCheck
  end interface

  interface
     subroutine pi_detectorsInfo3DRec () 
     end subroutine pi_detectorsInfo3DRec
  end interface

  interface
     subroutine pi_detectorsInfo () 
     end subroutine pi_detectorsInfo
  end interface

  interface
     subroutine pi_flushScreenProtons2Disk () 
     end subroutine pi_flushScreenProtons2Disk
  end interface

  interface
     subroutine pi_initializeBeams () 
     end subroutine pi_initializeBeams
  end interface

  interface
     subroutine pi_initializeDetectors () 
     end subroutine pi_initializeDetectors
  end interface

  interface
     function pi_maxConfinement3DRec (nc,y)
       integer, intent (in) :: nc
       real,    intent (in) :: y (:)
       real                 :: pi_maxConfinement3DRec (1:nc)
     end function pi_maxConfinement3DRec
  end interface

  interface
     function pi_minConfinement3DRec (nc,y)
       integer, intent (in) :: nc
       real,    intent (in) :: y (:)
       real                 :: pi_minConfinement3DRec (1:nc)
     end function pi_minConfinement3DRec
  end interface

  interface
     subroutine pi_openDetectorFiles (timeSimulation) 
       real, intent (in) :: timeSimulation
     end subroutine pi_openDetectorFiles
  end interface

  interface
     subroutine pi_printBeamsData () 
     end subroutine pi_printBeamsData
  end interface

  interface
     subroutine pi_printBlockVariable (blockID, variable, fileUnit)
       integer, intent (in) :: blockID
       integer, intent (in) :: variable
       integer, intent (in) :: fileUnit
     end subroutine pi_printBlockVariable
  end interface

  interface
     subroutine pi_printDetectorsData () 
     end subroutine pi_printDetectorsData
  end interface

  interface
     subroutine pi_printMainData () 
     end subroutine pi_printMainData
  end interface

  interface
     subroutine pi_printMatrix (fileUnit,                   &
                                title,                      &
                                rowMinMatrix, rowMaxMatrix, &
                                colMinMatrix, colMaxMatrix, &
                                rowMinPrint , rowMaxPrint,  &
                                colMinPrint , colMaxPrint,  &
                                matrix                      )

      integer,           intent (in) :: fileUnit
      character (len=*), intent (in) :: title
      integer,           intent (in) :: rowMinMatrix, rowMaxMatrix
      integer,           intent (in) :: colMinMatrix, colMaxMatrix
      integer,           intent (in) :: rowMinPrint , rowMaxPrint
      integer,           intent (in) :: colMinPrint , colMaxPrint
      real,              intent (in) :: matrix (rowMinMatrix : rowMaxMatrix , colMinMatrix : colMaxMatrix)
     end subroutine pi_printMatrix
  end interface

  interface
     subroutine pi_printProtonsData (fileLabel, processorID) 
       character (len=*), intent (in) :: fileLabel
       integer,           intent (in) :: processorID
     end subroutine pi_printProtonsData
  end interface

  interface
     subroutine pi_protonsBlockIDInfo () 
     end subroutine pi_protonsBlockIDInfo
  end interface

  interface
     subroutine pi_recordProtonOnScreen (px, py, pz, vx, vy, vz, detector)
       real,    intent (in) :: px, py, pz
       real,    intent (in) :: vx, vy, vz
       integer, intent (in) :: detector
     end subroutine pi_recordProtonOnScreen
  end interface

  interface
     subroutine pi_setupBeams () 
     end subroutine pi_setupBeams
  end interface

  interface
     subroutine pi_setupDetectors () 
     end subroutine pi_setupDetectors
  end interface

  interface
     subroutine pi_setupProtons () 
     end subroutine pi_setupProtons
  end interface

  interface
     subroutine pi_setupScreenProtons () 
     end subroutine pi_setupScreenProtons
  end interface

  interface
     real function pi_time2FacesParabolicPath1D (pos, vel, acc, minFace, maxFace, noFaceTime)
       real, intent (in) :: pos, vel, acc
       real, intent (in) :: minFace, maxFace
       real, intent (in) :: noFaceTime
     end function pi_time2FacesParabolicPath1D
  end interface

  interface
     subroutine pi_traceBlockProtons3DRec (protonFirst,  protonLast,          &
                                           iminBlock, imaxBlock,              &
                                           jminBlock, jmaxBlock,              &
                                           kminBlock, kmaxBlock,              &
                                           xminBlock, xmaxBlock,              &
                                           yminBlock, ymaxBlock,              &
                                           zminBlock, zmaxBlock,              &
                                           deltaX, deltaY, deltaZ,            &
                                           deltaInvX, deltaInvY, deltaInvZ,   &
                                           blockReflectMinX,                  &
                                           blockReflectMaxX,                  &
                                           blockReflectMinY,                  &
                                           blockReflectMaxY,                  &
                                           blockReflectMinZ,                  &
                                           blockReflectMaxZ                   ) 

       integer, intent (in)    :: protonFirst, protonLast   
       integer, intent (in)    :: iminBlock, imaxBlock
       integer, intent (in)    :: jminBlock, jmaxBlock
       integer, intent (in)    :: kminBlock, kmaxBlock
       real,    intent (in)    :: xminBlock, xmaxBlock
       real,    intent (in)    :: yminBlock, ymaxBlock
       real,    intent (in)    :: zminBlock, zmaxBlock
       real,    intent (in)    :: deltaX, deltaY, deltaZ
       real,    intent (in)    :: deltaInvX, deltaInvY, deltaInvZ
       logical, intent (in)    :: blockReflectMinX
       logical, intent (in)    :: blockReflectMaxX
       logical, intent (in)    :: blockReflectMinY
       logical, intent (in)    :: blockReflectMaxY
       logical, intent (in)    :: blockReflectMinZ
       logical, intent (in)    :: blockReflectMaxZ
     end subroutine pi_traceBlockProtons3DRec
  end interface

  interface
     function pi_traceODEfunction3DRec (t,y)
       real, intent (in) :: t
       real, intent (in) :: y (:)
       real :: pi_traceODEfunction3DRec (1:size (y))
     end function pi_traceODEfunction3DRec
  end interface

  interface
     subroutine pi_traceProtons3DRec () 
     end subroutine pi_traceProtons3DRec
  end interface

  interface
     subroutine pi_traceProtons () 
     end subroutine pi_traceProtons
  end interface

  interface
     subroutine pi_updateProtons (doMove) 
       logical, intent (in) :: doMove
     end subroutine pi_updateProtons
  end interface

  interface
     subroutine pi_write2DetectorFile (detector, bucketCount, bucket)
       integer, intent (in) :: detector
       integer, intent (in) :: bucketCount
       real,    intent (in) :: bucket (1:2,1:bucketCount)
     end subroutine pi_write2DetectorFile
  end interface

end Module pi_interface
