!!****if* source/diagnostics/ProtonImaging/localAPI/pi_recordProtonOnScreen
!!
!! NAME
!!
!!  pi_recordProtonOnScreen
!!
!! SYNOPSIS
!!
!!  call pi_recordProtonOnScreen (real    (in)    :: px,
!!                                real    (in)    :: py,
!!                                real    (in)    :: pz,
!!                                real    (in)    :: vx,
!!                                real    (in)    :: vy,
!!                                real    (in)    :: vz,
!!                                integer (in)    :: detector)
!!
!! DESCRIPTION
!!
!!  Records the proton on the specified detector screen. The proton has an initial
!!  position and direction (velocity). The routine records the (x,y) location on
!!  the screen in terms of local screen coordinates. The (x,y) screen coordinates
!!  are rescaled to the square screen side length, such that all protons within
!!  the screen have screen coordinate paris (x,y) within the range [0,1]. Protons
!!  falling outside the screen are also saved for eventual further diagnostic studies.
!!  Protons which miss the screen plane entirely are not recorded.
!!
!!  Procedure:
!!
!!                                    |\
!!                                    | \
!!                                    |  \                   C = detector center
!!                                    |   \
!!                                    | u1|\                 n = normal unit vector
!!                                    |   | \
!!                                    |   |  \         u1,u2,n = form a right hand rule
!!                      detector      |   |   |                  orthogonal vector system:
!!                                    |   C---|---> n                  n = u1 x u2
!!                       screen       \  / \  |
!!                                     \/   \ |              P = position of proton
!!                                     /\  u2\|
!!                                    /  \    |              v = velocity vector of proton
!!                                   /    \   |
!!                                  /      \  |          (x,y) = local screen coordinates
!!                                 /        \ |                  (basis = u1,u2) where proton
!!                                /          \|                   will hit the screen plane
!!                               /
!!                              /
!!                             /
!!                            P---> v -----------(x,y)
!!
!!
!!  From geometry:
!!
!!                    i) [(C - P) dot n] / [v dot n] > 0  ->  proton hits screen plane
!!
!!                   ii) x = u2 dot [(C - P) cross v] / [v dot n]
!!
!!                  iii) y = u1 dot [v cross (C - P)] / [v dot n]
!!
!!                   iv) if [v dot n] = 0  ->  parallel to screen plane, no hit
!!
!!
!! ARGUMENTS
!!
!!  px       : position global x-coordinate of the proton
!!  py       : position global y-coordinate of the proton
!!  pz       : position global z-coordinate of the proton
!!  vx       : velocity x-coordinate component of the proton
!!  vy       : velocity y-coordinate component of the proton
!!  vz       : velocity z-coordinate component of the proton
!!  detector : detector screen number on which to record the proton
!!
!! NOTES
!!
!!***

subroutine pi_recordProtonOnScreen (px, py, pz, vx, vy, vz, detector)

  implicit none

  real,    intent (in) :: px, py, pz
  real,    intent (in) :: vx, vy, vz
  integer, intent (in) :: detector

  return
end subroutine pi_recordProtonOnScreen
