!!****if* source/physics/Eos/EosMain/multiTemp/Gamma/eos_helmConstData
!!
!! NAME
!!
!!  eos_helmConstData
!!
!!
!! SYNOPSIS
!!
!!  use eos_helmConstData
!!
!!
!! DESCRIPTION
!!
!! mathematical and physical constants (in cgs,except e0 which is in ev)
!!
!! the 1986 codta recommended valeus of the physical constants
!! by coehn & taylor 
!!
!! ARGUMENTS
!!
!! PARAMETERS
!!
!!
!!  NOTES
!!    See Timmes and Swesty, 2000, AJSS, "The Accuracy, Consistency, and Speed of an Electron-Positron
!!    Equation of State Based on Table Interpolation of the Helmholtz Free Energy"
!!
!!
!!  COPIED FROM HELMHOLTZ EOS FOR THE TIME BEIN, until I figure out which
!!  constants (if any) are actually needed by a multiTemp Gamma EOS extension. - KW
!!  
!!***

#include "constants.h"

Module eos_helmConstData

  implicit none
  !
  !..physical constants   
  real,parameter ::     g       = 6.67259e-8,                           &
!       h       = 6.6260755e-27,                        &
!       hbar    = 0.5 * h/pi,                           &
!       e       = 4.8032068e-10,                        &
!       avo     = 6.0221367e23,                         &
!       c       = 2.99792458e10,                        &
!       kerg    = 1.380658e-16,                         &
       kev     = 8.617385e-5,                          &
!       amu     = 1.6605402e-24,                        &
       mn      = 1.6749286e-24,                        &
       mp      = 1.6726231e-24,                        &
       me      = 9.1093897e-28,                        &
!       rbohr   = hbar*hbar/(me * e * e),               &
!       fine    = e*e/(hbar*c),                         &
       hion    = 13.605698140e0,                       &
       ev2erg  = 1.602e-12,                             &
!       ssol     = 5.67051e-5,                          &
!       asol    = 4.0e0 * ssol / c,                     &
       rhonuc  = 2.342e14


  !..astronomical constants   
  real, parameter ::  msol    = 1.9892e33,                            &
       rsol    = 6.95997e10,                           &
       lsol    = 3.8268e33,                            &
       mearth  = 5.9764e27,                            &
       rearth  = 6.37e8,                               &
       ly      = 9.460528e17,                          &
       pc      = 3.261633e0 * ly,                      &
       au      = 1.495978921e13,                       &
       secyer  = 3.1558149984e7

  !.. derived constants
!!$  real, parameter :: &
!!$       avoinv  = 1.0e0/avo, & 
!!$       kergavo = kerg * avo, &
!!$       asoli3  = asol/3.0e0, &
!!$       sioncon = (2.0e0 * pi * amu * kerg)/(h*h)

  ! The following are initialized by eos_initPhysData - KW

  real, save :: eos_h, eos_hbar ! Planck const
  real, save :: eos_avo         ! Avogadro number
!!$  real, save :: eos_c           ! speed of light in vacuum
  real, save :: eos_kerg        ! Boltzmann constant, should be in erg/Kelvin
  real, save :: eos_amu         ! atomic mass unit
!!$  real, save :: eos_ssol        ! Stefan-Boltzmann constant
!!$  real, save :: eos_asol        ! derived from Stefan-Boltzmann constant
  real, save :: eos_avoinv          ! derived from Avogadro number
  real, save :: eos_kergavo         ! derived from Avogadro number
  real, save :: eos_ao3         ! Radiation constant / 3.0 - replaces asoli3
  real, save :: eos_sioncon     ! sioncon = (2.0e0 * pi * amu * kerg)/(h*h)

end Module eos_helmConstData
