!!****if* source/physics/materialProperties/Opacity/OpacityMain/Multispecies/method/LowTemp/op_setPEcoeffsAij4atomsZ100
!!
!! NAME
!!
!!  op_setPEcoeffsAij4atomsZ100
!!
!! SYNOPSIS
!!
!!  call op_setPEcoeffsAij4atomsZ100 ()
!!
!! DESCRIPTION
!!
!!  This routine sets the A(i,j,4) expansion coefficients for determining the
!!  photoelectric cross sections according to the F.Biggs and R.Lighthill report:
!!
!!       Analytical Approximations for X-Ray Cross Sections II
!!       Frank Biggs and Ruth Lighthill
!!       Weapons Effects Research Department
!!       Sandia Laboratories, December 1971
!!
!!  The atomic elements set in this routine are in the range: 74 =< Z =< 100.
!!  The data is NOT the updated data from the 1988 update of the report!
!!  This routine can only be called after all the necessary arrays have
!!  been allocated.
!!
!! ARGUMENTS
!!
!!***
subroutine op_setPEcoeffsAij4atomsZ100 ()

  use op_lowTempData,   ONLY : op_Aij4
  use Driver_interface, ONLY : Driver_abortFlash

  implicit none

  logical :: arrayExists
  logical :: goodShape
!
!
!   ...Check allocation and dimensions of the coefficient array.
!
!
  arrayExists = allocated (op_Aij4)

  if (.not.arrayExists) then
       call Driver_abortFlash ('[op_setPEcoeffsAij4atomsZ100] ERROR: array op_Aij4 not allocated')
  end if

  goodShape =       (size (op_Aij4,1) == 4)   &
              .and. (size (op_Aij4,2) == 13)  &
              .and. (size (op_Aij4,3) == 100)

  if (.not.goodShape) then
       call Driver_abortFlash ('[op_setPEcoeffsAij4atomsZ100] ERROR: array op_Aij4 has bad shape')
  end if
!
!
!   ...Set the expansion coefficient array. Last index denotes the
!      atomic order number Z.
!
!
  op_Aij4 (1:4, 1, 74) = (/ 1.480E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 74) = (/-6.028E+02, 5.107E+03,-8.969E+02, 5.534E+01/)
  op_Aij4 (1:4, 3, 74) = (/ 1.573E+02, 1.107E+04,-1.381E+03, 4.379E+02/)
  op_Aij4 (1:4, 4, 74) = (/-4.666E+02, 1.057E+04, 1.355E+04,-3.503E+03/)
  op_Aij4 (1:4, 5, 74) = (/-4.159E+02, 8.743E+03, 2.421E+04,-6.191E+03/)
  op_Aij4 (1:4, 6, 74) = (/-3.984E+02, 1.026E+04, 2.349E+04,-7.382E+03/)
  op_Aij4 (1:4, 7, 74) = (/-1.624E+02, 6.121E+03, 5.094E+04,-5.128E+04/)
  op_Aij4 (1:4, 8, 74) = (/-6.206E+01, 4.525E+03, 2.194E+05,-1.223E+05/)
  op_Aij4 (1:4, 9, 74) = (/-1.211E+02, 1.058E+04, 2.701E+05,-2.556E+05/)
  op_Aij4 (1:4,10, 74) = (/-2.105E+01, 4.623E+03, 5.109E+05,-1.835E+06/)
  op_Aij4 (1:4,11, 74) = (/ 5.276E+00, 2.999E+03, 4.828E+06,-1.030E+08/)
  op_Aij4 (1:4,12, 74) = (/ 3.099E+00, 6.112E+03, 3.348E+06, 1.310E+08/)
 
  op_Aij4 (1:4, 1, 75) = (/ 1.570E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 75) = (/-6.033E+02, 5.404E+03,-1.108E+03, 9.354E+01/)
  op_Aij4 (1:4, 3, 75) = (/-1.992E+02, 1.014E+04, 2.835E+03,-8.434E+02/)
  op_Aij4 (1:4, 4, 75) = (/-4.438E+02, 1.148E+04, 1.253E+04,-3.520E+03/)
  op_Aij4 (1:4, 5, 75) = (/-4.181E+02, 9.456E+03, 2.446E+04,-6.751E+03/)
  op_Aij4 (1:4, 6, 75) = (/-3.940E+02, 1.126E+04, 2.277E+04,-7.800E+03/)
  op_Aij4 (1:4, 7, 75) = (/-1.220E+02, 5.627E+03, 5.672E+04,-5.943E+04/)
  op_Aij4 (1:4, 8, 75) = (/-5.721E+01, 4.284E+03, 2.331E+05,-1.270E+05/)
  op_Aij4 (1:4, 9, 75) = (/-1.192E+02, 1.081E+04, 2.828E+05,-2.788E+05/)
  op_Aij4 (1:4,10, 75) = (/-8.548E+00, 3.122E+03, 5.972E+05,-2.571E+06/)
  op_Aij4 (1:4,11, 75) = (/ 4.901E+00, 3.621E+03, 4.969E+06,-1.085E+08/)
  op_Aij4 (1:4,12, 75) = (/ 3.238E+00, 6.600E+03, 3.237E+06, 2.204E+08/)
 
  op_Aij4 (1:4, 1, 76) = (/ 1.720E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 76) = (/-5.720E+02, 5.552E+03,-1.165E+03, 9.523E+01/)
  op_Aij4 (1:4, 3, 76) = (/-2.871E+02, 9.431E+03, 5.398E+03,-1.569E+03/)
  op_Aij4 (1:4, 4, 76) = (/-4.312E+02, 1.174E+04, 1.298E+04,-3.813E+03/)
  op_Aij4 (1:4, 5, 76) = (/-4.172E+02, 1.036E+04, 2.374E+04,-7.166E+03/)
  op_Aij4 (1:4, 6, 76) = (/-3.837E+02, 1.147E+04, 2.383E+04,-8.523E+03/)
  op_Aij4 (1:4, 7, 76) = (/-1.463E+02, 6.213E+03, 5.661E+04,-5.933E+04/)
  op_Aij4 (1:4, 8, 76) = (/-5.946E+01, 4.643E+03, 2.401E+05,-1.434E+05/)
  op_Aij4 (1:4, 9, 76) = (/-1.116E+02, 1.035E+04, 3.012E+05,-2.961E+05/)
  op_Aij4 (1:4,10, 76) = (/-1.803E+01, 4.548E+03, 5.700E+05,-2.246E+06/)
  op_Aij4 (1:4,11, 76) = (/ 5.735E+00, 3.318E+03, 5.232E+06,-1.209E+08/)
  op_Aij4 (1:4,12, 76) = (/ 3.351E+00, 6.829E+03, 3.509E+06, 1.607E+08/)
 
  op_Aij4 (1:4, 1, 77) = (/ 1.890E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 77) = (/-6.457E+02, 5.954E+03,-1.292E+03, 1.094E+02/)
  op_Aij4 (1:4, 3, 77) = (/-2.830E+02, 9.680E+03, 5.768E+03,-1.746E+03/)
  op_Aij4 (1:4, 4, 77) = (/-4.371E+02, 1.150E+04, 1.539E+04,-4.560E+03/)
  op_Aij4 (1:4, 5, 77) = (/-4.125E+02, 1.085E+04, 2.464E+04,-7.841E+03/)
  op_Aij4 (1:4, 6, 77) = (/-3.774E+02, 1.211E+04, 2.431E+04,-9.223E+03/)
  op_Aij4 (1:4, 7, 77) = (/-2.858E+01, 3.946E+03, 7.479E+04,-9.186E+04/)
  op_Aij4 (1:4, 8, 77) = (/-5.153E+01, 4.097E+03, 2.582E+05,-1.419E+05/)
  op_Aij4 (1:4, 9, 77) = (/-1.366E+02, 1.456E+04, 2.707E+05,-3.461E+05/)
  op_Aij4 (1:4,10, 77) = (/-1.995E+01, 4.906E+03, 6.064E+05,-2.648E+06/)
  op_Aij4 (1:4,11, 77) = (/ 6.213E+00, 3.458E+03, 5.442E+06,-1.286E+08/)
  op_Aij4 (1:4,12, 77) = (/ 3.505E+00, 7.126E+03, 3.805E+06, 1.114E+08/)
 
  op_Aij4 (1:4, 1, 78) = (/ 2.060E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 78) = (/-6.891E+02, 6.259E+03,-1.339E+03, 1.009E+02/)
  op_Aij4 (1:4, 3, 78) = (/-3.204E+02, 9.105E+03, 8.268E+03,-2.460E+03/)
  op_Aij4 (1:4, 4, 78) = (/-4.333E+02, 1.147E+04, 1.710E+04,-5.181E+03/)
  op_Aij4 (1:4, 5, 78) = (/-4.064E+02, 1.162E+04, 2.438E+04,-8.327E+03/)
  op_Aij4 (1:4, 6, 78) = (/-3.694E+02, 1.237E+04, 2.562E+04,-1.013E+04/)
  op_Aij4 (1:4, 7, 78) = (/-1.003E+02, 5.662E+03, 6.740E+04,-7.624E+04/)
  op_Aij4 (1:4, 8, 78) = (/-6.719E+01, 5.737E+03, 2.565E+05,-1.880E+05/)
  op_Aij4 (1:4, 9, 78) = (/-1.399E+02, 1.605E+04, 2.671E+05,-3.742E+05/)
  op_Aij4 (1:4,10, 78) = (/ 1.076E+01, 9.617E+02, 7.776E+05,-4.207E+06/)
  op_Aij4 (1:4,11, 78) = (/ 6.088E+00, 4.001E+03, 5.552E+06,-1.300E+08/)
  op_Aij4 (1:4,12, 78) = (/ 3.646E+00, 7.453E+03, 3.931E+06, 1.226E+08/)
 
  op_Aij4 (1:4, 1, 79) = (/ 2.200E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 79) = (/-7.040E+02, 6.562E+03,-1.438E+03, 1.267E+02/)
  op_Aij4 (1:4, 3, 79) = (/-3.105E+02, 9.446E+03, 8.461E+03,-2.642E+03/)
  op_Aij4 (1:4, 4, 79) = (/-4.203E+02, 1.212E+04, 1.690E+04,-5.448E+03/)
  op_Aij4 (1:4, 5, 79) = (/-3.968E+02, 1.263E+04, 2.339E+04,-8.644E+03/)
  op_Aij4 (1:4, 6, 79) = (/-3.646E+02, 1.234E+04, 2.840E+04,-1.148E+04/)
  op_Aij4 (1:4, 7, 79) = (/-5.314E+00, 3.877E+03, 8.307E+04,-1.051E+05/)
  op_Aij4 (1:4, 8, 79) = (/-8.194E+01, 7.574E+03, 2.506E+05,-2.318E+05/)
  op_Aij4 (1:4, 9, 79) = (/-1.134E+02, 1.194E+04, 3.355E+05,-3.921E+05/)
  op_Aij4 (1:4,10, 79) = (/-3.392E+00, 3.070E+03, 7.247E+05,-3.499E+06/)
  op_Aij4 (1:4,11, 79) = (/ 7.917E+00, 2.918E+03, 6.095E+06,-1.596E+08/)
  op_Aij4 (1:4,12, 79) = (/ 3.811E+00, 7.846E+03, 4.248E+06, 4.571E+07/)
 
  op_Aij4 (1:4, 1, 80) = (/ 2.210E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 80) = (/-8.035E+02, 7.031E+03,-1.602E+03, 1.385E+02/)
  op_Aij4 (1:4, 3, 80) = (/-3.336E+02, 7.497E+03, 1.461E+04,-4.020E+03/)
  op_Aij4 (1:4, 4, 80) = (/-3.961E+02, 1.286E+04, 1.582E+04,-5.482E+03/)
  op_Aij4 (1:4, 5, 80) = (/-3.830E+02, 1.343E+04, 2.276E+04,-8.999E+03/)
  op_Aij4 (1:4, 6, 80) = (/-3.553E+02, 1.248E+04, 3.007E+04,-1.263E+04/)
  op_Aij4 (1:4, 7, 80) = (/ 1.179E+01, 3.460E+03, 9.044E+04,-1.201E+05/)
  op_Aij4 (1:4, 8, 80) = (/-7.533E+01, 7.084E+03, 2.675E+05,-2.412E+05/)
  op_Aij4 (1:4, 9, 80) = (/-1.288E+02, 1.524E+04, 3.090E+05,-4.397E+05/)
  op_Aij4 (1:4,10, 80) = (/ 4.681E-01, 2.405E+03, 7.958E+05,-4.232E+06/)
  op_Aij4 (1:4,11, 80) = (/ 6.983E+00, 4.309E+03, 5.980E+06,-1.518E+08/)
  op_Aij4 (1:4,12, 80) = (/ 3.946E+00, 8.311E+03, 4.256E+06, 8.942E+07/)
 
  op_Aij4 (1:4, 1, 81) = (/ 2.210E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 81) = (/-9.684E+02, 7.861E+03,-2.292E+03, 3.194E+02/)
  op_Aij4 (1:4, 3, 81) = (/-3.275E+02, 7.729E+03, 1.535E+04,-4.426E+03/)
  op_Aij4 (1:4, 4, 81) = (/-4.087E+02, 1.156E+04, 2.169E+04,-7.188E+03/)
  op_Aij4 (1:4, 5, 81) = (/-3.790E+02, 1.294E+04, 2.638E+04,-1.049E+04/)
  op_Aij4 (1:4, 6, 81) = (/-3.455E+02, 1.245E+04, 3.232E+04,-1.400E+04/)
  op_Aij4 (1:4, 7, 81) = (/ 5.060E+01, 2.722E+03, 1.006E+05,-1.441E+05/)
  op_Aij4 (1:4, 8, 81) = (/-6.647E+01, 6.324E+03, 2.871E+05,-2.439E+05/)
  op_Aij4 (1:4, 9, 81) = (/-1.405E+02, 2.137E+04, 2.340E+05,-4.290E+05/)
  op_Aij4 (1:4,10, 81) = (/-1.053E+01, 4.195E+03, 7.543E+05,-3.747E+06/)
  op_Aij4 (1:4,11, 81) = (/ 7.684E+00, 4.048E+03, 6.314E+06,-1.714E+08/)
  op_Aij4 (1:4,12, 81) = (/ 4.082E+00, 8.543E+03, 4.521E+06, 5.178E+07/)
 
  op_Aij4 (1:4, 1, 82) = (/ 2.200E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 82) = (/-7.847E+02, 7.343E+03,-1.425E+03,-3.703E-01/)
  op_Aij4 (1:4, 3, 82) = (/-3.238E+02, 8.188E+03, 1.567E+04,-4.802E+03/)
  op_Aij4 (1:4, 4, 82) = (/-4.016E+02, 1.173E+04, 2.316E+04,-7.955E+03/)
  op_Aij4 (1:4, 5, 82) = (/-3.717E+02, 1.215E+04, 3.143E+04,-1.232E+04/)
  op_Aij4 (1:4, 6, 82) = (/-3.372E+02, 1.292E+04, 3.306E+04,-1.511E+04/)
  op_Aij4 (1:4, 7, 82) = (/-1.004E+02, 6.410E+03, 7.864E+04,-9.524E+04/)
  op_Aij4 (1:4, 8, 82) = (/-7.125E+01, 7.126E+03, 2.931E+05,-2.779E+05/)
  op_Aij4 (1:4, 9, 82) = (/-1.363E+02, 1.992E+04, 2.721E+05,-4.887E+05/)
  op_Aij4 (1:4,10, 82) = (/-8.206E+00, 4.058E+03, 7.950E+05,-4.115E+06/)
  op_Aij4 (1:4,11, 82) = (/ 7.595E+00, 4.692E+03, 6.429E+06,-1.747E+08/)
  op_Aij4 (1:4,12, 82) = (/ 4.240E+00, 8.883E+03, 4.754E+06, 3.448E+07/)
 
  op_Aij4 (1:4, 1, 83) = (/ 2.030E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 83) = (/-1.030E+03, 8.535E+03,-2.431E+03, 2.738E+02/)
  op_Aij4 (1:4, 3, 83) = (/-3.149E+02, 7.642E+03, 1.884E+04,-5.632E+03/)
  op_Aij4 (1:4, 4, 83) = (/-3.943E+02, 1.129E+04, 2.671E+04,-9.164E+03/)
  op_Aij4 (1:4, 5, 83) = (/-3.641E+02, 1.204E+04, 3.463E+04,-1.381E+04/)
  op_Aij4 (1:4, 6, 83) = (/-3.300E+02, 1.254E+04, 3.747E+04,-1.725E+04/)
  op_Aij4 (1:4, 7, 83) = (/ 2.320E+01, 3.479E+03, 1.066E+05,-1.595E+05/)
  op_Aij4 (1:4, 8, 83) = (/-6.824E+01, 7.005E+03, 3.089E+05,-2.951E+05/)
  op_Aij4 (1:4, 9, 83) = (/-1.337E+02, 1.999E+04, 2.903E+05,-5.342E+05/)
  op_Aij4 (1:4,10, 83) = (/-1.411E+00, 3.094E+03, 8.901E+05,-5.203E+06/)
  op_Aij4 (1:4,11, 83) = (/ 9.935E+00, 3.289E+03, 6.983E+06,-2.020E+08/)
  op_Aij4 (1:4,12, 83) = (/ 4.425E+00, 9.396E+03, 5.008E+06,-5.280E+07/)
 
  op_Aij4 (1:4, 1, 84) = (/ 1.830E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 84) = (/-9.975E+02, 8.677E+03,-2.206E+03, 1.699E+02/)
  op_Aij4 (1:4, 3, 84) = (/-3.167E+02, 8.401E+03, 1.878E+04,-6.087E+03/)
  op_Aij4 (1:4, 4, 84) = (/-3.899E+02, 1.134E+04, 2.963E+04,-1.038E+04/)
  op_Aij4 (1:4, 5, 84) = (/-3.604E+02, 1.242E+04, 3.688E+04,-1.531E+04/)
  op_Aij4 (1:4, 6, 84) = (/-3.216E+02, 1.192E+04, 4.405E+04,-2.004E+04/)
  op_Aij4 (1:4, 7, 84) = (/-4.664E+01, 5.421E+03, 9.815E+04,-1.380E+05/)
  op_Aij4 (1:4, 8, 84) = (/-7.471E+01, 8.106E+03, 3.143E+05,-3.397E+05/)
  op_Aij4 (1:4, 9, 84) = (/-1.335E+02, 2.199E+04, 2.786E+05,-5.569E+05/)
  op_Aij4 (1:4,10, 84) = (/ 7.789E+00, 1.767E+03, 1.003E+06,-6.443E+06/)
  op_Aij4 (1:4,11, 84) = (/ 8.525E+00, 5.102E+03, 6.990E+06,-2.009E+08/)
  op_Aij4 (1:4,12, 84) = (/ 4.633E+00, 9.868E+03, 5.142E+06, 1.775E+07/)
 
  op_Aij4 (1:4, 1, 85) = (/ 1.610E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 85) = (/-1.158E+03, 9.785E+03,-3.286E+03, 4.977E+02/)
  op_Aij4 (1:4, 3, 85) = (/-2.939E+02, 7.047E+03, 2.494E+04,-7.295E+03/)
  op_Aij4 (1:4, 4, 85) = (/-3.889E+02, 1.171E+04, 3.195E+04,-1.160E+04/)
  op_Aij4 (1:4, 5, 85) = (/-3.573E+02, 1.264E+04, 4.031E+04,-1.720E+04/)
  op_Aij4 (1:4, 6, 85) = (/-3.206E+02, 1.263E+04, 4.560E+04,-2.199E+04/)
  op_Aij4 (1:4, 7, 85) = (/ 1.227E+02, 1.460E+03, 1.347E+05,-2.192E+05/)
  op_Aij4 (1:4, 8, 85) = (/-6.880E+01, 7.565E+03, 3.411E+05,-3.547E+05/)
  op_Aij4 (1:4, 9, 85) = (/-1.280E+02, 1.941E+04, 3.508E+05,-6.556E+05/)
  op_Aij4 (1:4,10, 85) = (/-1.695E+00, 3.621E+03, 9.663E+05,-5.746E+06/)
  op_Aij4 (1:4,11, 85) = (/ 9.913E+00, 4.822E+03, 7.370E+06,-2.163E+08/)
  op_Aij4 (1:4,12, 85) = (/ 4.871E+00, 1.062E+04, 5.352E+06,-2.702E+07/)
 
  op_Aij4 (1:4, 1, 86) = (/ 1.380E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 86) = (/-9.559E+02, 9.029E+03,-2.571E+03, 3.310E+02/)
  op_Aij4 (1:4, 3, 86) = (/-2.893E+02, 7.615E+03, 2.331E+04,-7.562E+03/)
  op_Aij4 (1:4, 4, 86) = (/-3.616E+02, 1.088E+04, 3.423E+04,-1.248E+04/)
  op_Aij4 (1:4, 5, 86) = (/-2.652E+02, 7.797E+03, 5.879E+04,-1.903E+04/)
  op_Aij4 (1:4, 6, 86) = (/-2.980E+02, 1.182E+04, 4.838E+04,-2.359E+04/)
  op_Aij4 (1:4, 7, 86) = (/-6.112E+01, 5.945E+03, 1.001E+05,-1.407E+05/)
  op_Aij4 (1:4, 8, 86) = (/-6.756E+01, 7.739E+03, 3.363E+05,-3.748E+05/)
  op_Aij4 (1:4, 9, 86) = (/-1.220E+02, 1.976E+04, 3.335E+05,-6.657E+05/)
  op_Aij4 (1:4,10, 86) = (/-5.899E-01, 3.312E+03, 9.833E+05,-6.258E+06/)
  op_Aij4 (1:4,11, 86) = (/ 8.633E+00, 5.888E+03, 7.122E+06,-2.120E+08/)
  op_Aij4 (1:4,12, 86) = (/ 4.841E+00, 1.057E+04, 5.286E+06, 1.022E+07/)
 
  op_Aij4 (1:4, 1, 87) = (/ 1.250E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 87) = (/-7.818E+02, 8.542E+03,-1.652E+03, 4.668E+01/)
  op_Aij4 (1:4, 3, 87) = (/-2.906E+02, 8.104E+03, 2.416E+04,-8.306E+03/)
  op_Aij4 (1:4, 4, 87) = (/-3.615E+02, 1.150E+04, 3.541E+04,-1.363E+04/)
  op_Aij4 (1:4, 5, 87) = (/-3.299E+02, 1.230E+04, 4.485E+04,-2.020E+04/)
  op_Aij4 (1:4, 6, 87) = (/-2.983E+02, 1.294E+04, 4.768E+04,-2.521E+04/)
  op_Aij4 (1:4, 7, 87) = (/-2.362E+01, 5.338E+03, 1.099E+05,-1.583E+05/)
  op_Aij4 (1:4, 8, 87) = (/-6.236E+01, 7.249E+03, 3.612E+05,-3.893E+05/)
  op_Aij4 (1:4, 9, 87) = (/-1.213E+02, 2.052E+04, 3.455E+05,-7.195E+05/)
  op_Aij4 (1:4,10, 87) = (/-8.453E+00, 4.660E+03, 9.757E+05,-6.010E+06/)
  op_Aij4 (1:4,11, 87) = (/ 1.104E+01, 4.630E+03, 7.706E+06,-2.445E+08/)
  op_Aij4 (1:4,12, 87) = (/ 5.063E+00, 1.115E+04, 5.666E+06,-1.082E+08/)
 
  op_Aij4 (1:4, 1, 88) = (/ 1.170E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 88) = (/-9.110E+02, 9.350E+03,-2.300E+03, 2.228E+02/)
  op_Aij4 (1:4, 3, 88) = (/-2.592E+02, 6.680E+03, 3.041E+04,-9.387E+03/)
  op_Aij4 (1:4, 4, 88) = (/-3.547E+02, 1.164E+04, 3.753E+04,-1.492E+04/)
  op_Aij4 (1:4, 5, 88) = (/-3.193E+02, 1.201E+04, 4.919E+04,-2.237E+04/)
  op_Aij4 (1:4, 6, 88) = (/-2.920E+02, 1.314E+04, 5.024E+04,-2.757E+04/)
  op_Aij4 (1:4, 7, 88) = (/-5.651E+01, 6.065E+03, 1.108E+05,-1.601E+05/)
  op_Aij4 (1:4, 8, 88) = (/-6.085E+01, 7.287E+03, 3.762E+05,-4.161E+05/)
  op_Aij4 (1:4, 9, 88) = (/-1.192E+02, 2.119E+04, 3.500E+05,-7.644E+05/)
  op_Aij4 (1:4,10, 88) = (/-1.024E+01, 5.310E+03, 9.756E+05,-5.773E+06/)
  op_Aij4 (1:4,11, 88) = (/ 1.241E+01, 3.727E+03, 8.307E+06,-2.833E+08/)
  op_Aij4 (1:4,12, 88) = (/ 5.221E+00, 1.168E+04, 5.739E+06,-9.046E+07/)
 
  op_Aij4 (1:4, 1, 89) = (/ 1.110E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 89) = (/-1.115E+03, 1.053E+04,-3.094E+03, 2.300E+02/)
  op_Aij4 (1:4, 3, 89) = (/-2.814E+02, 8.281E+03, 2.798E+04,-1.015E+04/)
  op_Aij4 (1:4, 4, 89) = (/-3.474E+02, 1.153E+04, 4.113E+04,-1.657E+04/)
  op_Aij4 (1:4, 5, 89) = (/-3.138E+02, 1.208E+04, 5.316E+04,-2.471E+04/)
  op_Aij4 (1:4, 6, 89) = (/-2.887E+02, 1.389E+04, 5.091E+04,-2.962E+04/)
  op_Aij4 (1:4, 7, 89) = (/ 7.657E+01, 2.243E+03, 1.528E+05,-2.731E+05/)
  op_Aij4 (1:4, 8, 89) = (/-6.339E+01, 7.915E+03, 3.884E+05,-4.637E+05/)
  op_Aij4 (1:4, 9, 89) = (/-1.182E+02, 2.194E+04, 3.621E+05,-8.244E+05/)
  op_Aij4 (1:4,10, 89) = (/ 8.142E+00, 2.287E+03, 1.185E+06,-8.493E+06/)
  op_Aij4 (1:4,11, 89) = (/ 1.003E+01, 6.690E+03, 7.961E+06,-2.545E+08/)
  op_Aij4 (1:4,12, 89) = (/ 5.479E+00, 1.217E+04, 5.897E+06,-2.287E+07/)
 
  op_Aij4 (1:4, 1, 90) = (/ 1.050E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 90) = (/-9.331E+02, 9.938E+03,-2.564E+03, 2.942E+02/)
  op_Aij4 (1:4, 3, 90) = (/-2.325E+02, 6.130E+03, 3.633E+04,-1.104E+04/)
  op_Aij4 (1:4, 4, 90) = (/-3.308E+02, 1.094E+04, 4.517E+04,-1.812E+04/)
  op_Aij4 (1:4, 5, 90) = (/-3.073E+02, 1.244E+04, 5.401E+04,-2.638E+04/)
  op_Aij4 (1:4, 6, 90) = (/-2.797E+02, 1.474E+04, 4.897E+04,-3.059E+04/)
  op_Aij4 (1:4, 7, 90) = (/ 8.999E+00, 4.311E+03, 1.390E+05,-2.331E+05/)
  op_Aij4 (1:4, 8, 90) = (/-5.998E+01, 7.669E+03, 4.039E+05,-4.843E+05/)
  op_Aij4 (1:4, 9, 90) = (/-1.152E+02, 2.229E+04, 3.694E+05,-8.743E+05/)
  op_Aij4 (1:4,10, 90) = (/-1.965E+01, 7.238E+03, 9.916E+05,-5.869E+06/)
  op_Aij4 (1:4,11, 90) = (/ 1.150E+01, 6.085E+03, 8.292E+06,-2.758E+08/)
  op_Aij4 (1:4,12, 90) = (/ 5.620E+00, 1.245E+04, 6.338E+06,-1.549E+08/)
 
  op_Aij4 (1:4, 1, 91) = (/ 1.000E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 91) = (/-1.195E+03, 1.151E+04,-3.714E+03, 5.109E+01/)
  op_Aij4 (1:4, 3, 91) = (/-2.349E+02, 6.438E+03, 3.849E+04,-1.223E+04/)
  op_Aij4 (1:4, 4, 91) = (/-3.240E+02, 1.078E+04, 5.022E+04,-2.015E+04/)
  op_Aij4 (1:4, 5, 91) = (/-3.119E+02, 1.369E+04, 5.366E+04,-2.824E+04/)
  op_Aij4 (1:4, 6, 91) = (/-2.785E+02, 1.523E+04, 5.214E+04,-3.377E+04/)
  op_Aij4 (1:4, 7, 91) = (/ 1.482E+02, 3.411E+02, 1.839E+05,-3.542E+05/)
  op_Aij4 (1:4, 8, 91) = (/-6.436E+01, 8.617E+03, 4.180E+05,-5.495E+05/)
  op_Aij4 (1:4, 9, 91) = (/-1.129E+02, 2.138E+04, 4.231E+05,-9.875E+05/)
  op_Aij4 (1:4,10, 91) = (/ 1.526E+01, 1.356E+03, 1.351E+06,-1.082E+07/)
  op_Aij4 (1:4,11, 91) = (/ 1.254E+01, 5.894E+03, 8.877E+06,-3.120E+08/)
  op_Aij4 (1:4,12, 91) = (/ 5.918E+00, 1.306E+04, 6.683E+06,-1.778E+08/)
 
  op_Aij4 (1:4, 1, 92) = (/ 9.700E+02, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 92) = (/-1.023E+03, 1.062E+04,-1.978E+03,-9.188E+02/)
  op_Aij4 (1:4, 3, 92) = (/-2.568E+02, 7.987E+03, 3.435E+04,-1.308E+04/)
  op_Aij4 (1:4, 4, 92) = (/-3.148E+02, 1.094E+04, 5.041E+04,-2.121E+04/)
  op_Aij4 (1:4, 5, 92) = (/-2.917E+02, 1.249E+04, 5.991E+04,-3.088E+04/)
  op_Aij4 (1:4, 6, 92) = (/-2.653E+02, 1.346E+04, 6.280E+04,-3.903E+04/)
  op_Aij4 (1:4, 7, 92) = (/ 5.629E+00, 4.645E+03, 1.486E+05,-2.622E+05/)
  op_Aij4 (1:4, 8, 92) = (/-6.720E+01, 9.489E+03, 4.107E+05,-6.000E+05/)
  op_Aij4 (1:4, 9, 92) = (/-1.033E+02, 1.864E+04, 4.878E+05,-1.076E+06/)
  op_Aij4 (1:4,10, 92) = (/ 6.281E-01, 3.792E+03, 1.281E+06,-1.003E+07/)
  op_Aij4 (1:4,11, 92) = (/ 1.146E+01, 7.568E+03, 8.699E+06,-3.106E+08/)
  op_Aij4 (1:4,12, 92) = (/ 6.017E+00, 1.327E+04, 7.079E+06,-2.456E+08/)
 
  op_Aij4 (1:4, 1, 93) = (/ 9.500E+02, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 93) = (/-2.189E+03, 1.746E+04,-1.117E+04, 1.713E+03/)
  op_Aij4 (1:4, 3, 93) = (/-2.579E+02, 8.351E+03, 3.599E+04,-1.430E+04/)
  op_Aij4 (1:4, 4, 93) = (/-3.232E+02, 1.200E+04, 5.096E+04,-2.302E+04/)
  op_Aij4 (1:4, 5, 93) = (/-2.932E+02, 1.303E+04, 6.385E+04,-3.410E+04/)
  op_Aij4 (1:4, 6, 93) = (/-2.615E+02, 1.325E+04, 7.069E+04,-4.406E+04/)
  op_Aij4 (1:4, 7, 93) = (/ 6.006E+01, 3.205E+03, 1.724E+05,-3.290E+05/)
  op_Aij4 (1:4, 8, 93) = (/-6.355E+01, 9.093E+03, 4.439E+05,-6.341E+05/)
  op_Aij4 (1:4, 9, 93) = (/-1.103E+02, 2.501E+04, 3.841E+05,-1.044E+06/)
  op_Aij4 (1:4,10, 93) = (/ 5.579E+00, 3.489E+03, 1.354E+06,-1.078E+07/)
  op_Aij4 (1:4,11, 93) = (/ 1.272E+01, 7.422E+03, 9.191E+06,-3.336E+08/)
  op_Aij4 (1:4,12, 93) = (/ 6.330E+00, 1.392E+04, 7.484E+06,-3.064E+08/)
 
  op_Aij4 (1:4, 1, 94) = (/ 9.500E+02, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 94) = (/-1.043E+03, 1.139E+04,-2.517E+03,-6.929E+02/)
  op_Aij4 (1:4, 3, 94) = (/-2.027E+02, 5.876E+03, 4.636E+04,-1.482E+04/)
  op_Aij4 (1:4, 4, 94) = (/-3.181E+02, 1.245E+04, 5.096E+04,-2.427E+04/)
  op_Aij4 (1:4, 5, 94) = (/-2.888E+02, 1.369E+04, 6.306E+04,-3.581E+04/)
  op_Aij4 (1:4, 6, 94) = (/-2.404E+02, 1.162E+04, 8.310E+04,-4.888E+04/)
  op_Aij4 (1:4, 7, 94) = (/ 6.135E+01, 3.329E+03, 1.742E+05,-3.276E+05/)
  op_Aij4 (1:4, 8, 94) = (/-6.367E+01, 9.444E+03, 4.539E+05,-6.824E+05/)
  op_Aij4 (1:4, 9, 94) = (/-1.065E+02, 2.299E+04, 4.461E+05,-1.181E+06/)
  op_Aij4 (1:4,10, 94) = (/-1.248E+01, 6.703E+03, 1.250E+06,-9.411E+06/)
  op_Aij4 (1:4,11, 94) = (/ 1.781E+01, 3.158E+03, 1.075E+07,-4.693E+08/)
  op_Aij4 (1:4,12, 94) = (/ 6.490E+00, 1.422E+04, 8.179E+06,-5.340E+08/)
 
  op_Aij4 (1:4, 1, 95) = (/ 9.600E+02, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 95) = (/-8.786E+02, 1.063E+04,-5.553E+02,-1.750E+03/)
  op_Aij4 (1:4, 3, 95) = (/-2.044E+02, 6.152E+03, 4.844E+04,-1.624E+04/)
  op_Aij4 (1:4, 4, 95) = (/-3.188E+02, 1.325E+04, 5.127E+04,-2.586E+04/)
  op_Aij4 (1:4, 5, 95) = (/-2.907E+02, 1.505E+04, 6.139E+04,-3.761E+04/)
  op_Aij4 (1:4, 6, 95) = (/-2.357E+02, 1.168E+04, 8.876E+04,-5.342E+04/)
  op_Aij4 (1:4, 7, 95) = (/ 1.398E+02, 8.374E+02, 2.097E+05,-4.454E+05/)
  op_Aij4 (1:4, 8, 95) = (/-6.201E+01, 9.427E+03, 4.778E+05,-7.281E+05/)
  op_Aij4 (1:4, 9, 95) = (/-9.199E+01, 1.716E+04, 6.101E+05,-1.350E+06/)
  op_Aij4 (1:4,10, 95) = (/-3.730E+00, 5.425E+03, 1.376E+06,-1.109E+07/)
  op_Aij4 (1:4,11, 95) = (/ 1.966E+01, 2.217E+03, 1.154E+07,-5.298E+08/)
  op_Aij4 (1:4,12, 95) = (/ 6.763E+00, 1.479E+04, 8.647E+06,-6.116E+08/)
 
  op_Aij4 (1:4, 1, 96) = (/ 1.000E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 96) = (/-8.139E+02, 1.040E+04, 4.547E+02,-2.530E+03/)
  op_Aij4 (1:4, 3, 96) = (/-1.652E+02, 4.824E+03, 5.512E+04,-1.585E+04/)
  op_Aij4 (1:4, 4, 96) = (/-3.144E+02, 1.378E+04, 5.160E+04,-2.733E+04/)
  op_Aij4 (1:4, 5, 96) = (/-2.793E+02, 1.411E+04, 6.926E+04,-4.181E+04/)
  op_Aij4 (1:4, 6, 96) = (/-2.092E+02, 1.004E+04, 1.017E+05,-5.684E+04/)
  op_Aij4 (1:4, 7, 96) = (/ 6.769E+01, 3.243E+03, 1.909E+05,-3.844E+05/)
  op_Aij4 (1:4, 8, 96) = (/-5.879E+01, 9.126E+03, 4.981E+05,-7.597E+05/)
  op_Aij4 (1:4, 9, 96) = (/-1.010E+02, 2.212E+04, 5.193E+05,-1.402E+06/)
  op_Aij4 (1:4,10, 96) = (/-1.147E+01, 7.190E+03, 1.327E+06,-1.016E+07/)
  op_Aij4 (1:4,11, 96) = (/ 2.200E+01, 1.083E+03, 1.208E+07,-5.684E+08/)
  op_Aij4 (1:4,12, 96) = (/ 7.017E+00, 1.493E+04, 9.471E+06,-8.527E+08/)
 
  op_Aij4 (1:4, 1, 97) = (/ 1.100E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 97) = (/-1.092E+03, 1.257E+04,-3.090E+03,-6.861E+02/)
  op_Aij4 (1:4, 3, 97) = (/-2.062E+02, 6.715E+03, 5.159E+04,-1.915E+04/)
  op_Aij4 (1:4, 4, 97) = (/-3.108E+02, 1.396E+04, 5.452E+04,-2.964E+04/)
  op_Aij4 (1:4, 5, 97) = (/-2.726E+02, 1.392E+04, 7.484E+04,-4.567E+04/)
  op_Aij4 (1:4, 6, 97) = (/-2.161E+02, 1.101E+04, 1.029E+05,-6.243E+04/)
  op_Aij4 (1:4, 7, 97) = (/ 1.590E+02,-5.494E+01, 2.383E+05,-5.506E+05/)
  op_Aij4 (1:4, 8, 97) = (/-6.035E+01, 9.752E+03, 5.095E+05,-8.299E+05/)
  op_Aij4 (1:4, 9, 97) = (/-1.018E+02, 2.538E+04, 4.681E+05,-1.412E+06/)
  op_Aij4 (1:4,10, 97) = (/-1.308E+01, 7.916E+03, 1.354E+06,-1.049E+07/)
  op_Aij4 (1:4,11, 97) = (/ 2.374E+01, 1.750E+02, 1.285E+07,-6.352E+08/)
  op_Aij4 (1:4,12, 97) = (/ 7.219E+00, 1.592E+04, 9.491E+06,-8.283E+08/)
 
  op_Aij4 (1:4, 1, 98) = (/ 1.220E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 98) = (/-9.355E+02, 1.168E+04,-4.663E+02,-2.379E+03/)
  op_Aij4 (1:4, 3, 98) = (/-2.264E+02, 8.092E+03, 4.926E+04,-2.117E+04/)
  op_Aij4 (1:4, 4, 98) = (/-3.044E+02, 1.361E+04, 6.013E+04,-3.273E+04/)
  op_Aij4 (1:4, 5, 98) = (/-2.641E+02, 1.349E+04, 8.233E+04,-5.011E+04/)
  op_Aij4 (1:4, 6, 98) = (/-2.018E+02, 1.029E+04, 1.132E+05,-6.698E+04/)
  op_Aij4 (1:4, 7, 98) = (/ 7.527E+01, 3.285E+03, 2.079E+05,-4.385E+05/)
  op_Aij4 (1:4, 8, 98) = (/-5.797E+01, 9.571E+03, 5.352E+05,-8.755E+05/)
  op_Aij4 (1:4, 9, 98) = (/-9.148E+01, 1.942E+04, 6.490E+05,-1.673E+06/)
  op_Aij4 (1:4,10, 98) = (/-5.416E+01, 1.706E+04, 8.630E+05,-1.792E+06/)
  op_Aij4 (1:4,11, 98) = (/ 2.771E+01,-2.903E+03, 1.419E+07,-7.456E+08/)
  op_Aij4 (1:4,12, 98) = (/ 7.544E+00, 1.589E+04, 1.052E+07,-1.089E+09/)
 
  op_Aij4 (1:4, 1, 99) = (/ 1.330E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 99) = (/-8.936E+02, 1.145E+04, 1.094E+03,-3.692E+03/)
  op_Aij4 (1:4, 3, 99) = (/-2.152E+02, 7.720E+03, 5.372E+04,-2.276E+04/)
  op_Aij4 (1:4, 4, 99) = (/-2.914E+02, 1.268E+04, 6.828E+04,-3.620E+04/)
  op_Aij4 (1:4, 5, 99) = (/-2.568E+02, 1.341E+04, 8.685E+04,-5.387E+04/)
  op_Aij4 (1:4, 6, 99) = (/-2.104E+02, 1.155E+04, 1.107E+05,-7.279E+04/)
  op_Aij4 (1:4, 7, 99) = (/ 1.088E+02, 1.536E+03, 2.458E+05,-6.005E+05/)
  op_Aij4 (1:4, 8, 99) = (/-5.691E+01, 9.660E+03, 5.556E+05,-9.320E+05/)
  op_Aij4 (1:4, 9, 99) = (/-9.167E+01, 2.050E+04, 6.520E+05,-1.784E+06/)
  op_Aij4 (1:4,10, 99) = (/ 9.687E-01, 5.331E+03, 1.646E+06,-1.518E+07/)
  op_Aij4 (1:4,11, 99) = (/ 3.315E+01,-7.298E+03, 1.587E+07,-8.866E+08/)
  op_Aij4 (1:4,12, 99) = (/ 7.728E+00, 1.675E+04, 1.090E+07,-1.231E+09/)
 
  op_Aij4 (1:4, 1,100) = (/ 1.460E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2,100) = (/-1.082E+03, 1.308E+04,-9.100E+02,-2.846E+03/)
  op_Aij4 (1:4, 3,100) = (/-1.923E+02, 6.709E+03, 6.231E+04,-2.410E+04/)
  op_Aij4 (1:4, 4,100) = (/-2.559E+02, 1.024E+04, 8.602E+04,-3.977E+04/)
  op_Aij4 (1:4, 5,100) = (/-2.466E+02, 1.276E+04, 9.751E+04,-5.925E+04/)
  op_Aij4 (1:4, 6,100) = (/-2.304E+02, 1.518E+04, 9.220E+04,-7.454E+04/)
  op_Aij4 (1:4, 7,100) = (/ 8.675E+01, 2.132E+03, 2.568E+05,-6.538E+05/)
  op_Aij4 (1:4, 8,100) = (/-5.793E+01, 1.018E+04, 5.793E+05,-1.018E+06/)
  op_Aij4 (1:4, 9,100) = (/-8.637E+01, 1.890E+04, 7.445E+05,-1.949E+06/)
  op_Aij4 (1:4,10,100) = (/ 5.556E-01, 5.648E+03, 1.738E+06,-1.662E+07/)
  op_Aij4 (1:4,11,100) = (/ 2.968E+01,-2.026E+03, 1.484E+07,-7.825E+08/)
  op_Aij4 (1:4,12,100) = (/ 8.135E+00, 1.726E+04, 1.171E+07,-1.346E+09/)
!
!
!   ...Ready! 
!
!
  return
end subroutine op_setPEcoeffsAij4atomsZ100
