!!****if* source/physics/materialProperties/Opacity/OpacityMain/Multispecies/method/LowTemp/op_setPEcoeffsAij4atomsZ43
!!
!! NAME
!!
!!  op_setPEcoeffsAij4atomsZ43
!!
!! SYNOPSIS
!!
!!  call op_setPEcoeffsAij4atomsZ43 ()
!!
!! DESCRIPTION
!!
!!  This routine sets the A(i,j,4) expansion coefficients for determining the
!!  photoelectric cross sections according to the F.Biggs and R.Lighthill report:
!!
!!       Analytical Approximations for X-Ray Cross Sections II
!!       Frank Biggs and Ruth Lighthill
!!       Weapons Effects Research Department
!!       Sandia Laboratories, December 1971
!!
!!  The atomic elements set in this routine are in the range: 1 =< Z =< 43.
!!  The data is NOT the updated data from the 1988 update of the report!
!!  This routine can only be called after all the necessary arrays have
!!  been allocated.
!!
!! ARGUMENTS
!!
!!***
subroutine op_setPEcoeffsAij4atomsZ43 ()

  use op_lowTempData,    ONLY : op_Aij4
  use Driver_interface,  ONLY : Driver_abortFlash

  implicit none

  logical :: arrayExists
  logical :: goodShape
!
!
!   ...Check allocation and dimensions of the coefficient array.
!
!
  arrayExists = allocated (op_Aij4)

  if (.not.arrayExists) then
       call Driver_abortFlash ('[op_setPEcoeffsAij4atomsZ43] ERROR: array op_Aij4 not allocated')
  end if

  goodShape =       (size (op_Aij4,1) == 4)   &
              .and. (size (op_Aij4,2) == 13)  &
              .and. (size (op_Aij4,3) == 100)

  if (.not.goodShape) then
       call Driver_abortFlash ('[op_setPEcoeffsAij4atomsZ43] ERROR: array op_Aij4 has bad shape')
  end if
!
!
!   ...Set the expansion coefficient array. Last index denotes the
!      atomic order number Z.
!
!
  op_Aij4 (1:4, 1,  1) = (/ 1.000E-08, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2,  1) = (/-6.383E+01,-6.446E+00, 1.317E+01,-5.045E-02/)
  op_Aij4 (1:4, 3,  1) = (/ 3.051E+00,-7.818E+00, 1.144E+01, 6.959E-02/)
  op_Aij4 (1:4, 4,  1) = (/ 7.636E-02,-9.406E-01, 6.144E+00, 1.425E+00/)
  op_Aij4 (1:4, 5,  1) = (/ 1.180E-03,-8.236E-02, 2.886E+00, 5.534E+00/)
  op_Aij4 (1:4, 6,  1) = (/ 1.620E-05,-5.610E-03, 1.214E+00, 1.761E+01/)
  op_Aij4 (1:4, 7,  1) = (/ 1.034E-06,-4.114E-04, 6.287E-01, 3.927E+01/)
  op_Aij4 (1:4, 8,  1) = (/ 4.200E-07, 4.288E-04, 2.492E-01, 9.574E+01/)
 
  op_Aij4 (1:4, 1,  2) = (/ 1.000E-08, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2,  2) = (/-7.150E+02, 2.769E+02, 4.455E+01,-9.080E-01/)
  op_Aij4 (1:4, 3,  2) = (/ 1.857E+01,-4.079E+01, 8.609E+01,-2.618E+00/)
  op_Aij4 (1:4, 4,  2) = (/ 1.606E+00,-1.726E+01, 8.101E+01,-4.928E+00/)
  op_Aij4 (1:4, 5,  2) = (/ 3.663E-03,-3.782E-01, 2.288E+01, 6.006E+01/)
  op_Aij4 (1:4, 6,  2) = (/ 1.080E-04,-3.106E-02, 1.325E+01, 1.421E+02/)
  op_Aij4 (1:4, 7,  2) = (/ 2.060E-05,-1.074E-02, 1.181E+01, 1.704E+02/)
  op_Aij4 (1:4, 8,  2) = (/ 6.632E-06, 9.453E-03, 2.092E+00, 1.728E+03/)
 
  op_Aij4 (1:4, 1,  3) = (/-2.418E+02, 3.530E+01,-2.699E-01, 5.580E-04/)
  op_Aij4 (1:4, 2,  3) = (/-1.888E+01, 1.898E+01, 2.381E+02,-9.697E+00/)
  op_Aij4 (1:4, 3,  3) = (/ 4.375E+00,-5.094E+01, 3.053E+02,-3.064E+01/)
  op_Aij4 (1:4, 4,  3) = (/ 4.040E-02,-2.930E+00, 1.293E+02, 1.828E+02/)
  op_Aij4 (1:4, 5,  3) = (/ 1.414E-03,-3.913E-01, 7.450E+01, 5.746E+02/)
  op_Aij4 (1:4, 6,  3) = (/ 3.769E-05, 5.136E-03, 3.650E+01, 1.787E+03/)
  op_Aij4 (1:4, 7,  3) = (/ 2.854E-05, 4.028E-02, 8.213E+00, 8.286E+03/)
 
  op_Aij4 (1:4, 1,  4) = (/-1.139E+02, 9.342E+01,-1.314E+00, 5.228E-03/)
  op_Aij4 (1:4, 2,  4) = (/-6.254E+01, 1.968E+02, 5.011E+02,-3.138E+01/)
  op_Aij4 (1:4, 3,  4) = (/ 1.037E+01,-1.280E+02, 8.807E+02,-1.646E+02/)
  op_Aij4 (1:4, 4,  4) = (/ 1.018E-01,-8.264E+00, 4.158E+02, 4.366E+02/)
  op_Aij4 (1:4, 5,  4) = (/ 5.693E-03,-1.595E+00, 2.644E+02, 1.566E+03/)
  op_Aij4 (1:4, 6,  4) = (/ 4.261E-05, 9.215E-02, 9.639E+01, 7.141E+03/)
  op_Aij4 (1:4, 7,  4) = (/ 9.101E-05, 1.272E-01, 2.504E+01, 2.800E+04/)
 
  op_Aij4 (1:4, 1,  5) = (/-3.387E+02, 1.923E+02,-2.742E+00, 1.204E-02/)
  op_Aij4 (1:4, 2,  5) = (/-9.943E+02, 1.816E+03, 3.939E+02,-1.755E+01/)
  op_Aij4 (1:4, 3,  5) = (/ 3.689E+00,-8.834E+01, 1.525E+03,-2.145E+02/)
  op_Aij4 (1:4, 4,  5) = (/ 6.447E-01,-3.953E+01, 1.281E+03, 1.768E+02/)
  op_Aij4 (1:4, 5,  5) = (/ 1.852E-03,-1.356E+00, 5.252E+02, 5.160E+03/)
  op_Aij4 (1:4, 6,  5) = (/ 4.460E-04,-6.960E-02, 3.101E+02, 1.521E+04/)
  op_Aij4 (1:4, 7,  5) = (/ 2.273E-04, 3.032E-01, 1.013E+02, 5.374E+04/)
 
  op_Aij4 (1:4, 1,  6) = (/-6.531E+02, 3.748E+02,-5.790E+00, 2.678E-02/)
  op_Aij4 (1:4, 2,  6) = (/-9.022E+02, 1.760E+03, 1.549E+03,-2.280E+02/)
  op_Aij4 (1:4, 3,  6) = (/-7.363E+00,-1.537E+01, 2.672E+03,-4.482E+02/)
  op_Aij4 (1:4, 4,  6) = (/ 1.640E+00,-9.428E+01, 2.872E+03,-5.583E+02/)
  op_Aij4 (1:4, 5,  6) = (/ 5.621E-04,-2.533E+00, 1.169E+03, 9.909E+03/)
  op_Aij4 (1:4, 6,  6) = (/ 1.183E-03,-2.891E-01, 7.016E+02, 3.359E+04/)
  op_Aij4 (1:4, 7,  6) = (/ 4.997E-04, 6.323E-01, 2.925E+02, 9.317E+04/)
 
  op_Aij4 (1:4, 1,  7) = (/-8.291E+02, 5.716E+02,-1.010E+01, 5.031E-02/)
  op_Aij4 (1:4, 2,  7) = (/-2.338E+03, 5.732E+03,-2.082E+02, 1.482E+02/)
  op_Aij4 (1:4, 3,  7) = (/-4.940E+00,-8.442E+01, 4.620E+03,-1.186E+03/)
  op_Aij4 (1:4, 4,  7) = (/ 2.019E+00,-1.249E+02, 4.609E+03,-9.421E+02/)
  op_Aij4 (1:4, 5,  7) = (/ 1.709E-02,-8.196E+00, 2.345E+03, 1.369E+04/)
  op_Aij4 (1:4, 6,  7) = (/ 1.719E-03,-1.291E-01, 1.192E+03, 6.363E+04/)
  op_Aij4 (1:4, 7,  7) = (/ 9.085E-04, 1.142E+00, 5.293E+02, 1.787E+05/)
 
  op_Aij4 (1:4, 1,  8) = (/-8.968E+02, 8.086E+02,-1.742E+01, 1.025E-01/)
  op_Aij4 (1:4, 2,  8) = (/-7.181E+01, 4.748E+02, 5.542E+03,-1.363E+03/)
  op_Aij4 (1:4, 3,  8) = (/ 2.745E+00,-1.747E+02, 7.159E+03,-2.213E+03/)
  op_Aij4 (1:4, 4,  8) = (/ 3.774E-02,-1.559E+01, 4.045E+03, 1.810E+04/)
  op_Aij4 (1:4, 5,  8) = (/ 4.359E-03,-1.372E+00, 2.202E+03, 9.357E+04/)
  op_Aij4 (1:4, 6,  8) = (/ 1.521E-03, 1.827E+00, 1.132E+03, 1.837E+05/)
 
  op_Aij4 (1:4, 1,  9) = (/-6.041E+02, 8.206E+02,-1.627E+01, 8.640E-02/)
  op_Aij4 (1:4, 2,  9) = (/-8.907E+01, 6.850E+02, 7.217E+03,-2.022E+03/)
  op_Aij4 (1:4, 3,  9) = (/ 2.813E+00,-2.020E+02, 9.903E+03,-4.454E+03/)
  op_Aij4 (1:4, 4,  9) = (/ 6.097E-02,-2.435E+01, 6.099E+03, 2.258E+04/)
  op_Aij4 (1:4, 5,  9) = (/ 5.064E-03,-8.464E-01, 3.076E+03, 1.458E+05/)
  op_Aij4 (1:4, 6,  9) = (/ 2.262E-03, 2.631E+00, 1.701E+03, 3.144E+05/)
 
  op_Aij4 (1:4, 1, 10) = (/ 0.000E+00, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 10) = (/-1.183E+03, 1.545E+03,-5.393E+01, 5.024E-01/)
  op_Aij4 (1:4, 3, 10) = (/-8.680E+01, 7.434E+02, 1.077E+04,-3.822E+03/)
  op_Aij4 (1:4, 4, 10) = (/ 3.167E+00,-2.491E+02, 1.439E+04,-8.187E+03/)
  op_Aij4 (1:4, 5, 10) = (/ 1.288E-01,-4.471E+01, 9.862E+03, 2.497E+04/)
  op_Aij4 (1:4, 6, 10) = (/ 5.202E-03, 7.337E-02, 4.614E+03, 2.256E+05/)
  op_Aij4 (1:4, 7, 10) = (/ 3.536E-03, 4.790E+00, 1.147E+03, 9.882E+05/)
 
  op_Aij4 (1:4, 1, 11) = (/ 1.910E+01, 1.575E+00,-2.367E-02, 8.190E-05/)
  op_Aij4 (1:4, 2, 11) = (/-9.125E+02, 1.627E+03,-6.064E+01, 4.588E-01/)
  op_Aij4 (1:4, 3, 11) = (/ 1.732E+00,-1.853E+02, 1.712E+04,-8.415E+03/)
  op_Aij4 (1:4, 4, 11) = (/ 2.726E-01,-8.218E+01, 1.475E+04, 9.467E+03/)
  op_Aij4 (1:4, 5, 11) = (/ 2.083E-03, 4.548E+00, 5.521E+03, 3.358E+05/)
  op_Aij4 (1:4, 6, 11) = (/ 4.900E-03, 6.789E+00, 1.167E+03, 1.600E+06/)
 
  op_Aij4 (1:4, 1, 12) = (/ 3.150E+02,-6.639E+00, 3.825E-02,-2.656E-05/)
  op_Aij4 (1:4, 2, 12) = (/-8.778E+02, 1.897E+03,-2.962E+01,-2.277E+00/)
  op_Aij4 (1:4, 3, 12) = (/-6.648E-01,-1.084E+02, 2.219E+04,-1.241E+04/)
  op_Aij4 (1:4, 4, 12) = (/ 4.080E-01,-1.205E+02, 2.139E+04,-1.143E+02/)
  op_Aij4 (1:4, 5, 12) = (/ 8.338E-03, 2.236E+00, 8.819E+03, 4.284E+05/)
  op_Aij4 (1:4, 6, 12) = (/ 7.016E-03, 9.228E+00, 2.818E+03, 1.846E+06/)
 
  op_Aij4 (1:4, 1, 13) = (/ 4.303E+02,-8.931E+00, 4.919E-02, 6.325E-05/)
  op_Aij4 (1:4, 2, 13) = (/-4.530E+02, 1.420E+03, 2.528E+02,-2.366E+01/)
  op_Aij4 (1:4, 3, 13) = (/-3.674E+00,-1.622E+01, 2.732E+04,-1.752E+04/)
  op_Aij4 (1:4, 4, 13) = (/ 4.158E-01,-1.351E+02, 2.716E+04, 3.723E+02/)
  op_Aij4 (1:4, 5, 13) = (/ 1.125E-02, 2.747E+00, 1.174E+04, 5.695E+05/)
  op_Aij4 (1:4, 6, 13) = (/ 9.245E-03, 1.188E+01, 4.104E+03, 2.352E+06/)
 
  op_Aij4 (1:4, 1, 14) = (/ 8.816E+02,-2.314E+01, 1.090E-01, 1.725E-03/)
  op_Aij4 (1:4, 2, 14) = (/-3.946E+02, 1.498E+03, 4.883E+02,-5.508E+01/)
  op_Aij4 (1:4, 3, 14) = (/-4.056E+00,-1.089E+01, 3.544E+04,-2.595E+04/)
  op_Aij4 (1:4, 4, 14) = (/ 4.945E-01,-1.682E+02, 3.627E+04,-1.605E+04/)
  op_Aij4 (1:4, 5, 14) = (/ 2.716E-02,-5.428E+00, 1.775E+04, 6.765E+05/)
  op_Aij4 (1:4, 6, 14) = (/ 1.261E-02, 1.557E+01, 7.662E+03, 2.288E+06/)
 
  op_Aij4 (1:4, 1, 15) = (/ 9.405E+02,-9.183E+00,-7.379E-01, 1.434E-02/)
  op_Aij4 (1:4, 2, 15) = (/-2.678E+02, 1.287E+03, 9.558E+02,-1.269E+02/)
  op_Aij4 (1:4, 3, 15) = (/-6.098E+00, 7.712E+01, 4.196E+04,-3.437E+04/)
  op_Aij4 (1:4, 4, 15) = (/ 5.710E-01,-2.001E+02, 4.505E+04,-3.853E+04/)
  op_Aij4 (1:4, 5, 15) = (/ 3.613E-02,-8.426E+00, 2.277E+04, 8.080E+05/)
  op_Aij4 (1:4, 6, 15) = (/ 1.582E-02, 1.847E+01, 1.110E+04, 2.456E+06/)
 
  op_Aij4 (1:4, 1, 16) = (/ 1.850E+03,-7.648E+01, 4.856E-01, 1.655E-02/)
  op_Aij4 (1:4, 2, 16) = (/-3.580E+02, 1.751E+03, 1.186E+03,-1.847E+02/)
  op_Aij4 (1:4, 3, 16) = (/-1.021E+01, 2.346E+02, 5.164E+04,-4.707E+04/)
  op_Aij4 (1:4, 4, 16) = (/ 7.467E-01,-2.534E+02, 5.800E+04,-6.691E+04/)
  op_Aij4 (1:4, 5, 16) = (/ 5.442E-02,-1.719E+01, 3.154E+04, 9.102E+05/)
  op_Aij4 (1:4, 6, 16) = (/ 2.069E-02, 2.375E+01, 1.589E+04, 2.713E+06/)
 
  op_Aij4 (1:4, 1, 17) = (/ 1.558E+03,-1.641E+01,-1.325E+00, 3.215E-02/)
  op_Aij4 (1:4, 2, 17) = (/-2.174E+02, 1.397E+03, 1.833E+03,-3.166E+02/)
  op_Aij4 (1:4, 3, 17) = (/-1.192E+01, 3.253E+02, 5.927E+04,-5.980E+04/)
  op_Aij4 (1:4, 4, 17) = (/ 8.789E-01,-2.920E+02, 6.860E+04,-1.018E+05/)
  op_Aij4 (1:4, 5, 17) = (/ 6.556E-02,-2.175E+01, 3.895E+04, 9.737E+05/)
  op_Aij4 (1:4, 6, 17) = (/ 2.483E-02, 2.697E+01, 2.078E+04, 2.972E+06/)
 
  op_Aij4 (1:4, 1, 18) = (/ 2.556E+03,-1.624E+02, 3.134E+00, 2.626E-02/)
  op_Aij4 (1:4, 2, 18) = (/-1.949E+02, 1.368E+03, 2.385E+03,-4.739E+02/)
  op_Aij4 (1:4, 3, 18) = (/-1.446E+01, 4.359E+02, 6.578E+04,-7.284E+04/)
  op_Aij4 (1:4, 4, 18) = (/ 8.786E-01,-3.001E+02, 7.682E+04,-1.219E+05/)
  op_Aij4 (1:4, 5, 18) = (/ 8.835E-02,-3.637E+01, 4.777E+04, 9.352E+05/)
  op_Aij4 (1:4, 6, 18) = (/ 2.874E-02, 3.165E+01, 2.446E+04, 3.037E+06/)
 
  op_Aij4 (1:4, 1, 19) = (/ 5.292E+00, 3.717E-01,-4.740E-03, 1.436E-05/)
  op_Aij4 (1:4, 2, 19) = (/-1.733E+02, 5.359E+02,-3.738E+01, 7.107E-01/)
  op_Aij4 (1:4, 3, 19) = (/-1.595E+02, 1.345E+03, 3.549E+03,-7.756E+02/)
  op_Aij4 (1:4, 4, 19) = (/-2.113E+01, 7.161E+02, 8.183E+04,-9.859E+04/)
  op_Aij4 (1:4, 5, 19) = (/ 1.048E+00,-3.644E+02, 9.844E+04,-1.760E+05/)
  op_Aij4 (1:4, 6, 19) = (/ 1.040E-01,-3.553E+01, 6.096E+04, 1.226E+06/)
  op_Aij4 (1:4, 7, 19) = (/ 3.773E-02, 4.007E+01, 3.507E+04, 3.555E+06/)
 
  op_Aij4 (1:4, 1, 20) = (/ 1.442E+02,-3.204E+00, 2.579E-02,-8.084E-05/)
  op_Aij4 (1:4, 2, 20) = (/ 1.440E+01, 6.073E+02,-4.745E+01, 9.859E-01/)
  op_Aij4 (1:4, 3, 20) = (/ 7.171E+01, 6.790E+01, 6.646E+03,-1.903E+03/)
  op_Aij4 (1:4, 4, 20) = (/-3.015E+01, 1.141E+03, 9.459E+04,-1.195E+05/)
  op_Aij4 (1:4, 5, 20) = (/ 1.408E+00,-4.553E+02, 1.206E+05,-2.531E+05/)
  op_Aij4 (1:4, 6, 20) = (/ 1.154E-01,-3.465E+01, 7.523E+04, 1.368E+06/)
  op_Aij4 (1:4, 7, 20) = (/ 4.664E-02, 5.252E+01, 3.960E+04, 5.980E+06/)
 
  op_Aij4 (1:4, 1, 21) = (/ 1.088E+03, 5.062E+00,-2.564E-01, 1.298E-03/)
  op_Aij4 (1:4, 2, 21) = (/ 1.897E+02, 4.984E+02,-3.352E+01, 6.726E-01/)
  op_Aij4 (1:4, 3, 21) = (/-1.507E+02, 1.501E+03, 5.139E+03,-1.442E+03/)
  op_Aij4 (1:4, 4, 21) = (/ 4.615E-01,-2.614E+02, 1.217E+05,-2.057E+05/)
  op_Aij4 (1:4, 5, 21) = (/ 1.509E-01,-6.913E+01, 9.253E+04, 1.096E+06/)
  op_Aij4 (1:4, 6, 21) = (/ 5.204E-02, 6.443E+01, 3.314E+04, 9.765E+06/)
 
  op_Aij4 (1:4, 1, 22) = (/ 1.581E+03, 2.413E+01,-7.791E-01, 4.080E-03/)
  op_Aij4 (1:4, 2, 22) = (/-4.626E+01, 8.678E+02, 7.308E+03,-2.383E+03/)
  op_Aij4 (1:4, 3, 22) = (/ 2.074E-01,-2.292E+02, 1.371E+05,-2.489E+05/)
  op_Aij4 (1:4, 4, 22) = (/ 1.972E-01,-9.550E+01, 1.106E+05, 1.067E+06/)
  op_Aij4 (1:4, 5, 22) = (/ 6.044E-02, 7.449E+01, 4.320E+04, 9.379E+06/)
 
  op_Aij4 (1:4, 1, 23) = (/ 1.887E+03, 1.983E+01,-7.427E-01, 3.731E-03/)
  op_Aij4 (1:4, 2, 23) = (/-7.196E+01, 1.132E+03, 8.004E+03,-2.803E+03/)
  op_Aij4 (1:4, 3, 23) = (/ 3.940E-01,-2.733E+02, 1.564E+05,-3.184E+05/)
  op_Aij4 (1:4, 4, 23) = (/ 2.007E-01,-8.911E+01, 1.254E+05, 1.137E+06/)
  op_Aij4 (1:4, 5, 23) = (/ 6.962E-02, 7.762E+01, 5.692E+04, 1.006E+07/)
 
  op_Aij4 (1:4, 1, 24) = (/ 1.755E+03, 9.204E+01,-1.973E+00, 9.457E-03/)
  op_Aij4 (1:4, 2, 24) = (/ 3.932E+01, 1.537E+02, 1.228E+04,-4.984E+03/)
  op_Aij4 (1:4, 3, 24) = (/ 2.740E-01,-2.665E+02, 1.818E+05,-3.997E+05/)
  op_Aij4 (1:4, 4, 24) = (/ 2.727E-01,-1.288E+02, 1.543E+05, 9.755E+05/)
  op_Aij4 (1:4, 5, 24) = (/ 8.278E-02, 9.017E+01, 7.781E+04, 8.227E+06/)
 
  op_Aij4 (1:4, 1, 25) = (/ 5.885E+03,-1.396E+02, 1.353E+00,-4.829E-03/)
  op_Aij4 (1:4, 2, 25) = (/-5.021E+02, 1.265E+03,-8.805E+01, 1.272E+00/)
  op_Aij4 (1:4, 3, 25) = (/ 5.200E+01, 1.957E+01, 1.447E+04,-6.930E+03/)
  op_Aij4 (1:4, 4, 25) = (/ 2.930E-01,-2.703E+02, 2.034E+05,-4.886E+05/)
  op_Aij4 (1:4, 5, 25) = (/ 2.940E-01,-1.342E+02, 1.761E+05, 8.747E+05/)
  op_Aij4 (1:4, 6, 25) = (/ 9.427E-02, 1.057E+02, 8.606E+04, 1.091E+07/)
 
  op_Aij4 (1:4, 1, 26) = (/ 5.852E+03,-1.336E+02, 1.336E+00,-4.843E-03/)
  op_Aij4 (1:4, 2, 26) = (/-1.412E+02, 1.086E+03,-6.278E+01, 2.043E-01/)
  op_Aij4 (1:4, 3, 26) = (/-8.128E+00, 6.550E+02, 1.511E+04,-7.079E+03/)
  op_Aij4 (1:4, 4, 26) = (/ 2.151E-01,-2.680E+02, 2.341E+05,-6.063E+05/)
  op_Aij4 (1:4, 5, 26) = (/ 3.511E-01,-1.633E+02, 2.090E+05, 7.131E+05/)
  op_Aij4 (1:4, 6, 26) = (/ 1.107E-01, 1.252E+02, 1.008E+05, 1.273E+07/)
 
  op_Aij4 (1:4, 1, 27) = (/ 5.153E+03,-3.891E+01,-4.490E-01, 4.045E-03/)
  op_Aij4 (1:4, 2, 27) = (/-1.957E+02, 1.422E+03,-1.321E+02, 3.914E+00/)
  op_Aij4 (1:4, 3, 27) = (/ 2.092E+01, 3.311E+02, 1.820E+04,-9.303E+03/)
  op_Aij4 (1:4, 4, 27) = (/ 4.947E-02,-2.478E+02, 2.586E+05,-7.185E+05/)
  op_Aij4 (1:4, 5, 27) = (/ 3.588E-01,-1.492E+02, 2.296E+05, 8.856E+05/)
  op_Aij4 (1:4, 6, 27) = (/ 1.244E-01, 1.427E+02, 1.134E+05, 1.528E+07/)
 
  op_Aij4 (1:4, 1, 28) = (/ 6.887E+03,-1.547E+02, 1.332E+00,-4.244E-03/)
  op_Aij4 (1:4, 2, 28) = (/ 9.185E+01, 1.073E+03,-4.045E+01,-1.313E+00/)
  op_Aij4 (1:4, 3, 28) = (/ 0.000E+00, 6.600E+03, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 4, 28) = (/ 0.000E+00, 9.810E+03, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 5, 28) = (/-1.175E+02, 2.171E+03, 1.514E+04,-6.042E+03/)
  op_Aij4 (1:4, 6, 28) = (/ 1.685E-02,-2.522E+02, 3.005E+05,-9.000E+05/)
  op_Aij4 (1:4, 7, 28) = (/ 4.681E-01,-2.085E+02, 2.782E+05, 4.399E+05/)
  op_Aij4 (1:4, 8, 28) = (/ 1.470E-01, 1.687E+02, 1.419E+05, 1.446E+07/)
 
  op_Aij4 (1:4, 1, 29) = (/ 3.293E+03,-2.208E+01,-3.511E-01, 2.914E-03/)
  op_Aij4 (1:4, 2, 29) = (/-3.217E+02, 1.834E+03,-2.631E+02, 1.133E+01/)
  op_Aij4 (1:4, 3, 29) = (/-5.089E+02, 4.139E+03, 7.468E+03,-6.981E+02/)
  op_Aij4 (1:4, 4, 29) = (/-9.173E+01, 1.862E+03, 1.777E+04,-8.202E+03/)
  op_Aij4 (1:4, 5, 29) = (/ 7.440E-03,-2.371E+02, 3.202E+05,-1.028E+06/)
  op_Aij4 (1:4, 6, 29) = (/ 4.729E-01,-2.055E+02, 3.000E+05, 2.182E+05/)
  op_Aij4 (1:4, 7, 29) = (/ 1.589E-01, 1.848E+02, 1.452E+05, 1.929E+07/)
 
  op_Aij4 (1:4, 1, 30) = (/ 7.941E+03,-2.347E+02, 2.501E+00,-8.979E-03/)
  op_Aij4 (1:4, 2, 30) = (/-1.534E+02, 1.829E+03,-1.674E+02, 4.439E+00/)
  op_Aij4 (1:4, 3, 30) = (/-2.807E+04, 1.091E+05,-1.511E+05, 7.897E+04/)
  op_Aij4 (1:4, 4, 30) = (/-3.057E+03, 1.385E+04, 3.780E+02,-2.163E+02/)
  op_Aij4 (1:4, 5, 30) = (/-8.018E+01, 1.794E+03, 2.095E+04,-1.070E+04/)
  op_Aij4 (1:4, 6, 30) = (/-1.158E+00,-3.871E+01, 3.499E+05,-1.154E+06/)
  op_Aij4 (1:4, 7, 30) = (/ 4.820E-01,-1.832E+02, 3.296E+05, 6.802E+05/)
  op_Aij4 (1:4, 8, 30) = (/ 1.796E-01, 2.204E+02, 1.529E+05, 2.596E+07/)
 
  op_Aij4 (1:4, 1, 31) = (/ 0.000E+00, 0.000E+00, 0.000E+00, 3.400E-04/)
  op_Aij4 (1:4, 2, 31) = (/ 1.091E+04,-5.719E+02, 1.060E+01,-6.716E-02/)
  op_Aij4 (1:4, 3, 31) = (/-6.776E+02, 2.582E+03,-2.726E+02, 8.301E+00/)
  op_Aij4 (1:4, 4, 31) = (/-1.457E+04, 6.363E+04,-1.011E+05, 6.416E+04/)
  op_Aij4 (1:4, 5, 31) = (/ 3.470E+03,-8.674E+03, 2.548E+04,-8.411E+03/)
  op_Aij4 (1:4, 6, 31) = (/-9.431E+01, 2.078E+03, 2.172E+04,-1.101E+04/)
  op_Aij4 (1:4, 7, 31) = (/-1.253E+00,-2.897E+01, 3.783E+05,-1.357E+06/)
  op_Aij4 (1:4, 8, 31) = (/ 5.591E-01,-2.217E+02, 3.625E+05, 3.392E+05/)
  op_Aij4 (1:4, 9, 31) = (/ 1.949E-01, 2.371E+02, 1.767E+05, 2.400E+07/)
 
  op_Aij4 (1:4, 1, 32) = (/ 0.000E+00, 0.000E+00, 0.000E+00, 1.800E-03/)
  op_Aij4 (1:4, 2, 32) = (/ 1.942E+04,-1.740E+03, 5.357E+01,-5.554E-01/)
  op_Aij4 (1:4, 3, 32) = (/-9.082E+02, 3.097E+03,-3.039E+02, 7.152E+00/)
  op_Aij4 (1:4, 4, 32) = (/-7.402E+03, 3.636E+04,-6.670E+04, 5.346E+04/)
  op_Aij4 (1:4, 5, 32) = (/-3.400E+02, 3.232E+03, 1.163E+04,-1.089E+03/)
  op_Aij4 (1:4, 6, 32) = (/-5.940E+01, 1.646E+03, 2.621E+04,-1.507E+04/)
  op_Aij4 (1:4, 7, 32) = (/-2.025E+00, 1.283E+02, 4.069E+05,-1.510E+06/)
  op_Aij4 (1:4, 8, 32) = (/ 6.478E-01,-2.603E+02, 4.045E+05,-5.153E+04/)
  op_Aij4 (1:4, 9, 32) = (/ 2.155E-01, 2.609E+02, 2.074E+05, 2.220E+07/)
 
  op_Aij4 (1:4, 1, 33) = (/ 1.642E+02,-2.624E+01, 1.048E+00,-5.116E-03/)
  op_Aij4 (1:4, 2, 33) = (/ 1.858E+04,-2.363E+03, 1.082E+02,-1.737E+00/)
  op_Aij4 (1:4, 3, 33) = (/-8.391E+02, 3.143E+03,-4.102E+02, 1.448E+01/)
  op_Aij4 (1:4, 4, 33) = (/-4.525E+03, 2.468E+04,-5.133E+04, 5.102E+04/)
  op_Aij4 (1:4, 5, 33) = (/-3.858E+02, 1.013E+04, 3.603E+03,-7.657E+02/)
  op_Aij4 (1:4, 6, 33) = (/-5.339E+01, 1.649E+03, 2.950E+04,-1.800E+04/)
  op_Aij4 (1:4, 7, 33) = (/-1.655E+00, 8.588E+01, 4.504E+05,-1.795E+06/)
  op_Aij4 (1:4, 8, 33) = (/ 7.880E-01,-3.490E+02, 4.641E+05,-1.258E+06/)
  op_Aij4 (1:4, 9, 33) = (/ 2.393E-01, 2.906E+02, 2.360E+05, 2.145E+07/)
 
  op_Aij4 (1:4, 1, 34) = (/ 3.589E+02,-4.858E+01, 1.838E+00,-8.229E-03/)
  op_Aij4 (1:4, 2, 34) = (/ 4.647E+04,-7.676E+03, 4.318E+02,-8.207E+00/)
  op_Aij4 (1:4, 3, 34) = (/-9.469E+02, 3.639E+03,-4.086E+02, 1.010E+01/)
  op_Aij4 (1:4, 4, 34) = (/-6.637E+02, 4.194E+03,-1.104E+04, 2.806E+04/)
  op_Aij4 (1:4, 5, 34) = (/-4.265E+02, 9.660E+03, 5.343E+03,-1.174E+03/)
  op_Aij4 (1:4, 6, 34) = (/-3.968E+01, 1.459E+03, 3.327E+04,-2.228E+04/)
  op_Aij4 (1:4, 7, 34) = (/-3.461E+00, 4.401E+02, 4.690E+05,-1.943E+06/)
  op_Aij4 (1:4, 8, 34) = (/ 8.564E-01,-3.723E+02, 5.020E+05,-1.430E+06/)
  op_Aij4 (1:4, 9, 34) = (/ 2.590E-01, 3.237E+02, 2.541E+05, 2.321E+07/)
 
  op_Aij4 (1:4, 1, 35) = (/ 7.741E+02,-7.717E+01, 2.608E+00,-1.018E-02/)
  op_Aij4 (1:4, 2, 35) = (/ 3.275E+02, 2.964E+03,-4.214E+02, 1.494E+01/)
  op_Aij4 (1:4, 3, 35) = (/ 4.049E+03,-1.310E+04, 1.813E+04,-6.454E+03/)
  op_Aij4 (1:4, 4, 35) = (/ 1.461E+02,-1.050E+03, 3.457E+03, 2.000E+04/)
  op_Aij4 (1:4, 5, 35) = (/-4.318E+02, 1.007E+04, 6.548E+03,-1.535E+03/)
  op_Aij4 (1:4, 6, 35) = (/-4.033E+01, 1.607E+03, 3.681E+04,-2.478E+04/)
  op_Aij4 (1:4, 7, 35) = (/-3.955E+00, 5.482E+02, 5.202E+05,-2.262E+06/)
  op_Aij4 (1:4, 8, 35) = (/ 9.718E-01,-4.209E+02, 5.662E+05,-2.099E+06/)
  op_Aij4 (1:4, 9, 35) = (/ 2.907E-01, 3.653E+02, 2.908E+05, 2.417E+07/)
 
  op_Aij4 (1:4, 1, 36) = (/ 5.868E+01, 5.694E+01,-3.164E+00, 5.185E-02/)
  op_Aij4 (1:4, 2, 36) = (/-1.048E+03, 4.558E+03,-7.909E+02, 3.526E+01/)
  op_Aij4 (1:4, 3, 36) = (/-1.462E+03, 6.037E+03,-2.364E+03, 5.585E+02/)
  op_Aij4 (1:4, 4, 36) = (/ 3.611E+02,-2.974E+03, 1.308E+04, 1.268E+04/)
  op_Aij4 (1:4, 5, 36) = (/-4.163E+02, 1.017E+04, 7.535E+03,-1.891E+03/)
  op_Aij4 (1:4, 6, 36) = (/-9.924E-01, 8.455E+02, 4.446E+04,-3.490E+04/)
  op_Aij4 (1:4, 7, 36) = (/-3.076E+00, 4.257E+02, 5.613E+05,-2.587E+06/)
  op_Aij4 (1:4, 8, 36) = (/ 1.050E+00,-4.435E+02, 6.114E+05,-3.028E+06/)
  op_Aij4 (1:4, 9, 36) = (/ 3.136E-01, 3.952E+02, 3.252E+05, 2.248E+07/)
 
  op_Aij4 (1:4, 1, 37) = (/ 8.623E-01, 7.942E-02,-1.525E-03, 6.932E-06/)
  op_Aij4 (1:4, 2, 37) = (/ 1.558E+02, 9.161E+00,-1.414E+00, 7.049E-02/)
  op_Aij4 (1:4, 3, 37) = (/-1.446E+03, 5.719E+03,-1.213E+03, 6.655E+01/)
  op_Aij4 (1:4, 4, 37) = (/ 3.524E+02,-3.194E+03, 1.657E+04, 1.224E+04/)
  op_Aij4 (1:4, 5, 37) = (/-4.025E+02, 1.067E+04, 8.396E+03,-2.275E+03/)
  op_Aij4 (1:4, 6, 37) = (/ 5.234E+00, 7.494E+02, 5.033E+04,-4.276E+04/)
  op_Aij4 (1:4, 7, 37) = (/-2.207E+00, 2.653E+02, 6.269E+05,-3.163E+06/)
  op_Aij4 (1:4, 8, 37) = (/ 1.101E+00,-4.280E+02, 6.664E+05,-3.481E+06/)
  op_Aij4 (1:4, 9, 37) = (/ 3.467E-01, 4.515E+02, 3.525E+05, 2.783E+07/)
 
  op_Aij4 (1:4, 1, 38) = (/ 4.666E+01,-1.282E+00, 1.181E-02,-3.649E-05/)
  op_Aij4 (1:4, 2, 38) = (/ 7.687E+02,-6.434E+01, 1.634E+00, 3.457E-02/)
  op_Aij4 (1:4, 3, 38) = (/-2.899E+03, 8.153E+03,-1.741E+03, 9.548E+01/)
  op_Aij4 (1:4, 4, 38) = (/ 1.053E+03,-2.725E+03, 8.076E+03,-2.820E+03/)
  op_Aij4 (1:4, 5, 38) = (/ 2.746E+02,-2.830E+03, 2.149E+04, 7.655E+03/)
  op_Aij4 (1:4, 6, 38) = (/-4.181E+02, 1.025E+04, 1.159E+04,-3.199E+03/)
  op_Aij4 (1:4, 7, 38) = (/-4.367E+01, 1.966E+03, 4.835E+04,-3.788E+04/)
  op_Aij4 (1:4, 8, 38) = (/-5.177E-01,-2.546E+01, 6.941E+05,-3.733E+06/)
  op_Aij4 (1:4, 9, 38) = (/ 1.296E+00,-5.192E+02, 7.384E+05,-5.043E+06/)
  op_Aij4 (1:4,10, 38) = (/ 3.800E-01, 4.950E+02, 4.111E+05, 1.955E+07/)
 
  op_Aij4 (1:4, 1, 39) = (/ 1.646E+03,-1.524E+01,-3.033E-02, 4.574E-04/)
  op_Aij4 (1:4, 2, 39) = (/ 4.222E+01, 7.685E+01,-4.225E+00, 8.261E-02/)
  op_Aij4 (1:4, 3, 39) = (/-2.128E+03, 8.026E+03,-2.124E+03, 1.548E+02/)
  op_Aij4 (1:4, 4, 39) = (/ 3.585E+02,-4.223E+02, 6.620E+03,-2.706E+03/)
  op_Aij4 (1:4, 5, 39) = (/-8.813E+01, 1.184E+03, 2.198E+04,-1.440E+03/)
  op_Aij4 (1:4, 6, 39) = (/-4.141E+02, 1.043E+04, 1.385E+04,-4.020E+03/)
  op_Aij4 (1:4, 7, 39) = (/-1.282E+01, 1.204E+03, 6.028E+04,-5.752E+04/)
  op_Aij4 (1:4, 8, 39) = (/ 4.265E-01,-1.646E+02, 7.684E+05,-4.421E+06/)
  op_Aij4 (1:4, 9, 39) = (/ 1.334E+00,-4.917E+02, 8.066E+05,-5.879E+06/)
  op_Aij4 (1:4,10, 39) = (/ 4.194E-01, 5.651E+02, 4.353E+05, 2.981E+07/)
 
  op_Aij4 (1:4, 1, 40) = (/ 1.910E+02, 3.638E+01,-4.958E-01, 1.598E-03/)
  op_Aij4 (1:4, 2, 40) = (/-8.578E+03, 1.742E+04,-5.382E+03, 4.570E+02/)
  op_Aij4 (1:4, 3, 40) = (/-3.681E+02, 3.235E+03, 1.544E+03,-2.121E+02/)
  op_Aij4 (1:4, 4, 40) = (/-2.164E+02, 3.512E+03, 2.053E+04,-3.382E+03/)
  op_Aij4 (1:4, 5, 40) = (/-3.996E+02, 1.064E+04, 1.548E+04,-4.776E+03/)
  op_Aij4 (1:4, 6, 40) = (/-3.011E+01, 1.794E+03, 6.153E+04,-5.699E+04/)
  op_Aij4 (1:4, 7, 40) = (/-5.900E-01, 1.945E+02, 8.031E+05,-4.579E+06/)
  op_Aij4 (1:4, 8, 40) = (/ 1.474E+00,-5.530E+02, 8.907E+05,-7.927E+06/)
  op_Aij4 (1:4, 9, 40) = (/ 4.562E-01, 6.248E+02, 4.760E+05, 3.216E+07/)
 
  op_Aij4 (1:4, 1, 41) = (/ 5.478E+02,-1.855E+01, 1.190E+00,-1.007E-02/)
  op_Aij4 (1:4, 2, 41) = (/-7.034E+03, 1.685E+04,-5.527E+03, 5.032E+02/)
  op_Aij4 (1:4, 3, 41) = (/ 7.252E+02,-1.862E+03, 9.507E+03,-3.524E+03/)
  op_Aij4 (1:4, 4, 41) = (/-2.981E+02, 6.188E+03, 1.735E+04,-4.420E+03/)
  op_Aij4 (1:4, 5, 41) = (/-3.923E+02, 1.102E+04, 1.751E+04,-5.733E+03/)
  op_Aij4 (1:4, 6, 41) = (/-1.658E+01, 1.435E+03, 7.050E+04,-7.037E+04/)
  op_Aij4 (1:4, 7, 41) = (/-2.292E+00, 5.516E+02, 8.555E+05,-5.018E+06/)
  op_Aij4 (1:4, 8, 41) = (/ 1.599E+00,-5.764E+02, 9.643E+05,-8.517E+06/)
  op_Aij4 (1:4, 9, 41) = (/ 4.986E-01, 7.224E+02, 4.910E+05, 4.104E+07/)
 
  op_Aij4 (1:4, 1, 42) = (/ 5.429E+02, 2.506E+00, 5.446E-01,-5.466E-03/)
  op_Aij4 (1:4, 2, 42) = (/-6.303E+03, 1.614E+04,-5.496E+03, 5.143E+02/)
  op_Aij4 (1:4, 3, 42) = (/-1.114E+02, 1.895E+03, 5.405E+03,-2.261E+03/)
  op_Aij4 (1:4, 4, 42) = (/-3.027E+02, 7.576E+03, 1.570E+04,-4.795E+03/)
  op_Aij4 (1:4, 5, 42) = (/-3.784E+02, 1.099E+04, 1.983E+04,-6.801E+03/)
  op_Aij4 (1:4, 6, 42) = (/-1.840E+01, 1.552E+03, 7.635E+04,-8.181E+04/)
  op_Aij4 (1:4, 7, 42) = (/-2.560E+00, 7.317E+02, 9.005E+05,-5.349E+06/)
  op_Aij4 (1:4, 8, 42) = (/ 1.540E+00,-4.834E+02, 1.021E+06,-9.300E+06/)
  op_Aij4 (1:4, 9, 42) = (/ 5.359E-01, 7.662E+02, 5.243E+05, 5.200E+07/)
 
  op_Aij4 (1:4, 1, 43) = (/ 2.495E+02, 6.721E+01,-5.844E-01,-2.482E-04/)
  op_Aij4 (1:4, 2, 43) = (/-6.741E+03, 1.690E+04,-5.576E+03, 5.102E+02/)
  op_Aij4 (1:4, 3, 43) = (/-3.012E+02, 3.094E+03, 4.154E+03,-1.733E+03/)
  op_Aij4 (1:4, 4, 43) = (/-2.871E+02, 7.198E+03, 1.867E+04,-5.767E+03/)
  op_Aij4 (1:4, 5, 43) = (/-1.101E+02, 1.635E+04, 3.183E+03,-1.491E+03/)
  op_Aij4 (1:4, 6, 43) = (/-2.936E+01, 1.924E+03, 7.955E+04,-8.669E+04/)
  op_Aij4 (1:4, 7, 43) = (/ 5.721E-01, 7.164E+01, 1.004E+06,-6.678E+06/)
  op_Aij4 (1:4, 8, 43) = (/ 1.769E+00,-5.691E+02, 1.097E+06,-1.069E+07/)
  op_Aij4 (1:4, 9, 43) = (/ 5.748E-01, 8.119E+02, 6.113E+05, 3.605E+07/)
!
!
!   ...Ready! 
!
!
  return
end subroutine op_setPEcoeffsAij4atomsZ43
