!!****if* source/physics/materialProperties/Opacity/OpacityMain/Multispecies/method/LowTemp/op_setPEcoeffsAij4atomsZ73
!!
!! NAME
!!
!!  op_setPEcoeffsAij4atomsZ73
!!
!! SYNOPSIS
!!
!!  call op_setPEcoeffsAij4atomsZ73 ()
!!
!! DESCRIPTION
!!
!!  This routine sets the A(i,j,4) expansion coefficients for determining the
!!  photoelectric cross sections according to the F.Biggs and R.Lighthill report:
!!
!!       Analytical Approximations for X-Ray Cross Sections II
!!       Frank Biggs and Ruth Lighthill
!!       Weapons Effects Research Department
!!       Sandia Laboratories, December 1971
!!
!!  The atomic elements set in this routine are in the range: 44 =< Z =< 73.
!!  The data is NOT the updated data from the 1988 update of the report!
!!  This routine can only be called after all the necessary arrays have
!!  been allocated.
!!
!! ARGUMENTS
!!
!!***
subroutine op_setPEcoeffsAij4atomsZ73 ()

  use op_lowTempData,   ONLY : op_Aij4
  use Driver_interface, ONLY : Driver_abortFlash

  implicit none

  logical :: arrayExists
  logical :: goodShape
!
!
!   ...Check allocation and dimensions of the coefficient array.
!
!
  arrayExists = allocated (op_Aij4)

  if (.not.arrayExists) then
       call Driver_abortFlash ('[op_setPEcoeffsAij4atomsZ73] ERROR: array op_Aij4 not allocated')
  end if

  goodShape =       (size (op_Aij4,1) == 4)   &
              .and. (size (op_Aij4,2) == 13)  &
              .and. (size (op_Aij4,3) == 100)

  if (.not.goodShape) then
       call Driver_abortFlash ('[op_setPEcoeffsAij4atomsZ73] ERROR: array op_Aij4 has bad shape')
  end if
!
!
!   ...Set the expansion coefficient array. Last index denotes the
!      atomic order number Z.
!
!
  op_Aij4 (1:4, 1, 44) = (/ 7.391E+02, 3.482E+01, 4.295E-01,-6.811E-03/)
  op_Aij4 (1:4, 2, 44) = (/-8.171E+03, 2.199E+04,-8.951E+03, 1.037E+03/)
  op_Aij4 (1:4, 3, 44) = (/ 3.087E+02,-1.095E+02, 1.012E+04,-4.421E+03/)
  op_Aij4 (1:4, 4, 44) = (/-2.831E+02, 7.880E+03, 1.913E+04,-6.518E+03/)
  op_Aij4 (1:4, 5, 44) = (/-3.547E+02, 1.093E+04, 2.575E+04,-9.586E+03/)
  op_Aij4 (1:4, 6, 44) = (/-1.311E+01, 1.428E+03, 9.237E+04,-1.148E+05/)
  op_Aij4 (1:4, 7, 44) = (/ 1.007E-02, 1.277E+02, 1.086E+06,-7.755E+06/)
  op_Aij4 (1:4, 8, 44) = (/ 1.633E+00,-4.030E+02, 1.144E+06,-9.817E+06/)
  op_Aij4 (1:4, 9, 44) = (/ 6.218E-01, 9.117E+02, 5.873E+05, 6.614E+07/)
 
  op_Aij4 (1:4, 1, 45) = (/ 8.935E+02, 2.878E+00, 1.008E+00,-9.542E-03/)
  op_Aij4 (1:4, 2, 45) = (/-8.948E+03, 2.146E+04,-7.129E+03, 4.790E+02/)
  op_Aij4 (1:4, 3, 45) = (/-5.992E+02, 4.521E+03, 3.942E+03,-1.812E+03/)
  op_Aij4 (1:4, 4, 45) = (/-2.665E+02, 7.197E+03, 2.390E+04,-7.946E+03/)
  op_Aij4 (1:4, 5, 45) = (/-3.285E+02, 9.342E+03, 3.454E+04,-1.203E+04/)
  op_Aij4 (1:4, 6, 45) = (/-2.532E+01, 2.009E+03, 9.447E+04,-1.115E+05/)
  op_Aij4 (1:4, 7, 45) = (/-3.043E+00, 9.079E+02, 1.126E+06,-8.086E+06/)
  op_Aij4 (1:4, 8, 45) = (/ 1.870E+00,-4.570E+02, 1.251E+06,-1.191E+07/)
  op_Aij4 (1:4, 9, 45) = (/ 6.726E-01, 9.644E+02, 7.281E+05, 4.409E+07/)
 
  op_Aij4 (1:4, 1, 46) = (/ 2.673E+02, 8.162E+01,-1.115E+00, 3.804E-03/)
  op_Aij4 (1:4, 2, 46) = (/-1.793E+04, 4.014E+04,-1.899E+04, 2.659E+03/)
  op_Aij4 (1:4, 3, 46) = (/-8.864E+02, 6.126E+03, 2.706E+03,-1.656E+03/)
  op_Aij4 (1:4, 4, 46) = (/-2.530E+02, 7.129E+03, 2.620E+04,-9.068E+03/)
  op_Aij4 (1:4, 5, 46) = (/-3.225E+02, 1.001E+04, 3.550E+04,-1.354E+04/)
  op_Aij4 (1:4, 6, 46) = (/ 1.642E+01, 4.603E+02, 1.181E+05,-1.737E+05/)
  op_Aij4 (1:4, 7, 46) = (/-2.888E+00, 8.714E+02, 1.203E+06,-9.306E+06/)
  op_Aij4 (1:4, 8, 46) = (/ 1.857E+00,-3.662E+02, 1.308E+06,-1.219E+07/)
  op_Aij4 (1:4, 9, 46) = (/ 7.151E-01, 1.066E+03, 7.318E+05, 6.061E+07/)
 
  op_Aij4 (1:4, 1, 47) = (/ 1.221E+03, 7.507E+01,-1.363E+00, 5.328E-03/)
  op_Aij4 (1:4, 2, 47) = (/-4.940E+03, 1.907E+04,-7.444E+03, 5.703E+02/)
  op_Aij4 (1:4, 3, 47) = (/ 1.918E+02, 5.874E+02, 1.228E+04,-6.006E+03/)
  op_Aij4 (1:4, 4, 47) = (/-2.547E+02, 7.851E+03, 2.733E+04,-1.038E+04/)
  op_Aij4 (1:4, 5, 47) = (/-3.170E+02, 1.045E+04, 3.850E+04,-1.560E+04/)
  op_Aij4 (1:4, 6, 47) = (/-1.817E+00, 1.265E+03, 1.192E+05,-1.684E+05/)
  op_Aij4 (1:4, 7, 47) = (/-4.061E+00, 1.339E+03, 1.260E+06,-9.752E+06/)
  op_Aij4 (1:4, 8, 47) = (/ 2.065E+00,-4.444E+02, 1.433E+06,-1.533E+07/)
  op_Aij4 (1:4, 9, 47) = (/ 7.734E-01, 1.157E+03, 8.006E+05, 6.214E+07/)
 
  op_Aij4 (1:4, 1, 48) = (/ 5.246E+02, 1.111E+02,-1.429E+00, 3.266E-03/)
  op_Aij4 (1:4, 2, 48) = (/-2.053E+03, 1.198E+04,-2.209E+03,-5.686E+02/)
  op_Aij4 (1:4, 3, 48) = (/-2.454E+02, 8.162E+03, 2.822E+04,-1.156E+04/)
  op_Aij4 (1:4, 4, 48) = (/-2.677E+02, 8.244E+03, 4.935E+04,-1.791E+04/)
  op_Aij4 (1:4, 5, 48) = (/-2.306E+01, 2.129E+03, 1.161E+05,-1.547E+05/)
  op_Aij4 (1:4, 6, 48) = (/-8.206E+00, 2.269E+03, 1.268E+06,-9.746E+06/)
  op_Aij4 (1:4, 7, 48) = (/ 2.254E+00,-4.904E+02, 1.506E+06,-1.642E+07/)
  op_Aij4 (1:4, 8, 48) = (/ 8.123E-01, 1.229E+03, 8.686E+05, 5.293E+07/)
 
  op_Aij4 (1:4, 1, 49) = (/ 0.000E+00, 0.000E+00, 1.850E-02, 0.000E+00/)
  op_Aij4 (1:4, 2, 49) = (/ 1.917E+03, 1.793E+00,-4.297E-02,-5.864E-03/)
  op_Aij4 (1:4, 3, 49) = (/-5.531E+02, 5.425E+03, 5.739E+03,-2.983E+03/)
  op_Aij4 (1:4, 4, 49) = (/-2.324E+02, 7.725E+03, 3.333E+04,-1.361E+04/)
  op_Aij4 (1:4, 5, 49) = (/-2.675E+02, 8.828E+03, 5.249E+04,-2.065E+04/)
  op_Aij4 (1:4, 6, 49) = (/-2.149E+01, 2.143E+03, 1.256E+05,-1.775E+05/)
  op_Aij4 (1:4, 7, 49) = (/-2.097E+00, 1.045E+03, 1.428E+06,-1.216E+07/)
  op_Aij4 (1:4, 8, 49) = (/ 2.233E+00,-3.526E+02, 1.578E+06,-1.747E+07/)
  op_Aij4 (1:4, 9, 49) = (/ 8.685E-01, 1.331E+03, 9.171E+05, 6.234E+07/)
 
  op_Aij4 (1:4, 1, 50) = (/ 0.000E+00, 0.000E+00, 7.300E-02, 0.000E+00/)
  op_Aij4 (1:4, 2, 50) = (/ 1.755E+03,-3.700E+01, 6.164E+00,-1.436E-01/)
  op_Aij4 (1:4, 3, 50) = (/ 3.184E+04,-5.161E+04, 3.430E+04,-6.716E+03/)
  op_Aij4 (1:4, 4, 50) = (/-8.411E+02, 7.174E+03, 3.429E+03,-1.945E+03/)
  op_Aij4 (1:4, 5, 50) = (/-2.078E+02, 6.795E+03, 3.953E+04,-1.546E+04/)
  op_Aij4 (1:4, 6, 50) = (/-2.735E+02, 1.006E+04, 5.154E+04,-2.316E+04/)
  op_Aij4 (1:4, 7, 50) = (/ 1.409E+01, 5.245E+02, 1.554E+05,-2.738E+05/)
  op_Aij4 (1:4, 8, 50) = (/-6.053E+00, 2.152E+03, 1.430E+06,-1.218E+07/)
  op_Aij4 (1:4, 9, 50) = (/ 2.434E+00,-4.441E+02, 1.694E+06,-2.116E+07/)
  op_Aij4 (1:4,10, 50) = (/ 9.156E-01, 1.438E+03, 9.506E+05, 6.990E+07/)
 
  op_Aij4 (1:4, 1, 51) = (/ 6.617E+02,-3.674E+01, 7.073E-01,-2.415E-03/)
  op_Aij4 (1:4, 2, 51) = (/ 2.657E+03,-4.585E+02, 4.392E+01,-9.883E-01/)
  op_Aij4 (1:4, 3, 51) = (/ 1.852E+04,-3.091E+04, 2.529E+04,-5.381E+03/)
  op_Aij4 (1:4, 4, 51) = (/-7.504E+02, 7.011E+03, 4.377E+03,-1.656E+03/)
  op_Aij4 (1:4, 5, 51) = (/-2.154E+02, 7.884E+03, 3.884E+04,-1.742E+04/)
  op_Aij4 (1:4, 6, 51) = (/-2.413E+02, 8.640E+03, 6.267E+04,-2.629E+04/)
  op_Aij4 (1:4, 7, 51) = (/-2.300E+01, 2.351E+03, 1.410E+05,-2.130E+05/)
  op_Aij4 (1:4, 8, 51) = (/-9.816E+00, 3.157E+03, 1.443E+06,-1.193E+07/)
  op_Aij4 (1:4, 9, 51) = (/ 2.470E+00,-3.671E+02, 1.779E+06,-2.260E+07/)
  op_Aij4 (1:4,10, 51) = (/ 9.713E-01, 1.520E+03, 1.016E+06, 7.466E+07/)
 
  op_Aij4 (1:4, 1, 52) = (/ 0.000E+00, 0.000E+00, 2.300E-01, 0.000E+00/)
  op_Aij4 (1:4, 2, 52) = (/ 1.891E+03,-3.992E+02, 5.548E+01,-1.669E+00/)
  op_Aij4 (1:4, 3, 52) = (/ 4.769E+04,-9.744E+04, 7.551E+04,-1.741E+04/)
  op_Aij4 (1:4, 4, 52) = (/-4.397E+01, 2.059E+03, 1.571E+04,-9.021E+03/)
  op_Aij4 (1:4, 5, 52) = (/-1.946E+02, 7.129E+03, 4.399E+04,-1.939E+04/)
  op_Aij4 (1:4, 6, 52) = (/-2.372E+02, 9.166E+03, 6.308E+04,-2.911E+04/)
  op_Aij4 (1:4, 7, 52) = (/ 1.086E+00, 1.149E+03, 1.640E+05,-2.950E+05/)
  op_Aij4 (1:4, 8, 52) = (/-3.401E+00, 1.832E+03, 1.571E+06,-1.426E+07/)
  op_Aij4 (1:4, 9, 52) = (/ 2.389E+00,-1.733E+02, 1.798E+06,-2.273E+07/)
  op_Aij4 (1:4,10, 52) = (/ 1.007E+00, 1.586E+03, 1.076E+06, 7.154E+07/)
 
  op_Aij4 (1:4, 1, 53) = (/ 4.617E+02,-1.755E+01, 2.146E-01, 5.709E-03/)
  op_Aij4 (1:4, 2, 53) = (/ 2.438E+03,-6.588E+02, 1.076E+02,-3.949E+00/)
  op_Aij4 (1:4, 3, 53) = (/-1.216E+03, 2.425E+04,-2.237E+04, 8.492E+03/)
  op_Aij4 (1:4, 4, 53) = (/-6.432E+02, 7.074E+03, 5.262E+03,-2.211E+03/)
  op_Aij4 (1:4, 5, 53) = (/-1.833E+02, 6.763E+03, 5.201E+04,-2.240E+04/)
  op_Aij4 (1:4, 6, 53) = (/-2.150E+02, 8.275E+03, 7.525E+04,-3.297E+04/)
  op_Aij4 (1:4, 7, 53) = (/-2.566E+01, 2.615E+03, 1.595E+05,-2.557E+05/)
  op_Aij4 (1:4, 8, 53) = (/-3.670E+00, 1.872E+03, 1.719E+06,-1.646E+07/)
  op_Aij4 (1:4, 9, 53) = (/ 2.225E+00, 1.829E+02, 1.881E+06,-2.157E+07/)
  op_Aij4 (1:4,10, 53) = (/ 1.098E+00, 1.755E+03, 1.154E+06, 8.971E+07/)
 
  op_Aij4 (1:4, 1, 54) = (/ 2.648E+02,-1.704E+01, 9.316E-01,-7.699E-03/)
  op_Aij4 (1:4, 2, 54) = (/ 3.266E+03,-1.120E+03, 1.619E+02,-6.243E+00/)
  op_Aij4 (1:4, 3, 54) = (/ 7.789E+04,-1.758E+05, 1.423E+05,-3.620E+04/)
  op_Aij4 (1:4, 4, 54) = (/-2.422E+04, 6.454E+04,-3.914E+04, 8.599E+03/)
  op_Aij4 (1:4, 5, 54) = (/-6.606E+02, 7.269E+03, 6.119E+03,-3.383E+03/)
  op_Aij4 (1:4, 6, 54) = (/-1.565E+02, 5.749E+03, 5.985E+04,-2.396E+04/)
  op_Aij4 (1:4, 7, 54) = (/-2.236E+02, 9.444E+03, 7.463E+04,-3.742E+04/)
  op_Aij4 (1:4, 8, 54) = (/-2.010E+01, 2.400E+03, 1.720E+05,-2.977E+05/)
  op_Aij4 (1:4, 9, 54) = (/-1.622E+00, 1.830E+03, 1.780E+06,-1.721E+07/)
  op_Aij4 (1:4,10, 54) = (/ 2.082E+00, 3.625E+02, 1.962E+06,-2.448E+07/)
  op_Aij4 (1:4,11, 54) = (/ 1.148E+00, 1.892E+03, 1.133E+06, 1.244E+08/)
 
  op_Aij4 (1:4, 1, 55) = (/ 5.253E+02,-4.708E+01, 1.701E+00,-3.056E-03/)
  op_Aij4 (1:4, 2, 55) = (/-1.999E+03, 3.398E+03,-8.813E+02, 6.592E+01/)
  op_Aij4 (1:4, 3, 55) = (/-9.499E+03, 4.412E+04,-3.666E+04, 1.130E+04/)
  op_Aij4 (1:4, 4, 55) = (/-3.460E+02, 8.781E+03, 1.848E+03,-3.182E+02/)
  op_Aij4 (1:4, 5, 55) = (/-2.807E+02, 4.779E+03, 1.240E+04,-6.939E+03/)
  op_Aij4 (1:4, 6, 55) = (/-8.402E+01, 2.872E+03, 7.699E+04,-1.853E+04/)
  op_Aij4 (1:4, 7, 55) = (/-2.254E+02, 1.015E+04, 7.940E+04,-4.284E+04/)
  op_Aij4 (1:4, 8, 55) = (/-1.987E+00, 1.400E+03, 2.040E+05,-4.166E+05/)
  op_Aij4 (1:4, 9, 55) = (/-4.238E+00, 2.444E+03, 1.873E+06,-1.899E+07/)
  op_Aij4 (1:4,10, 55) = (/ 2.278E+00, 4.044E+02, 2.085E+06,-2.635E+07/)
  op_Aij4 (1:4,11, 55) = (/ 1.226E+00, 2.030E+03, 1.249E+06, 1.169E+08/)
 
  op_Aij4 (1:4, 1, 56) = (/ 6.159E+02,-6.640E+01, 2.701E+00,-1.599E-02/)
  op_Aij4 (1:4, 2, 56) = (/ 1.191E+03, 6.631E+02,-9.405E+01, 6.376E+00/)
  op_Aij4 (1:4, 3, 56) = (/-1.422E+04, 4.480E+04,-3.020E+04, 8.387E+03/)
  op_Aij4 (1:4, 4, 56) = (/-2.930E+02, 8.866E+03, 1.354E+03,-2.310E+02/)
  op_Aij4 (1:4, 5, 56) = (/-2.751E+02, 9.345E+03, 1.542E+03,-2.889E+02/)
  op_Aij4 (1:4, 6, 56) = (/-6.491E+02, 7.592E+03, 8.802E+03,-6.067E+03/)
  op_Aij4 (1:4, 7, 56) = (/-1.084E+02, 4.043E+03, 7.718E+04,-2.532E+04/)
  op_Aij4 (1:4, 8, 56) = (/-1.836E+02, 7.952E+03, 9.545E+04,-4.532E+04/)
  op_Aij4 (1:4, 9, 56) = (/ 8.708E+00, 7.581E+02, 2.258E+05,-5.011E+05/)
  op_Aij4 (1:4,10, 56) = (/-5.846E+00, 3.162E+03, 1.902E+06,-1.948E+07/)
  op_Aij4 (1:4,11, 56) = (/ 2.590E+00, 2.297E+02, 2.236E+06,-3.193E+07/)
  op_Aij4 (1:4,12, 56) = (/ 1.281E+00, 2.144E+03, 1.303E+06, 1.193E+08/)
 
  op_Aij4 (1:4, 1, 57) = (/ 5.406E+02,-5.009E+01, 2.070E+00,-1.381E-02/)
  op_Aij4 (1:4, 2, 57) = (/ 6.683E+02, 9.673E+02,-2.153E+02, 1.381E+01/)
  op_Aij4 (1:4, 3, 57) = (/-2.289E+04, 6.758E+04,-4.780E+04, 1.250E+04/)
  op_Aij4 (1:4, 4, 57) = (/-4.024E+02, 8.837E+03, 2.398E+03,-4.154E+02/)
  op_Aij4 (1:4, 5, 57) = (/-3.424E+02, 9.463E+03, 2.343E+03,-4.528E+02/)
  op_Aij4 (1:4, 6, 57) = (/-8.911E+02, 1.007E+04, 3.513E+03,-2.142E+03/)
  op_Aij4 (1:4, 7, 57) = (/-9.085E+01, 3.459E+03, 8.639E+04,-2.558E+04/)
  op_Aij4 (1:4, 8, 57) = (/-1.485E+02, 6.357E+03, 1.121E+05,-4.626E+04/)
  op_Aij4 (1:4, 9, 57) = (/-6.527E+00, 1.901E+03, 2.209E+05,-4.602E+05/)
  op_Aij4 (1:4,10, 57) = (/-7.588E+00, 3.685E+03, 1.991E+06,-2.054E+07/)
  op_Aij4 (1:4,11, 57) = (/ 2.727E+00, 3.137E+02, 2.356E+06,-3.362E+07/)
  op_Aij4 (1:4,12, 57) = (/ 1.365E+00, 2.298E+03, 1.393E+06, 1.220E+08/)
 
  op_Aij4 (1:4, 1, 58) = (/ 1.525E+03,-1.451E+02, 5.492E+00,-4.193E-02/)
  op_Aij4 (1:4, 2, 58) = (/ 1.513E+03, 5.125E+02,-2.459E+02, 2.320E+01/)
  op_Aij4 (1:4, 3, 58) = (/ 5.551E+04,-1.769E+05, 2.024E+05,-7.125E+04/)
  op_Aij4 (1:4, 4, 58) = (/ 4.656E+03, 6.580E+03,-3.256E+03, 8.079E+02/)
  op_Aij4 (1:4, 5, 58) = (/-2.828E+02, 1.019E+04, 2.005E+03,-4.180E+02/)
  op_Aij4 (1:4, 6, 58) = (/-3.900E+02, 5.970E+03, 1.533E+04,-9.604E+03/)
  op_Aij4 (1:4, 7, 58) = (/-7.263E+01, 2.822E+03, 9.606E+04,-2.458E+04/)
  op_Aij4 (1:4, 8, 58) = (/-1.695E+02, 7.899E+03, 1.139E+05,-5.644E+04/)
  op_Aij4 (1:4, 9, 58) = (/ 1.939E+01, 3.799E+02, 2.661E+05,-6.583E+05/)
  op_Aij4 (1:4,10, 58) = (/-1.029E+01, 4.409E+03, 2.101E+06,-2.280E+07/)
  op_Aij4 (1:4,11, 58) = (/ 2.723E+00, 4.916E+02, 2.494E+06,-3.595E+07/)
  op_Aij4 (1:4,12, 58) = (/ 1.464E+00, 2.444E+03, 1.486E+06, 1.376E+08/)
 
  op_Aij4 (1:4, 1, 59) = (/ 1.779E+03,-1.728E+02, 7.854E+00,-6.296E-02/)
  op_Aij4 (1:4, 2, 59) = (/ 1.020E+03, 1.017E+03,-3.883E+02, 3.566E+01/)
  op_Aij4 (1:4, 3, 59) = (/-6.489E+03, 2.485E+04,-1.060E+04, 2.589E+03/)
  op_Aij4 (1:4, 4, 59) = (/-5.349E+02, 8.049E+03, 5.499E+03,-9.313E+02/)
  op_Aij4 (1:4, 5, 59) = (/-3.829E+02, 1.003E+04, 3.437E+03,-7.202E+02/)
  op_Aij4 (1:4, 6, 59) = (/-1.340E+02, 4.371E+03, 1.917E+04,-1.118E+04/)
  op_Aij4 (1:4, 7, 59) = (/-4.523E+01, 1.773E+03, 1.075E+05,-1.884E+04/)
  op_Aij4 (1:4, 8, 59) = (/-1.401E+02, 6.504E+03, 1.315E+05,-5.758E+04/)
  op_Aij4 (1:4, 9, 59) = (/ 8.492E+00, 8.664E+02, 2.819E+05,-7.256E+05/)
  op_Aij4 (1:4,10, 59) = (/-9.126E+00, 4.223E+03, 2.290E+06,-2.709E+07/)
  op_Aij4 (1:4,11, 59) = (/ 3.207E+00, 3.424E+02, 2.697E+06,-4.123E+07/)
  op_Aij4 (1:4,12, 59) = (/ 1.557E+00, 2.710E+03, 1.566E+06, 1.383E+08/)
 
  op_Aij4 (1:4, 1, 60) = (/-6.505E+02, 2.083E+02,-8.262E+00, 1.263E-01/)
  op_Aij4 (1:4, 2, 60) = (/ 5.619E+02, 1.452E+03,-5.060E+02, 4.545E+01/)
  op_Aij4 (1:4, 3, 60) = (/-5.304E+02, 8.032E+03, 3.889E+03,-5.907E+02/)
  op_Aij4 (1:4, 4, 60) = (/ 2.239E+02, 1.199E+04,-1.181E+03, 2.855E+02/)
  op_Aij4 (1:4, 5, 60) = (/-2.457E+02, 5.622E+03, 1.700E+04,-9.398E+03/)
  op_Aij4 (1:4, 6, 60) = (/-6.644E+01, 2.796E+03, 1.098E+05,-2.924E+04/)
  op_Aij4 (1:4, 7, 60) = (/-1.114E+02, 5.206E+03, 1.457E+05,-5.559E+04/)
  op_Aij4 (1:4, 8, 60) = (/ 8.287E+00, 9.600E+02, 2.935E+05,-7.637E+05/)
  op_Aij4 (1:4, 9, 60) = (/-9.944E+00, 4.886E+03, 2.320E+06,-2.702E+07/)
  op_Aij4 (1:4,10, 60) = (/ 3.820E+00, 8.492E+01, 2.868E+06,-4.750E+07/)
  op_Aij4 (1:4,11, 60) = (/ 1.633E+00, 2.833E+03, 1.759E+06, 9.295E+07/)
 
  op_Aij4 (1:4, 1, 61) = (/ 1.690E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 61) = (/ 1.440E+03, 6.486E+02,-1.094E+02, 5.512E+00/)
  op_Aij4 (1:4, 3, 61) = (/-6.403E+00, 7.436E+03, 2.087E+01,-3.609E+00/)
  op_Aij4 (1:4, 4, 61) = (/-5.546E+02, 7.912E+03, 4.958E+03,-7.606E+02/)
  op_Aij4 (1:4, 5, 61) = (/-5.157E+02, 8.937E+03, 6.066E+03,-1.148E+03/)
  op_Aij4 (1:4, 6, 61) = (/-4.607E+02, 9.760E+03, 6.028E+03,-1.298E+03/)
  op_Aij4 (1:4, 7, 61) = (/-1.912E+02, 5.138E+03, 2.120E+04,-1.345E+04/)
  op_Aij4 (1:4, 8, 61) = (/-7.165E+01, 3.169E+03, 1.166E+05,-3.499E+04/)
  op_Aij4 (1:4, 9, 61) = (/-2.589E+02, 8.215E+03,-1.102E+05, 1.824E+06/)
  op_Aij4 (1:4,10, 61) = (/-4.251E+01, 4.740E+03, 2.377E+05,-4.279E+05/)
  op_Aij4 (1:4,11, 61) = (/ 9.003E-02, 2.739E+03, 2.641E+06,-3.336E+07/)
  op_Aij4 (1:4,12, 61) = (/ 2.857E+00, 1.090E+03, 2.888E+06,-4.432E+07/)
  op_Aij4 (1:4,13, 61) = (/ 1.742E+00, 3.069E+03, 1.744E+06, 1.719E+08/)
 
  op_Aij4 (1:4, 1, 62) = (/ 1.680E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 62) = (/-3.980E+02, 2.937E+03,-5.059E+02, 2.698E+01/)
  op_Aij4 (1:4, 3, 62) = (/ 2.417E+03, 6.924E+03,-2.629E+03, 5.790E+02/)
  op_Aij4 (1:4, 4, 62) = (/-5.499E+02, 7.881E+03, 5.664E+03,-8.952E+02/)
  op_Aij4 (1:4, 5, 62) = (/-4.675E+02, 9.685E+03, 5.381E+03,-1.113E+03/)
  op_Aij4 (1:4, 6, 62) = (/-4.771E+02, 9.283E+03, 7.693E+03,-1.657E+03/)
  op_Aij4 (1:4, 7, 62) = (/-1.944E+02, 5.237E+03, 2.215E+04,-1.434E+04/)
  op_Aij4 (1:4, 8, 62) = (/-3.580E+01, 1.576E+03, 1.290E+05,-2.201E+04/)
  op_Aij4 (1:4, 9, 62) = (/-6.400E+01, 3.084E+03, 1.751E+05,-4.595E+04/)
  op_Aij4 (1:4,10, 62) = (/ 7.096E+00, 1.071E+03, 3.267E+05,-9.193E+05/)
  op_Aij4 (1:4,11, 62) = (/-1.082E+01, 5.919E+03, 2.475E+06,-2.997E+07/)
  op_Aij4 (1:4,12, 62) = (/ 3.844E+00, 4.077E+02, 3.137E+06,-5.576E+07/)
  op_Aij4 (1:4,13, 62) = (/ 1.799E+00, 3.181E+03, 1.898E+06, 1.263E+08/)
 
  op_Aij4 (1:4, 1, 63) = (/ 1.660E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 63) = (/-2.970E+02, 2.959E+03,-5.177E+02, 2.881E+01/)
  op_Aij4 (1:4, 3, 63) = (/ 8.618E+02, 8.414E+03,-1.992E+03, 4.240E+02/)
  op_Aij4 (1:4, 4, 63) = (/-5.507E+02, 7.936E+03, 6.518E+03,-1.058E+03/)
  op_Aij4 (1:4, 5, 63) = (/-4.589E+02, 1.019E+04, 5.669E+03,-1.234E+03/)
  op_Aij4 (1:4, 6, 63) = (/-4.913E+02, 8.740E+03, 1.016E+04,-2.142E+03/)
  op_Aij4 (1:4, 7, 63) = (/-2.288E+02, 5.934E+03, 2.154E+04,-1.326E+04/)
  op_Aij4 (1:4, 8, 63) = (/-2.466E+01, 1.115E+03, 1.400E+05,-1.768E+04/)
  op_Aij4 (1:4, 9, 63) = (/-6.266E+01, 3.143E+03, 1.878E+05,-5.089E+04/)
  op_Aij4 (1:4,10, 63) = (/-4.306E+00, 2.026E+03, 3.280E+05,-9.012E+05/)
  op_Aij4 (1:4,11, 63) = (/-7.610E+00, 5.488E+03, 2.642E+06,-3.287E+07/)
  op_Aij4 (1:4,12, 63) = (/ 3.783E+00, 7.655E+02, 3.245E+06,-5.731E+07/)
  op_Aij4 (1:4,13, 63) = (/ 1.904E+00, 3.400E+03, 2.020E+06, 1.315E+08/)
 
  op_Aij4 (1:4, 1, 64) = (/ 1.630E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 64) = (/ 4.170E+01, 2.626E+03,-3.982E+02, 2.107E+01/)
  op_Aij4 (1:4, 3, 64) = (/ 1.902E+03, 7.831E+03,-2.992E+03, 7.117E+02/)
  op_Aij4 (1:4, 4, 64) = (/-5.394E+02, 7.849E+03, 7.295E+03,-1.216E+03/)
  op_Aij4 (1:4, 5, 64) = (/-4.486E+02, 1.029E+04, 6.091E+03,-1.380E+03/)
  op_Aij4 (1:4, 6, 64) = (/-4.679E+02, 9.424E+03, 9.571E+03,-2.201E+03/)
  op_Aij4 (1:4, 7, 64) = (/-3.017E+02, 6.938E+03, 1.944E+04,-1.104E+04/)
  op_Aij4 (1:4, 8, 64) = (/-5.768E+01, 2.841E+03, 1.366E+05,-4.131E+04/)
  op_Aij4 (1:4, 9, 64) = (/-3.358E+01, 1.701E+03, 2.029E+05,-3.312E+04/)
  op_Aij4 (1:4,10, 64) = (/-8.668E+00, 2.379E+03, 3.331E+05,-9.285E+05/)
  op_Aij4 (1:4,11, 64) = (/ 3.209E+00, 1.579E+03, 3.144E+06,-4.804E+07/)
  op_Aij4 (1:4,12, 64) = (/ 1.966E+00, 3.553E+03, 2.103E+06, 1.345E+08/)
 
  op_Aij4 (1:4, 1, 65) = (/ 1.600E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 65) = (/-1.794E+02, 2.952E+03,-4.363E+02, 2.180E+01/)
  op_Aij4 (1:4, 3, 65) = (/ 4.709E+02, 8.920E+03,-1.596E+03, 3.598E+02/)
  op_Aij4 (1:4, 4, 65) = (/-5.345E+02, 7.632E+03, 8.660E+03,-1.449E+03/)
  op_Aij4 (1:4, 5, 65) = (/-4.988E+02, 9.401E+03, 9.226E+03,-2.000E+03/)
  op_Aij4 (1:4, 6, 65) = (/-4.700E+02, 9.391E+03, 1.117E+04,-2.613E+03/)
  op_Aij4 (1:4, 7, 65) = (/-1.902E+02, 5.724E+03, 2.615E+04,-1.782E+04/)
  op_Aij4 (1:4, 8, 65) = (/-1.568E+02, 1.116E+04, 8.757E+04,-7.647E+04/)
  op_Aij4 (1:4, 9, 65) = (/-4.533E+01, 2.421E+03, 2.126E+05,-4.859E+04/)
  op_Aij4 (1:4,10, 65) = (/-4.721E+00, 2.127E+03, 3.644E+05,-1.109E+06/)
  op_Aij4 (1:4,11, 65) = (/ 3.903E+00, 1.250E+03, 3.405E+06,-5.637E+07/)
  op_Aij4 (1:4,12, 65) = (/ 2.077E+00, 3.763E+03, 2.261E+06, 1.155E+08/)
 
  op_Aij4 (1:4, 1, 66) = (/ 1.570E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 66) = (/-2.930E+02, 3.204E+03,-4.744E+02, 2.422E+01/)
  op_Aij4 (1:4, 3, 66) = (/ 5.198E+02, 9.201E+03,-1.861E+03, 4.412E+02/)
  op_Aij4 (1:4, 4, 66) = (/-5.242E+02, 7.751E+03, 9.341E+03,-1.624E+03/)
  op_Aij4 (1:4, 5, 66) = (/-4.635E+02, 1.050E+04, 8.066E+03,-1.930E+03/)
  op_Aij4 (1:4, 6, 66) = (/-4.608E+02, 9.448E+03, 1.216E+04,-2.942E+03/)
  op_Aij4 (1:4, 7, 66) = (/-1.077E+02, 4.675E+03, 3.253E+04,-2.519E+04/)
  op_Aij4 (1:4, 8, 66) = (/-6.326E+01, 3.407E+03, 1.499E+05,-5.526E+04/)
  op_Aij4 (1:4, 9, 66) = (/-7.262E+01, 4.176E+03, 2.130E+05,-8.026E+04/)
  op_Aij4 (1:4,10, 66) = (/-1.800E+01, 3.374E+03, 3.526E+05,-9.982E+05/)
  op_Aij4 (1:4,11, 66) = (/ 4.112E+00, 1.344E+03, 3.520E+06,-5.867E+07/)
  op_Aij4 (1:4,12, 66) = (/ 2.166E+00, 3.928E+03, 2.396E+06, 1.007E+08/)
 
  op_Aij4 (1:4, 1, 67) = (/ 1.530E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 67) = (/-2.278E+02, 3.277E+03,-5.051E+02, 2.412E+01/)
  op_Aij4 (1:4, 3, 67) = (/ 1.858E+03, 8.664E+03,-3.790E+03, 1.030E+03/)
  op_Aij4 (1:4, 4, 67) = (/-5.221E+02, 8.169E+03, 9.852E+03,-1.807E+03/)
  op_Aij4 (1:4, 5, 67) = (/-4.878E+02, 9.589E+03, 1.119E+04,-2.580E+03/)
  op_Aij4 (1:4, 6, 67) = (/-4.549E+02, 9.349E+03, 1.378E+04,-3.393E+03/)
  op_Aij4 (1:4, 7, 67) = (/-1.379E+02, 5.032E+03, 3.428E+04,-2.787E+04/)
  op_Aij4 (1:4, 8, 67) = (/-4.939E+01, 2.705E+03, 1.628E+05,-5.056E+04/)
  op_Aij4 (1:4, 9, 67) = (/-4.426E+01, 2.555E+03, 2.349E+05,-5.948E+04/)
  op_Aij4 (1:4,10, 67) = (/-2.988E+00, 2.094E+03, 4.072E+05,-1.335E+06/)
  op_Aij4 (1:4,11, 67) = (/ 5.131E+00, 5.023E+02, 4.000E+06,-7.974E+07/)
  op_Aij4 (1:4,12, 67) = (/ 2.274E+00, 4.258E+03, 2.387E+06, 1.450E+08/)
 
  op_Aij4 (1:4, 1, 68) = (/ 1.520E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 68) = (/-4.193E+02, 3.700E+03,-5.990E+02, 3.331E+01/)
  op_Aij4 (1:4, 3, 68) = (/ 3.499E+03, 6.300E+03,-3.724E+03, 1.119E+03/)
  op_Aij4 (1:4, 4, 68) = (/-5.148E+02, 8.464E+03, 1.038E+04,-1.998E+03/)
  op_Aij4 (1:4, 5, 68) = (/-4.781E+02, 8.546E+03, 1.463E+04,-3.195E+03/)
  op_Aij4 (1:4, 6, 68) = (/-4.470E+02, 9.490E+03, 1.489E+04,-3.799E+03/)
  op_Aij4 (1:4, 7, 68) = (/-2.279E+02, 6.501E+03, 3.083E+04,-2.363E+04/)
  op_Aij4 (1:4, 8, 68) = (/-5.162E+01, 2.947E+03, 1.710E+05,-5.828E+04/)
  op_Aij4 (1:4, 9, 68) = (/-5.360E+01, 3.254E+03, 2.449E+05,-7.821E+04/)
  op_Aij4 (1:4,10, 68) = (/ 6.456E+00, 1.211E+03, 4.583E+05,-1.684E+06/)
  op_Aij4 (1:4,11, 68) = (/ 4.025E+00, 2.031E+03, 3.795E+06,-6.651E+07/)
  op_Aij4 (1:4,12, 68) = (/ 2.386E+00, 4.497E+03, 2.558E+06, 1.403E+08/)
 
  op_Aij4 (1:4, 1, 69) = (/ 1.500E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 69) = (/-4.117E+02, 3.892E+03,-6.641E+02, 4.384E+01/)
  op_Aij4 (1:4, 3, 69) = (/ 3.309E+03, 6.805E+03,-4.124E+03, 1.286E+03/)
  op_Aij4 (1:4, 4, 69) = (/-5.074E+02, 8.420E+03, 1.161E+04,-2.277E+03/)
  op_Aij4 (1:4, 5, 69) = (/-4.701E+02, 8.553E+03, 1.620E+04,-3.609E+03/)
  op_Aij4 (1:4, 6, 69) = (/-4.405E+02, 9.321E+03, 1.702E+04,-4.382E+03/)
  op_Aij4 (1:4, 7, 69) = (/-2.641E+02, 7.170E+03, 3.109E+04,-2.392E+04/)
  op_Aij4 (1:4, 8, 69) = (/-5.189E+01, 3.077E+03, 1.808E+05,-6.499E+04/)
  op_Aij4 (1:4, 9, 69) = (/-5.311E+01, 3.348E+03, 2.598E+05,-8.657E+04/)
  op_Aij4 (1:4,10, 69) = (/-3.887E+00, 2.132E+03, 4.637E+05,-1.723E+06/)
  op_Aij4 (1:4,11, 69) = (/ 4.124E+00, 2.299E+03, 3.956E+06,-7.043E+07/)
  op_Aij4 (1:4,12, 69) = (/ 2.512E+00, 4.774E+03, 2.690E+06, 1.454E+08/)
 
  op_Aij4 (1:4, 1, 70) = (/ 1.490E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 70) = (/-5.724E+02, 4.268E+03,-7.115E+02, 4.693E+01/)
  op_Aij4 (1:4, 3, 70) = (/-2.392E+02, 2.574E+03, 1.221E+04,-1.237E+03/)
  op_Aij4 (1:4, 4, 70) = (/-4.649E+02, 1.046E+04, 8.110E+03,-1.884E+03/)
  op_Aij4 (1:4, 5, 70) = (/-4.593E+02, 8.718E+03, 1.696E+04,-3.936E+03/)
  op_Aij4 (1:4, 6, 70) = (/-4.300E+02, 9.905E+03, 1.674E+04,-4.633E+03/)
  op_Aij4 (1:4, 7, 70) = (/-2.294E+02, 6.908E+03, 3.414E+04,-2.810E+04/)
  op_Aij4 (1:4, 8, 70) = (/-5.328E+01, 3.291E+03, 1.869E+05,-7.332E+04/)
  op_Aij4 (1:4, 9, 70) = (/-7.311E+01, 4.929E+03, 2.602E+05,-1.239E+05/)
  op_Aij4 (1:4,10, 70) = (/-2.434E+01, 4.269E+03, 4.243E+05,-1.379E+06/)
  op_Aij4 (1:4,11, 70) = (/ 4.575E+00, 2.312E+03, 4.086E+06,-7.562E+07/)
  op_Aij4 (1:4,12, 70) = (/ 2.606E+00, 4.980E+03, 2.895E+06, 9.923E+07/)
 
  op_Aij4 (1:4, 1, 71) = (/ 1.490E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 71) = (/-5.832E+02, 4.426E+03,-6.739E+02, 3.958E+01/)
  op_Aij4 (1:4, 3, 71) = (/ 7.177E+02, 1.052E+04,-3.382E+03, 1.011E+03/)
  op_Aij4 (1:4, 4, 71) = (/-4.936E+02, 9.095E+03, 1.255E+04,-2.717E+03/)
  op_Aij4 (1:4, 5, 71) = (/-4.436E+02, 8.391E+03, 1.928E+04,-4.456E+03/)
  op_Aij4 (1:4, 6, 71) = (/-4.229E+02, 1.051E+04, 1.693E+04,-4.987E+03/)
  op_Aij4 (1:4, 7, 71) = (/-1.794E+02, 6.176E+03, 4.132E+04,-3.872E+04/)
  op_Aij4 (1:4, 8, 71) = (/-5.213E+01, 3.333E+03, 1.974E+05,-7.977E+04/)
  op_Aij4 (1:4, 9, 71) = (/-5.333E+01, 3.627E+03, 2.858E+05,-1.071E+05/)
  op_Aij4 (1:4,10, 71) = (/ 3.726E+00, 1.486E+03, 5.352E+05,-2.222E+06/)
  op_Aij4 (1:4,11, 71) = (/ 5.266E+00, 2.137E+03, 4.329E+06,-8.327E+07/)
  op_Aij4 (1:4,12, 71) = (/ 2.736E+00, 5.209E+03, 3.153E+06, 5.239E+07/)
 
  op_Aij4 (1:4, 1, 72) = (/ 1.480E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 72) = (/-5.793E+02, 4.429E+03,-4.123E+02,-2.341E+01/)
  op_Aij4 (1:4, 3, 72) = (/ 6.148E+02, 1.077E+04,-3.332E+03, 1.032E+03/)
  op_Aij4 (1:4, 4, 72) = (/-4.835E+02, 9.549E+03, 1.284E+04,-2.957E+03/)
  op_Aij4 (1:4, 5, 72) = (/-4.488E+02, 9.643E+03, 1.803E+04,-4.705E+03/)
  op_Aij4 (1:4, 6, 72) = (/-4.146E+02, 1.043E+04, 1.883E+04,-5.684E+03/)
  op_Aij4 (1:4, 7, 72) = (/-4.615E+02, 1.113E+04, 1.850E+04,-2.974E+03/)
  op_Aij4 (1:4, 8, 72) = (/-6.090E+01, 4.112E+03, 2.015E+05,-9.934E+04/)
  op_Aij4 (1:4, 9, 72) = (/-7.386E+01, 5.387E+03, 2.859E+05,-1.536E+05/)
  op_Aij4 (1:4,10, 72) = (/-1.873E+01, 4.102E+03, 4.698E+05,-1.572E+06/)
  op_Aij4 (1:4,11, 72) = (/ 4.753E+00, 2.673E+03, 4.462E+06,-8.960E+07/)
  op_Aij4 (1:4,12, 72) = (/ 2.845E+00, 5.554E+03, 3.013E+06, 1.535E+08/)
 
  op_Aij4 (1:4, 1, 73) = (/ 1.470E+03, 0.000E+00, 0.000E+00, 0.000E+00/)
  op_Aij4 (1:4, 2, 73) = (/-5.773E+02, 4.832E+03,-8.075E+02, 4.906E+01/)
  op_Aij4 (1:4, 3, 73) = (/ 2.954E+02, 1.102E+04,-2.147E+03, 6.678E+02/)
  op_Aij4 (1:4, 4, 73) = (/-4.771E+02, 1.003E+04, 1.338E+04,-3.254E+03/)
  op_Aij4 (1:4, 5, 73) = (/-4.305E+02, 8.881E+03, 2.202E+04,-5.544E+03/)
  op_Aij4 (1:4, 6, 73) = (/-4.034E+02, 9.726E+03, 2.291E+04,-6.762E+03/)
  op_Aij4 (1:4, 7, 73) = (/-8.593E+01, 4.826E+03, 5.410E+04,-5.683E+04/)
  op_Aij4 (1:4, 8, 73) = (/-5.521E+01, 3.828E+03, 2.145E+05,-1.019E+05/)
  op_Aij4 (1:4, 9, 73) = (/-4.987E+01, 3.641E+03, 3.160E+05,-1.250E+05/)
  op_Aij4 (1:4,10, 73) = (/-2.176E+00, 2.172E+03, 5.708E+05,-2.482E+06/)
  op_Aij4 (1:4,11, 73) = (/ 5.194E+00, 2.794E+03, 4.645E+06,-9.648E+07/)
  op_Aij4 (1:4,12, 73) = (/ 2.973E+00, 5.785E+03, 3.320E+06, 9.608E+07/)
!
!
!   ...Ready! 
!
!
  return
end subroutine op_setPEcoeffsAij4atomsZ73
