!!****if* source/physics/sourceTerms/Burn/BurnMain/nuclearBurn/Aprox13/bn_mapNetworkToSpecies
!!
!! NAME
!!
!!  bn_mapNetworkToSpecies
!!
!! SYNOPSIS
!!
!!  call bn_mapNetworkToSpecies(integer, intent(IN)  :: networkIn,
!!                              integer, intent(OUT) :: specieOut)
!!
!! DESCRIPTION
!!
!!  Maps network indices required by Aprox13 to species generated by Flash.h
!!
!!
!! ARGUMENTS
!!
!!  networkIn -- integer index of the network as defined in Aprox13 routines
!!  specieOut -- integer corresponding to the same specie in Flash.h
!!
!!***

subroutine bn_mapNetworkToSpecies(networkIn,specieOut)

  use Driver_interface, ONLY: Driver_abortFlash
  use Logfile_interface, ONLY: Logfile_stampMessage

  implicit none

#include "Flash.h"

  integer, intent(IN)  ::   networkIn
  integer, intent(OUT) :: specieOut

  !! Local variable for error diagnostics
  character*256 :: internalFile
  


  select case (networkIn)
  case (1)
     specieOut = HE4_SPEC
  case (2)
     specieOut = C12_SPEC
  case (3)
     specieOut = O16_SPEC
  case (4)
     specieOut = NE20_SPEC
  case (5)
     specieOut = MG24_SPEC
  case (6)
     specieOut = SI28_SPEC
  case (7)
     specieOut = S32_SPEC
  case (8)
     specieOut = AR36_SPEC
  case (9)
     specieOut = CA40_SPEC
  case (10)
     specieOut = TI44_SPEC
  case (11)
     specieOut = CR48_SPEC
  case (12)
     specieOut = FE52_SPEC
  case (13)
     specieOut = NI56_SPEC
  case default
     !! Abort
     call Logfile_stampMessage('ERROR in bn_mapNetworkToSpecies')
     write(internalFile,*) &
          '[bn_mapNetworkToSpecies] Index ',networkIn,' cannot be mapped to a Flash.h species'
     call Logfile_stampMessage(internalFile)
     call Driver_abortFlash(internalFile)
  end select


  return
end subroutine bn_mapNetworkToSpecies
