!!****if* source/physics/sourceTerms/Burn/BurnMain/nuclearBurn/Aprox19/bn_gift
!!
!! NAME
!!  
!!  bn_gift
!!
!!
!! SYNOPSIS
!! 
!!  call bn_gift(real(INOUT)     :: ab, 
!!               integer(IN)     :: n1, 
!!               integer(IN)     :: n2)
!!  
!! DESCRIPTION
!!
!!  Inverts (??) sparse matrix ab using a hard-wired  
!!  routine generated by the GIFT package.
!!  Called by the network integration routines bn_netIntRosen and bn_netIntBader
!!
!! ARGUMENTS
!!
!!  ab:  matrix
!!  n1:  dimension of ab
!!  n2:  dimension of ab
!!
!! NOTES
!!
!!   this version was called gsub_aprox19_100,
!!    renamed to the generic "bn_gift"
!!
!!   the FLASH center no longer has access to the GIFT
!!   package, so these types of routines can no longer
!!   be generated
!!
!!***

subroutine bn_gift(ab,n1,n2)
  implicit none
  integer, INTENT(in) :: n1,n2
  real, INTENT(inout) :: ab(n1,n2)
  real c,tmp( 19)

  tmp( 19)   = 1.0e0/ab( 19, 19)
  c           = ab( 18, 19) * tmp(  19)
  ab( 18, 20) = ab( 18, 20) - ab( 19, 20) * c
  ab( 18,  3) = ab( 18,  3) - ab( 19,  3) * c
  ab( 18, 15) = ab( 18, 15) - ab( 19, 15) * c
  ab( 18, 16) = ab( 18, 16) - ab( 19, 16) * c
  ab( 18, 17) = ab( 18, 17) - ab( 19, 17) * c
  ab( 18, 18) = ab( 18, 18) - ab( 19, 18) * c
  c           = ab( 17, 19) * tmp(  19)
  ab( 17, 20) = ab( 17, 20) - ab( 19, 20) * c
  ab( 17,  3) = ab( 17,  3) - ab( 19,  3) * c
  ab( 17, 15) = ab( 17, 15) - ab( 19, 15) * c
  ab( 17, 16) = ab( 17, 16) - ab( 19, 16) * c
  ab( 17, 17) = ab( 17, 17) - ab( 19, 17) * c
  ab( 17, 18) = ab( 17, 18) - ab( 19, 18) * c
  c           = ab( 16, 19) * tmp(  19)
  ab( 16, 20) = ab( 16, 20) - ab( 19, 20) * c
  ab( 16,  3) = ab( 16,  3) - ab( 19,  3) * c
  ab( 16, 15) = ab( 16, 15) - ab( 19, 15) * c
  ab( 16, 16) = ab( 16, 16) - ab( 19, 16) * c
  ab( 16, 17) = ab( 16, 17) - ab( 19, 17) * c
  ab( 16, 18) = ab( 16, 18) - ab( 19, 18) * c
  c           = ab( 15, 19) * tmp(  19)
  ab( 15, 20) = ab( 15, 20) - ab( 19, 20) * c
  ab( 15,  3) = ab( 15,  3) - ab( 19,  3) * c
  ab( 15, 15) = ab( 15, 15) - ab( 19, 15) * c
  ab( 15, 16) = ab( 15, 16) - ab( 19, 16) * c
  ab( 15, 17) = ab( 15, 17) - ab( 19, 17) * c
  ab( 15, 18) = ab( 15, 18) - ab( 19, 18) * c
  c           = ab(  3, 19) * tmp(  19)
  ab(  3, 20) = ab(  3, 20) - ab( 19, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 19,  3) * c
  ab(  3, 15) = ab(  3, 15) - ab( 19, 15) * c
  ab(  3, 16) = ab(  3, 16) - ab( 19, 16) * c
  ab(  3, 17) = ab(  3, 17) - ab( 19, 17) * c
  ab(  3, 18) = ab(  3, 18) - ab( 19, 18) * c

  tmp( 18)   = 1.0e0/ab( 18, 18)
  c           = ab( 17, 18) * tmp(  18)
  ab( 17, 20) = ab( 17, 20) - ab( 18, 20) * c
  ab( 17,  3) = ab( 17,  3) - ab( 18,  3) * c
  ab( 17, 15) = ab( 17, 15) - ab( 18, 15) * c
  ab( 17, 16) = ab( 17, 16) - ab( 18, 16) * c
  ab( 17, 17) = ab( 17, 17) - ab( 18, 17) * c
  c           = ab( 16, 18) * tmp(  18)
  ab( 16, 20) = ab( 16, 20) - ab( 18, 20) * c
  ab( 16,  3) = ab( 16,  3) - ab( 18,  3) * c
  ab( 16, 15) = ab( 16, 15) - ab( 18, 15) * c
  ab( 16, 16) = ab( 16, 16) - ab( 18, 16) * c
  ab( 16, 17) = ab( 16, 17) - ab( 18, 17) * c
  c           = ab( 15, 18) * tmp(  18)
  ab( 15, 20) = ab( 15, 20) - ab( 18, 20) * c
  ab( 15,  3) = ab( 15,  3) - ab( 18,  3) * c
  ab( 15, 15) = ab( 15, 15) - ab( 18, 15) * c
  ab( 15, 16) = ab( 15, 16) - ab( 18, 16) * c
  ab( 15, 17) = ab( 15, 17) - ab( 18, 17) * c
  c           = ab(  3, 18) * tmp(  18)
  ab(  3, 20) = ab(  3, 20) - ab( 18, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 18,  3) * c
  ab(  3, 15) = ab(  3, 15) - ab( 18, 15) * c
  ab(  3, 16) = ab(  3, 16) - ab( 18, 16) * c
  ab(  3, 17) = ab(  3, 17) - ab( 18, 17) * c

  tmp( 17)   = 1.0e0/ab( 17, 17)
  c           = ab( 16, 17) * tmp(  17)
  ab( 16, 20) = ab( 16, 20) - ab( 17, 20) * c
  ab( 16,  3) = ab( 16,  3) - ab( 17,  3) * c
  ab( 16, 15) = ab( 16, 15) - ab( 17, 15) * c
  ab( 16, 16) = ab( 16, 16) - ab( 17, 16) * c
  c           = ab( 15, 17) * tmp(  17)
  ab( 15, 20) = ab( 15, 20) - ab( 17, 20) * c
  ab( 15,  3) = ab( 15,  3) - ab( 17,  3) * c
  ab( 15, 15) = ab( 15, 15) - ab( 17, 15) * c
  ab( 15, 16) = ab( 15, 16) - ab( 17, 16) * c
  c           = ab(  3, 17) * tmp(  17)
  ab(  3, 20) = ab(  3, 20) - ab( 17, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 17,  3) * c
  ab(  3, 15) = ab(  3, 15) - ab( 17, 15) * c
  ab(  3, 16) = ab(  3, 16) - ab( 17, 16) * c

  tmp( 16)   = 1.0e0/ab( 16, 16)
  c           = ab( 15, 16) * tmp(  16)
  ab( 15, 20) = ab( 15, 20) - ab( 16, 20) * c
  ab( 15,  3) = ab( 15,  3) - ab( 16,  3) * c
  ab( 15, 15) = ab( 15, 15) - ab( 16, 15) * c
  c           = ab(  3, 16) * tmp(  16)
  ab(  3, 20) = ab(  3, 20) - ab( 16, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 16,  3) * c
  ab(  3, 15) = ab(  3, 15) - ab( 16, 15) * c

  tmp( 15)   = 1.0e0/ab( 15, 15)
  c           = ab( 14, 15) * tmp(  15)
  ab( 14, 20) = ab( 14, 20) - ab( 15, 20) * c
  ab( 14,  3) = ab( 14,  3) - ab( 15,  3) * c
  ab( 14, 14) = ab( 14, 14) - ab( 15, 14) * c
  c           = ab(  3, 15) * tmp(  15)
  ab(  3, 20) = ab(  3, 20) - ab( 15, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 15,  3) * c
  ab(  3, 14) = ab(  3, 14) - ab( 15, 14) * c

  tmp( 14)   = 1.0e0/ab( 14, 14)
  c           = ab( 13, 14) * tmp(  14)
  ab( 13, 20) = ab( 13, 20) - ab( 14, 20) * c
  ab( 13,  3) = ab( 13,  3) - ab( 14,  3) * c
  ab( 13, 13) = ab( 13, 13) - ab( 14, 13) * c
  c           = ab(  3, 14) * tmp(  14)
  ab(  3, 20) = ab(  3, 20) - ab( 14, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 14,  3) * c
  ab(  3, 13) = ab(  3, 13) - ab( 14, 13) * c

  tmp( 13)   = 1.0e0/ab( 13, 13)
  c           = ab( 12, 13) * tmp(  13)
  ab( 12, 20) = ab( 12, 20) - ab( 13, 20) * c
  ab( 12,  3) = ab( 12,  3) - ab( 13,  3) * c
  ab( 12, 12) = ab( 12, 12) - ab( 13, 12) * c
  c           = ab(  3, 13) * tmp(  13)
  ab(  3, 20) = ab(  3, 20) - ab( 13, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 13,  3) * c
  ab(  3, 12) = ab(  3, 12) - ab( 13, 12) * c

  tmp( 12)   = 1.0e0/ab( 12, 12)
  c           = ab( 11, 12) * tmp(  12)
  ab( 11, 20) = ab( 11, 20) - ab( 12, 20) * c
  ab( 11,  3) = ab( 11,  3) - ab( 12,  3) * c
  ab( 11, 11) = ab( 11, 11) - ab( 12, 11) * c
  c           = ab(  3, 12) * tmp(  12)
  ab(  3, 20) = ab(  3, 20) - ab( 12, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 12,  3) * c
  ab(  3, 11) = ab(  3, 11) - ab( 12, 11) * c

  tmp( 11)   = 1.0e0/ab( 11, 11)
  c           = ab( 10, 11) * tmp(  11)
  ab( 10, 20) = ab( 10, 20) - ab( 11, 20) * c
  ab( 10,  3) = ab( 10,  3) - ab( 11,  3) * c
  ab( 10, 10) = ab( 10, 10) - ab( 11, 10) * c
  c           = ab(  3, 11) * tmp(  11)
  ab(  3, 20) = ab(  3, 20) - ab( 11, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 11,  3) * c
  ab(  3, 10) = ab(  3, 10) - ab( 11, 10) * c

  tmp( 10)   = 1.0e0/ab( 10, 10)
  c           = ab(  9, 10) * tmp(  10)
  ab(  9, 20) = ab(  9, 20) - ab( 10, 20) * c
  ab(  9,  3) = ab(  9,  3) - ab( 10,  3) * c
  ab(  9,  6) = ab(  9,  6) - ab( 10,  6) * c
  ab(  9,  9) = ab(  9,  9) - ab( 10,  9) * c
  c           = ab(  3, 10) * tmp(  10)
  ab(  3, 20) = ab(  3, 20) - ab( 10, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab( 10,  3) * c
  ab(  3,  6) = ab(  3,  6) - ab( 10,  6) * c
  ab(  3,  9) = ab(  3,  9) - ab( 10,  9) * c

  tmp(  9)   = 1.0e0/ab(  9,  9)
  c           = ab(  8,  9) * tmp(   9)
  ab(  8, 20) = ab(  8, 20) - ab(  9, 20) * c
  ab(  8,  3) = ab(  8,  3) - ab(  9,  3) * c
  ab(  8,  4) = ab(  8,  4) - ab(  9,  4) * c
  ab(  8,  6) = ab(  8,  6) - ab(  9,  6) * c
  ab(  8,  8) = ab(  8,  8) - ab(  9,  8) * c
  c           = ab(  3,  9) * tmp(   9)
  ab(  3, 20) = ab(  3, 20) - ab(  9, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab(  9,  3) * c
  ab(  3,  4) = ab(  3,  4) - ab(  9,  4) * c
  ab(  3,  6) = ab(  3,  6) - ab(  9,  6) * c
  ab(  3,  8) = ab(  3,  8) - ab(  9,  8) * c

  tmp(  8)   = 1.0e0/ab(  8,  8)
  c           = ab(  7,  8) * tmp(   8)
  ab(  7, 20) = ab(  7, 20) - ab(  8, 20) * c
  ab(  7,  3) = ab(  7,  3) - ab(  8,  3) * c
  ab(  7,  4) = ab(  7,  4) - ab(  8,  4) * c
  ab(  7,  6) = ab(  7,  6) - ab(  8,  6) * c
  ab(  7,  7) = ab(  7,  7) - ab(  8,  7) * c
  c           = ab(  3,  8) * tmp(   8)
  ab(  3, 20) = ab(  3, 20) - ab(  8, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab(  8,  3) * c
  ab(  3,  4) = ab(  3,  4) - ab(  8,  4) * c
  ab(  3,  6) = ab(  3,  6) - ab(  8,  6) * c
  ab(  3,  7) = ab(  3,  7) - ab(  8,  7) * c

  tmp(  7)   = 1.0e0/ab(  7,  7)
  c           = ab(  6,  7) * tmp(   7)
  ab(  6, 20) = ab(  6, 20) - ab(  7, 20) * c
  ab(  6,  3) = ab(  6,  3) - ab(  7,  3) * c
  ab(  6,  4) = ab(  6,  4) - ab(  7,  4) * c
  ab(  6,  5) = ab(  6,  5) - ab(  7,  5) * c
  ab(  6,  6) = ab(  6,  6) - ab(  7,  6) * c
  c           = ab(  3,  7) * tmp(   7)
  ab(  3, 20) = ab(  3, 20) - ab(  7, 20) * c
  ab(  3,  3) = ab(  3,  3) - ab(  7,  3) * c
  ab(  3,  4) = ab(  3,  4) - ab(  7,  4) * c
  ab(  3,  5) = ab(  3,  5) - ab(  7,  5) * c
  ab(  3,  6) = ab(  3,  6) - ab(  7,  6) * c

  tmp(  6)   = 1.0e0/ab(  6,  6)
  c           = ab(  5,  6) * tmp(   6)
  ab(  5, 20) = ab(  5, 20) - ab(  6, 20) * c
  ab(  5,  1) = ab(  5,  1) - ab(  6,  1) * c
  ab(  5,  3) = ab(  5,  3) - ab(  6,  3) * c
  ab(  5,  4) = ab(  5,  4) - ab(  6,  4) * c
  ab(  5,  5) = ab(  5,  5) - ab(  6,  5) * c
  c           = ab(  4,  6) * tmp(   6)
  ab(  4, 20) = ab(  4, 20) - ab(  6, 20) * c
  ab(  4,  1) = ab(  4,  1) - ab(  6,  1) * c
  ab(  4,  3) = ab(  4,  3) - ab(  6,  3) * c
  ab(  4,  4) = ab(  4,  4) - ab(  6,  4) * c
  ab(  4,  5) = ab(  4,  5) - ab(  6,  5) * c
  c           = ab(  3,  6) * tmp(   6)
  ab(  3, 20) = ab(  3, 20) - ab(  6, 20) * c
  ab(  3,  1) = ab(  3,  1) - ab(  6,  1) * c
  ab(  3,  3) = ab(  3,  3) - ab(  6,  3) * c
  ab(  3,  4) = ab(  3,  4) - ab(  6,  4) * c
  ab(  3,  5) = ab(  3,  5) - ab(  6,  5) * c
  c           = ab(  1,  6) * tmp(   6)
  ab(  1, 20) = ab(  1, 20) - ab(  6, 20) * c
  ab(  1,  1) = ab(  1,  1) - ab(  6,  1) * c
  ab(  1,  3) = ab(  1,  3) - ab(  6,  3) * c
  ab(  1,  4) = ab(  1,  4) - ab(  6,  4) * c
  ab(  1,  5) = ab(  1,  5) - ab(  6,  5) * c

  tmp(  5)   = 1.0e0/ab(  5,  5)
  c           = ab(  4,  5) * tmp(   5)
  ab(  4, 20) = ab(  4, 20) - ab(  5, 20) * c
  ab(  4,  1) = ab(  4,  1) - ab(  5,  1) * c
  ab(  4,  3) = ab(  4,  3) - ab(  5,  3) * c
  ab(  4,  4) = ab(  4,  4) - ab(  5,  4) * c
  c           = ab(  3,  5) * tmp(   5)
  ab(  3, 20) = ab(  3, 20) - ab(  5, 20) * c
  ab(  3,  1) = ab(  3,  1) - ab(  5,  1) * c
  ab(  3,  3) = ab(  3,  3) - ab(  5,  3) * c
  ab(  3,  4) = ab(  3,  4) - ab(  5,  4) * c
  c           = ab(  1,  5) * tmp(   5)
  ab(  1, 20) = ab(  1, 20) - ab(  5, 20) * c
  ab(  1,  1) = ab(  1,  1) - ab(  5,  1) * c
  ab(  1,  3) = ab(  1,  3) - ab(  5,  3) * c
  ab(  1,  4) = ab(  1,  4) - ab(  5,  4) * c

  tmp(  4)   = 1.0e0/ab(  4,  4)
  c           = ab(  3,  4) * tmp(   4)
  ab(  3, 20) = ab(  3, 20) - ab(  4, 20) * c
  ab(  3,  1) = ab(  3,  1) - ab(  4,  1) * c
  ab(  3,  3) = ab(  3,  3) - ab(  4,  3) * c
  c           = ab(  1,  4) * tmp(   4)
  ab(  1, 20) = ab(  1, 20) - ab(  4, 20) * c
  ab(  1,  1) = ab(  1,  1) - ab(  4,  1) * c
  ab(  1,  3) = ab(  1,  3) - ab(  4,  3) * c

  tmp(  3)   = 1.0e0/ab(  3,  3)
  c           = ab(  2,  3) * tmp(   3)
  ab(  2, 20) = ab(  2, 20) - ab(  3, 20) * c
  ab(  2,  1) = ab(  2,  1) - ab(  3,  1) * c
  ab(  2,  2) = ab(  2,  2) - ab(  3,  2) * c
  c           = ab(  1,  3) * tmp(   3)
  ab(  1, 20) = ab(  1, 20) - ab(  3, 20) * c
  ab(  1,  1) = ab(  1,  1) - ab(  3,  1) * c
  ab(  1,  2) = ab(  1,  2) - ab(  3,  2) * c

  tmp(  2)   = 1.0e0/ab(  2,  2)
  c           = ab(  1,  2) * tmp(   2)
  ab(  1, 20) = ab(  1, 20) - ab(  2, 20) * c
  ab(  1,  1) = ab(  1,  1) - ab(  2,  1) * c


  ! backsubstitution
  tmp(  1)     = 1.0e0/ab(  1,  1)
  ab(  1, 20) =   ab(  1, 20) * tmp(   1)
  ab(  2, 20) = ( ab(  2, 20) & 
       &                  - ab(  2,  1) * ab(  1, 20) & 
       &                                              ) *tmp(   2)
  ab(  3, 20) = ( ab(  3, 20) & 
       &                  - ab(  3,  1) * ab(  1, 20) & 
       &                  - ab(  3,  2) * ab(  2, 20) & 
       &                                              ) *tmp(   3)
  ab(  4, 20) = ( ab(  4, 20) & 
       &                  - ab(  4,  1) * ab(  1, 20) & 
       &                  - ab(  4,  3) * ab(  3, 20) & 
       &                                              ) *tmp(   4)
  ab(  5, 20) = ( ab(  5, 20) & 
       &                  - ab(  5,  1) * ab(  1, 20) & 
       &                  - ab(  5,  3) * ab(  3, 20) & 
       &                  - ab(  5,  4) * ab(  4, 20) & 
       &                                              ) *tmp(   5)
  ab(  6, 20) = ( ab(  6, 20) & 
       &                  - ab(  6,  1) * ab(  1, 20) & 
       &                  - ab(  6,  3) * ab(  3, 20) & 
       &                  - ab(  6,  4) * ab(  4, 20) & 
       &                  - ab(  6,  5) * ab(  5, 20) & 
       &                                              ) *tmp(   6)
  ab(  7, 20) = ( ab(  7, 20) & 
       &                  - ab(  7,  3) * ab(  3, 20) & 
       &                  - ab(  7,  4) * ab(  4, 20) & 
       &                  - ab(  7,  5) * ab(  5, 20) & 
       &                  - ab(  7,  6) * ab(  6, 20) & 
       &                                              ) *tmp(   7)
  ab(  8, 20) = ( ab(  8, 20) & 
       &                  - ab(  8,  3) * ab(  3, 20) & 
       &                  - ab(  8,  4) * ab(  4, 20) & 
       &                  - ab(  8,  6) * ab(  6, 20) & 
       &                  - ab(  8,  7) * ab(  7, 20) & 
       &                                              ) *tmp(   8)
  ab(  9, 20) = ( ab(  9, 20) & 
       &                  - ab(  9,  3) * ab(  3, 20) & 
       &                  - ab(  9,  4) * ab(  4, 20) & 
       &                  - ab(  9,  6) * ab(  6, 20) & 
       &                  - ab(  9,  8) * ab(  8, 20) & 
       &                                              ) *tmp(   9)
  ab( 10, 20) = ( ab( 10, 20) & 
       &                  - ab( 10,  3) * ab(  3, 20) & 
       &                  - ab( 10,  6) * ab(  6, 20) & 
       &                  - ab( 10,  9) * ab(  9, 20) & 
       &                                              ) *tmp(  10)
  ab( 11, 20) = ( ab( 11, 20) & 
       &                  - ab( 11,  3) * ab(  3, 20) & 
       &                  - ab( 11, 10) * ab( 10, 20) & 
       &                                              ) *tmp(  11)
  ab( 12, 20) = ( ab( 12, 20) & 
       &                  - ab( 12,  3) * ab(  3, 20) & 
       &                  - ab( 12, 11) * ab( 11, 20) & 
       &                                              ) *tmp(  12)
  ab( 13, 20) = ( ab( 13, 20) & 
       &                  - ab( 13,  3) * ab(  3, 20) & 
       &                  - ab( 13, 12) * ab( 12, 20) & 
       &                                              ) *tmp(  13)
  ab( 14, 20) = ( ab( 14, 20) & 
       &                  - ab( 14,  3) * ab(  3, 20) & 
       &                  - ab( 14, 13) * ab( 13, 20) & 
       &                                              ) *tmp(  14)
  ab( 15, 20) = ( ab( 15, 20) & 
       &                  - ab( 15,  3) * ab(  3, 20) & 
       &                  - ab( 15, 14) * ab( 14, 20) & 
       &                                              ) *tmp(  15)
  ab( 16, 20) = ( ab( 16, 20) & 
       &                  - ab( 16,  3) * ab(  3, 20) & 
       &                  - ab( 16, 15) * ab( 15, 20) & 
       &                                              ) *tmp(  16)
  ab( 17, 20) = ( ab( 17, 20) & 
       &                  - ab( 17,  3) * ab(  3, 20) & 
       &                  - ab( 17, 15) * ab( 15, 20) & 
       &                  - ab( 17, 16) * ab( 16, 20) & 
       &                                              ) *tmp(  17)
  ab( 18, 20) = ( ab( 18, 20) & 
       &                  - ab( 18,  3) * ab(  3, 20) & 
       &                  - ab( 18, 15) * ab( 15, 20) & 
       &                  - ab( 18, 16) * ab( 16, 20) & 
       &                  - ab( 18, 17) * ab( 17, 20) & 
       &                                              ) *tmp(  18)
  ab( 19, 20) = ( ab( 19, 20) & 
       &                  - ab( 19,  3) * ab(  3, 20) & 
       &                  - ab( 19, 15) * ab( 15, 20) & 
       &                  - ab( 19, 16) * ab( 16, 20) & 
       &                  - ab( 19, 17) * ab( 17, 20) & 
       &                  - ab( 19, 18) * ab( 18, 20) & 
       &                                              ) *tmp(  19)
  return
end subroutine bn_gift


