!!****if* source/physics/sourceTerms/Burn/BurnMain/nuclearBurn/Iso7/bn_gift
!!
!! NAME
!!  
!!  bn_gift
!!
!!
!! SYNOPSIS
!! 
!!  call bn_gift(real(INOUT)     :: ab, 
!!               integer(IN)     :: n1, 
!!               integer(IN)     :: n2)
!!  
!! DESCRIPTION
!!
!!  Inverts (??) sparse matrix ab using a hard-wired  
!!  routine generated by the GIFT package.
!!  Called by the network integration routines bn_netIntRosen and bn_netIntBader
!!
!! ARGUMENTS
!!
!!  ab:  matrix
!!  n1:  dimension of ab
!!  n2:  dimension of ab
!!
!! NOTES
!!
!!   this version was called gsub_iso7_100,
!!    renamed to the generic "bn_gift"
!!
!!   the FLASH center no longer has access to the GIFT
!!   package, so these types of routines can no longer
!!   be generated
!!
!!***

subroutine bn_gift(ab,n1,n2)
  implicit none
  integer, INTENT(in) :: n1,n2
  real, INTENT(inout) :: ab(n1,n2)
  real c,tmp(  7)

  tmp(  7)   = 1.0e0/ab(  7,  7)
  c           = ab(  6,  7) * tmp(   7)
  ab(  6,  8) = ab(  6,  8) - ab(  7,  8) * c
  ab(  6,  1) = ab(  6,  1) - ab(  7,  1) * c
  ab(  6,  6) = ab(  6,  6) - ab(  7,  6) * c
  c           = ab(  1,  7) * tmp(   7)
  ab(  1,  8) = ab(  1,  8) - ab(  7,  8) * c
  ab(  1,  1) = ab(  1,  1) - ab(  7,  1) * c
  ab(  1,  6) = ab(  1,  6) - ab(  7,  6) * c

  tmp(  6)   = 1.0e0/ab(  6,  6)
  c           = ab(  5,  6) * tmp(   6)
  ab(  5,  8) = ab(  5,  8) - ab(  6,  8) * c
  ab(  5,  1) = ab(  5,  1) - ab(  6,  1) * c
  ab(  5,  2) = ab(  5,  2) - ab(  6,  2) * c
  ab(  5,  3) = ab(  5,  3) - ab(  6,  3) * c
  ab(  5,  5) = ab(  5,  5) - ab(  6,  5) * c
  c           = ab(  1,  6) * tmp(   6)
  ab(  1,  8) = ab(  1,  8) - ab(  6,  8) * c
  ab(  1,  1) = ab(  1,  1) - ab(  6,  1) * c
  ab(  1,  2) = ab(  1,  2) - ab(  6,  2) * c
  ab(  1,  3) = ab(  1,  3) - ab(  6,  3) * c
  ab(  1,  5) = ab(  1,  5) - ab(  6,  5) * c

  tmp(  5)   = 1.0e0/ab(  5,  5)
  c           = ab(  4,  5) * tmp(   5)
  ab(  4,  8) = ab(  4,  8) - ab(  5,  8) * c
  ab(  4,  1) = ab(  4,  1) - ab(  5,  1) * c
  ab(  4,  2) = ab(  4,  2) - ab(  5,  2) * c
  ab(  4,  3) = ab(  4,  3) - ab(  5,  3) * c
  ab(  4,  4) = ab(  4,  4) - ab(  5,  4) * c
  c           = ab(  1,  5) * tmp(   5)
  ab(  1,  8) = ab(  1,  8) - ab(  5,  8) * c
  ab(  1,  1) = ab(  1,  1) - ab(  5,  1) * c
  ab(  1,  2) = ab(  1,  2) - ab(  5,  2) * c
  ab(  1,  3) = ab(  1,  3) - ab(  5,  3) * c
  ab(  1,  4) = ab(  1,  4) - ab(  5,  4) * c

  tmp(  4)   = 1.0e0/ab(  4,  4)
  c           = ab(  3,  4) * tmp(   4)
  ab(  3,  8) = ab(  3,  8) - ab(  4,  8) * c
  ab(  3,  1) = ab(  3,  1) - ab(  4,  1) * c
  ab(  3,  2) = ab(  3,  2) - ab(  4,  2) * c
  ab(  3,  3) = ab(  3,  3) - ab(  4,  3) * c
  c           = ab(  1,  4) * tmp(   4)
  ab(  1,  8) = ab(  1,  8) - ab(  4,  8) * c
  ab(  1,  1) = ab(  1,  1) - ab(  4,  1) * c
  ab(  1,  2) = ab(  1,  2) - ab(  4,  2) * c
  ab(  1,  3) = ab(  1,  3) - ab(  4,  3) * c

  tmp(  3)   = 1.0e0/ab(  3,  3)
  c           = ab(  2,  3) * tmp(   3)
  ab(  2,  8) = ab(  2,  8) - ab(  3,  8) * c
  ab(  2,  1) = ab(  2,  1) - ab(  3,  1) * c
  ab(  2,  2) = ab(  2,  2) - ab(  3,  2) * c
  c           = ab(  1,  3) * tmp(   3)
  ab(  1,  8) = ab(  1,  8) - ab(  3,  8) * c
  ab(  1,  1) = ab(  1,  1) - ab(  3,  1) * c
  ab(  1,  2) = ab(  1,  2) - ab(  3,  2) * c

  tmp(  2)   = 1.0e0/ab(  2,  2)
  c           = ab(  1,  2) * tmp(   2)
  ab(  1,  8) = ab(  1,  8) - ab(  2,  8) * c
  ab(  1,  1) = ab(  1,  1) - ab(  2,  1) * c


  ! backsubstitution
  tmp(  1)     = 1.0e0/ab(  1,  1)
  ab(  1,  8) =   ab(  1,  8) * tmp(   1)
  ab(  2,  8) = ( ab(  2,  8) & 
       &                  - ab(  2,  1) * ab(  1,  8) & 
       &                                              ) *tmp(   2)
  ab(  3,  8) = ( ab(  3,  8) & 
       &                  - ab(  3,  1) * ab(  1,  8) & 
       &                  - ab(  3,  2) * ab(  2,  8) & 
       &                                              ) *tmp(   3)
  ab(  4,  8) = ( ab(  4,  8) & 
       &                  - ab(  4,  1) * ab(  1,  8) & 
       &                  - ab(  4,  2) * ab(  2,  8) & 
       &                  - ab(  4,  3) * ab(  3,  8) & 
       &                                              ) *tmp(   4)
  ab(  5,  8) = ( ab(  5,  8) & 
       &                  - ab(  5,  1) * ab(  1,  8) & 
       &                  - ab(  5,  2) * ab(  2,  8) & 
       &                  - ab(  5,  3) * ab(  3,  8) & 
       &                  - ab(  5,  4) * ab(  4,  8) & 
       &                                              ) *tmp(   5)
  ab(  6,  8) = ( ab(  6,  8) & 
       &                  - ab(  6,  1) * ab(  1,  8) & 
       &                  - ab(  6,  2) * ab(  2,  8) & 
       &                  - ab(  6,  3) * ab(  3,  8) & 
       &                  - ab(  6,  5) * ab(  5,  8) & 
       &                                              ) *tmp(   6)
  ab(  7,  8) = ( ab(  7,  8) & 
       &                  - ab(  7,  1) * ab(  1,  8) & 
       &                  - ab(  7,  6) * ab(  6,  8) & 
       &                                              ) *tmp(   7)
  return
end subroutine bn_gift


