!!****if* source/physics/sourceTerms/PrimordialChemistry/PrimordialChemistryMain/GA08/pchem_gift
!!
!! NAME
!!  
!!  pchem_gift
!!
!!
!! SYNOPSIS
!! 
!!  call pchem_gift(real(INOUT)     :: ab, 
!!               integer(IN)     :: n1, 
!!               integer(IN)     :: n2)
!!  
!! DESCRIPTION
!!
!!  Inverts (??) sparse matrix ab using a hard-wired  
!!  routine generated by the GIFT package.
!!  Called by the network integration routines pchem_netIntRosen and pchem_netIntBader
!!
!! ARGUMENTS
!!
!!  ab:  matrix
!!  n1:  dimension of ab
!!  n2:  dimension of ab
!!
!! NOTES
!!
!!   this version was called gsub_aprox13_100,
!!    renamed to the generic "pchem_gift"
!!
!!   the FLASH center no longer has access to the GIFT
!!   package, so these types of routines can no longer
!!   be generated
!!
!!***


subroutine pchem_gift(ab,n1,n2)
  implicit none
  integer, INTENT(in) :: n1,n2
  real, INTENT(inout) :: ab(n1,n2)
  real c,tmp( 13)

  tmp( 13)   = 1.0e0/ab( 13, 13)
  c           = ab( 12, 13) * tmp(  13)
  ab( 12, 14) = ab( 12, 14) - ab( 13, 14) * c
  ab( 12,  1) = ab( 12,  1) - ab( 13,  1) * c
  ab( 12, 12) = ab( 12, 12) - ab( 13, 12) * c
  c           = ab(  1, 13) * tmp(  13)
  ab(  1, 14) = ab(  1, 14) - ab( 13, 14) * c
  ab(  1,  1) = ab(  1,  1) - ab( 13,  1) * c
  ab(  1, 12) = ab(  1, 12) - ab( 13, 12) * c

  tmp( 12)   = 1.0e0/ab( 12, 12)
  c           = ab( 11, 12) * tmp(  12)
  ab( 11, 14) = ab( 11, 14) - ab( 12, 14) * c
  ab( 11,  1) = ab( 11,  1) - ab( 12,  1) * c
  ab( 11, 11) = ab( 11, 11) - ab( 12, 11) * c
  c           = ab(  1, 12) * tmp(  12)
  ab(  1, 14) = ab(  1, 14) - ab( 12, 14) * c
  ab(  1,  1) = ab(  1,  1) - ab( 12,  1) * c
  ab(  1, 11) = ab(  1, 11) - ab( 12, 11) * c

  tmp( 11)   = 1.0e0/ab( 11, 11)
  c           = ab( 10, 11) * tmp(  11)
  ab( 10, 14) = ab( 10, 14) - ab( 11, 14) * c
  ab( 10,  1) = ab( 10,  1) - ab( 11,  1) * c
  ab( 10, 10) = ab( 10, 10) - ab( 11, 10) * c
  c           = ab(  1, 11) * tmp(  11)
  ab(  1, 14) = ab(  1, 14) - ab( 11, 14) * c
  ab(  1,  1) = ab(  1,  1) - ab( 11,  1) * c
  ab(  1, 10) = ab(  1, 10) - ab( 11, 10) * c

  tmp( 10)   = 1.0e0/ab( 10, 10)
  c           = ab(  9, 10) * tmp(  10)
  ab(  9, 14) = ab(  9, 14) - ab( 10, 14) * c
  ab(  9,  1) = ab(  9,  1) - ab( 10,  1) * c
  ab(  9,  9) = ab(  9,  9) - ab( 10,  9) * c
  c           = ab(  1, 10) * tmp(  10)
  ab(  1, 14) = ab(  1, 14) - ab( 10, 14) * c
  ab(  1,  1) = ab(  1,  1) - ab( 10,  1) * c
  ab(  1,  9) = ab(  1,  9) - ab( 10,  9) * c

  tmp(  9)   = 1.0e0/ab(  9,  9)
  c           = ab(  8,  9) * tmp(   9)
  ab(  8, 14) = ab(  8, 14) - ab(  9, 14) * c
  ab( 8, 1) = ab( 8, 1) - ab( 9, 1) * c
  ab( 8, 8) = ab( 8, 8) - ab( 9, 8) * c
  c = ab( 1, 9) * tmp( 9)
  ab( 1, 14) = ab( 1, 14) - ab( 9, 14) * c
  ab( 1, 1) = ab( 1, 1) - ab( 9, 1) * c
  ab( 1, 8) = ab( 1, 8) - ab( 9, 8) * c

  tmp( 8) = 1.0e0/ab( 8, 8)
  c = ab( 7, 8) * tmp( 8)
  ab( 7, 14) = ab( 7, 14) - ab( 8, 14) * c
  ab( 7, 1) = ab( 7, 1) - ab( 8, 1) * c
  ab( 7, 7) = ab( 7, 7) - ab( 8, 7) * c
  c = ab( 1, 8) * tmp( 8)
  ab( 1, 14) = ab( 1, 14) - ab( 8, 14) * c
  ab( 1, 1) = ab( 1, 1) - ab( 8, 1) * c
  ab( 1, 7) = ab( 1, 7) - ab( 8, 7) * c

  tmp( 7) = 1.0e0/ab( 7, 7)
  c = ab( 6, 7) * tmp( 7)
  ab( 6, 14) = ab( 6, 14) - ab( 7, 14) * c
  ab( 6, 1) = ab( 6, 1) - ab( 7, 1) * c
  ab( 6, 3) = ab( 6, 3) - ab( 7, 3) * c
  ab( 6, 6) = ab( 6, 6) - ab( 7, 6) * c
  c = ab( 1, 7) * tmp( 7)
  ab( 1, 14) = ab( 1, 14) - ab( 7, 14) * c
  ab( 1, 1) = ab( 1, 1) - ab( 7, 1) * c
  ab( 1, 3) = ab( 1, 3) - ab( 7, 3) * c
  ab( 1, 6) = ab( 1, 6) - ab( 7, 6) * c

  tmp( 6) = 1.0e0/ab( 6, 6)
  c = ab( 5, 6) * tmp( 6)
  ab( 5, 14) = ab( 5, 14) - ab( 6, 14) * c
  ab( 5, 1) = ab( 5, 1) - ab( 6, 1) * c
  ab( 5, 2) = ab( 5, 2) - ab( 6, 2) * c
  ab( 5, 3) = ab( 5, 3) - ab( 6, 3) * c
  ab( 5, 5) = ab( 5, 5) - ab( 6, 5) * c
  c = ab( 1, 6) * tmp( 6)
  ab( 1, 14) = ab( 1, 14) - ab( 6, 14) * c
  ab( 1, 1) = ab( 1, 1) - ab( 6, 1) * c
  ab( 1, 2) = ab( 1, 2) - ab( 6, 2) * c
  ab( 1, 3) = ab( 1, 3) - ab( 6, 3) * c
  ab( 1, 5) = ab( 1, 5) - ab( 6, 5) * c

  tmp( 5) = 1.0e0/ab( 5, 5)
  c = ab( 4, 5) * tmp( 5)
  ab( 4, 14) = ab( 4, 14) - ab( 5, 14) * c
  ab( 4, 1) = ab( 4, 1) - ab( 5, 1) * c
  ab( 4, 2) = ab( 4, 2) - ab( 5, 2) * c
  ab( 4, 3) = ab( 4, 3) - ab( 5, 3) * c
  ab( 4, 4) = ab( 4, 4) - ab( 5, 4) * c
  c = ab( 1, 5) * tmp( 5)
  ab( 1, 14) = ab( 1, 14) - ab( 5, 14) * c
  ab( 1, 1) = ab( 1, 1) - ab( 5, 1) * c
  ab( 1, 2) = ab( 1, 2) - ab( 5, 2) * c
  ab( 1, 3) = ab( 1, 3) - ab( 5, 3) * c
  ab( 1, 4) = ab( 1, 4) - ab( 5, 4) * c

  tmp( 4) = 1.0e0/ab( 4, 4)
  c = ab( 3, 4) * tmp( 4)
  ab( 3, 14) = ab( 3, 14) - ab( 4, 14) * c
  ab( 3, 1) = ab( 3, 1) - ab( 4, 1) * c
  ab( 3, 2) = ab( 3, 2) - ab( 4, 2) * c
  ab( 3, 3) = ab( 3, 3) - ab( 4, 3) * c
  c = ab( 1, 4) * tmp( 4)
  ab( 1, 14) = ab( 1, 14) - ab( 4, 14) * c
  ab( 1, 1) = ab( 1, 1) - ab( 4, 1) * c
  ab( 1, 2) = ab( 1, 2) - ab( 4, 2) * c
  ab( 1, 3) = ab( 1, 3) - ab( 4, 3) * c

  tmp( 3) = 1.0e0/ab( 3, 3)
  c = ab( 2, 3) * tmp( 3)
  ab( 2, 14) = ab( 2, 14) - ab( 3, 14) * c
  ab( 2, 1) = ab( 2, 1) - ab( 3, 1) * c
  ab( 2, 2) = ab( 2, 2) - ab( 3, 2) * c
  c = ab( 1, 3) * tmp( 3)
  ab( 1, 14) = ab( 1, 14) - ab( 3, 14) * c
  ab( 1, 1) = ab( 1, 1) - ab( 3, 1) * c
  ab( 1, 2) = ab( 1, 2) - ab( 3, 2) * c

  tmp( 2) = 1.0e0/ab( 2, 2)
  c = ab( 1, 2) * tmp( 2)
  ab( 1, 14) = ab( 1, 14) - ab( 2, 14) * c
  ab( 1, 1) = ab( 1, 1) - ab( 2, 1) * c


  ! backsubstitution
  tmp( 1) = 1.0e0/ab( 1, 1)
  ab( 1, 14) = ab( 1, 14) * tmp( 1)
  ab( 2, 14) = ( ab( 2, 14) &
       & - ab( 2, 1) * ab( 1, 14) &
       & ) *tmp( 2)
  ab( 3, 14) = ( ab( 3, 14) &
       & - ab( 3, 1) * ab( 1, 14) &
       & - ab( 3, 2) * ab( 2, 14) &
       & ) *tmp( 3)
  ab( 4, 14) = ( ab( 4, 14) &
       & - ab( 4, 1) * ab( 1, 14) &
       & - ab( 4, 2) * ab( 2, 14) &
       & - ab( 4, 3) * ab( 3, 14) &
       & ) *tmp( 4)
  ab( 5, 14) = ( ab( 5, 14) &
       & - ab( 5, 1) * ab( 1, 14) &
       & - ab( 5, 2) * ab( 2, 14) &
       & - ab( 5, 3) * ab( 3, 14) &
       & - ab( 5, 4) * ab( 4, 14) &
       & ) *tmp( 5)
  ab( 6, 14) = ( ab( 6, 14) &
       & - ab( 6, 1) * ab( 1, 14) &
       & - ab( 6, 2) * ab( 2, 14) &
       & - ab( 6, 3) * ab( 3, 14) &
       & - ab( 6, 5) * ab( 5, 14) &
       & ) *tmp( 6)
  ab( 7, 14) = ( ab( 7, 14) &
       & - ab( 7, 1) * ab( 1, 14) &
       & - ab( 7, 3) * ab( 3, 14) &
       & - ab( 7, 6) * ab( 6, 14) &
       & ) *tmp( 7)
  ab( 8, 14) = ( ab( 8, 14) &
       & - ab( 8, 1) * ab( 1, 14) &
       & - ab( 8, 7) * ab( 7, 14) &
       & ) *tmp( 8)
  ab( 9, 14) = ( ab( 9, 14) &
       & - ab( 9, 1) * ab( 1, 14) &
       & - ab( 9, 8) * ab( 8, 14) &
       & ) *tmp( 9)
  ab( 10, 14) = ( ab( 10, 14) &
       & - ab( 10, 1) * ab( 1, 14) &
       & - ab( 10, 9) * ab( 9, 14) &
       & ) *tmp( 10)
  ab( 11, 14) = ( ab( 11, 14) &
       & - ab( 11, 1) * ab( 1, 14) &
       & - ab( 11, 10) * ab( 10, 14) &
       & ) *tmp( 11)
  ab( 12, 14) = ( ab( 12, 14) &
       & - ab( 12, 1) * ab( 1, 14) &
       & - ab( 12, 11) * ab( 11, 14) &
       & ) *tmp( 12)
  ab( 13, 14) = ( ab( 13, 14) &
       & - ab( 13, 1) * ab( 1, 14) &
       & - ab( 13, 12) * ab( 12, 14) &
       & ) *tmp( 13)
  return
end subroutine pchem_gift

