!!****if* source/physics/sourceTerms/PrimordialChemistry/PrimordialChemistryMain/GA08/pchem_mapNetworkToSpecies
!!  NAME
!!
!!  pchem_mapNetworkToSpecies
!!
!!  SYNOPSIS
!!
!!   call pchem_mapNetworkToSpecies(integer, intent(IN)  :: networkIN,
!!                                 integer, intent(OUT) :: specieOut)
!!
!!  DESCRIPTION
!!
!!   Maps network indicies required by chemistry to species generated by Flash.h
!!
!!  ARGUMENTS
!!
!!  networkIn --> integer index of the network as defined in GlobChem reoutines
!!  specieOut --> integer corresponding to the same specie in Flash.h
!!
!!****

subroutine pchem_mapNetworkToSpecies(networkIn, specieOut)

   use Driver_interface, ONLY: Driver_abortFlash
   use Logfile_interface, ONLY: Logfile_stampMessage

   implicit none

#include "Flash.h"
  
   integer, intent(IN)  :: networkIN
   integer, intent(OUT) :: specieOut

   !! Local variable for error diagnostics
   character*256  :: internalFile

   select case (networkIn)
   case (1)
      specieOut = H_SPEC
   case (2)
      specieOut = HP_SPEC
   case (3)
      specieOut = HM_SPEC
   case (4) 
      specieOut = D_SPEC
   case (5)
      specieOut = DP_SPEC
   case (6)
      specieOut = DM_SPEC
   case (7)
      specieOut = HE_SPEC
   case (8)
      specieOut = HEPP_SPEC
   case (9)
      specieOut = HEP_SPEC
   case (10)
      specieOut = H2P_SPEC
   case (11)
      specieOut = H2_SPEC
   case (12)
      specieOut = HDP_SPEC
   case (13)
      specieOut = HD_SPEC
   case(14)
      specieOut = D2_SPEC
   case(15)
      specieOut = D2P_SPEC
   case (16)
      specieOut = ELEC_SPEC
   case default
      !!Abort
      call Logfile_stampMessage('ERROR in pchem_mapNetworkToSpecies')
      write(internalFile, *) '[pchem_mapNetworkToSpecies] Index',networkIn,' cannot be mapped to a Flash.h species'
      call Logfile_stampMessage(internalFile)
      call Driver_abortFlash(internalFile)
   end select


   return
end subroutine pchem_mapNetworkToSpecies
